/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : CMT_ex1.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : main processing source file of cmtw & elc fit module.
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Functions for using CMTW & ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdio.h>
#include "cmsis_os.h"
#include "Serial.h"
#include "r_adc_if.h"
#include "r_cmt_if.h"
#include "r_elc_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define CMTW_MATCH_COUNT_3SEC   (28125000U)

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
int32_t         main(void);

static int32_t  sample_setup(void);
static void     sample_process(void);
static void     sample_release(void);

static void     cmtw_ch0_inhr_callback(void);
static void     adc_inhr_callback(int32_t event);

bool_t gb_cmtw_end_flag = false;
bool_t gb_adc_end_flag  = false;
bool_t main_end_flag    = false;

int32_t main( void )
{
    int32_t     ret;

    /* Initialize Serial */
    SER_Init();

    /* Initialize ADC */
    R_ADC_Init();

    /* Initialize ELC */
    R_ELC_Init();

    /* Initialize CMT */
    R_CMT_Init();

    /* Setup */
    ret = sample_setup();

    if (ret == ESUCCESS)
    {
        sample_process();
    }

    sample_release();

    while (1) {                         /* endless loop                        */
        osDelay(5);                     /* wait for timeout: 5ms               */
    }

    return ret;
}

/*******************************************************************************
* Function Name: sample_setup
* Description  : setup every driver to sample program.
* Arguments    : None.
* Return Value : true -
*                   setup successful
*                false -
*                   setup fail
*******************************************************************************/
static int32_t sample_setup(void)
{
    int32_t         ret;
    cmt_cfg_t       cmt_config;
    elc_cmd_s12ad_t elc_s12ad_config;
    adc_cfg_t       adc_config;
    adc_ch_cfg_t    adc_ch_config;

    memset((void *)&cmt_config, 0, sizeof(cmt_config));
    memset((void *)&elc_s12ad_config, 0, sizeof(elc_s12ad_config));
    memset((void *)&adc_config, 0, sizeof(adc_config));
    memset((void *)&adc_ch_config, 0, sizeof(adc_ch_config));

    /* ADC driver setting */
    /* ADC driver initialize */
    adc_config.trigger   = ADC_TRIG_ELCTRG0; /* The default setting */
    adc_config.priority  = VIC_PRIORITY_15;
    adc_config.add_cnt   = ADC_ADD_OFF;
    adc_config.alignment = ADC_ALIGN_RIGHT;
    adc_config.clearing  = ADC_CLEAR_AFTER_READ_ON;
    ret = R_ADC_Open(ADC_MODE_SS_ONE_CH, &adc_config, &adc_inhr_callback);

    if (ESUCCESS == ret)
    {
        /* A/D conversion channel setting */
        /* Select AN007(Potentiometer)    */
        adc_ch_config.chan_mask = ADC_MASK_CH7;
        ret = R_ADC_Control(ADC_CMD_ENABLE_CHANS, &adc_ch_config);
        if (ESUCCESS == ret)
        {
            /* Enable of the A/D conversion end interrupt */
            ret = R_ADC_Control(ADC_CMD_ENABLE_INT, NULL);
            if (ESUCCESS == ret)
            {
                /* Start A/D conversion trigger */
                ret = R_ADC_Control(ADC_CMD_ENABLE_TRIG, NULL);
            }
        }
    }

    if (ESUCCESS == ret)
    {
        /* ELC driver setting */
        ret = R_ELC_Open();

        if (ESUCCESS == ret)
        {
            elc_s12ad_config.elc_s12ad_ch      = ELC_S12AD_CH_0;
            elc_s12ad_config.event_link_enable = true;
            elc_s12ad_config.resource          = ELC_RESOURCE_CMTW0_COMPARE_MATCH;

            ret = R_ELC_Control(ELC_CMD_SET_EVENT_S12AD, &elc_s12ad_config);
        }

        if (ESUCCESS == ret)
        {
            ret = R_ELC_LinkStart();
        }
    }

    if (ESUCCESS == ret)
    {
        /* CMTW driver initialize */
        cmt_config.time_cnt_param.pclk_div     = CMT_PCLK_DIV_8;
        cmt_config.time_cnt_param.cnt_size     = CMTW_CNT_SIZE_32BIT;
        cmt_config.time_cnt_param.clear_factor = CMTW_CNT_CLEAR_COMPARE_MATCH;

        cmt_config.mode_param.compare_match.mode_enable       = true;
        cmt_config.mode_param.compare_match.compare_match_cnt = CMTW_MATCH_COUNT_3SEC;
        cmt_config.mode_param.compare_match.intr_priority     = VIC_PRIORITY_15;
        cmt_config.mode_param.compare_match.p_callback        = &cmtw_ch0_inhr_callback;

        cmt_config.mode_param.input_capture[CMTW_INPUT_CAPTURE_NUM_0].mode_enable   = false;
        cmt_config.mode_param.input_capture[CMTW_INPUT_CAPTURE_NUM_1].mode_enable   = false;
        cmt_config.mode_param.output_compare[CMTW_OUTPUT_COMPARE_NUM_0].mode_enable = false;
        cmt_config.mode_param.output_compare[CMTW_OUTPUT_COMPARE_NUM_1].mode_enable = false;

        ret = R_CMT_Open(CMTW_CHANNEL_0, &cmt_config);

        if (ESUCCESS == ret)
        {
            ret = R_CMT_StartPeriodic(CMTW_CHANNEL_0);
        }
    }

    return ret;
}
/*******************************************************************************
 End of function sample_setup
*******************************************************************************/

/*******************************************************************************
* Function Name: sample_release
* Description  : setup every driver to sample program.
* Arguments    : None.
* Return Value : true -
*                   setup successful
*                false -
*                   setup fail
*******************************************************************************/
void sample_release(void)
{
    R_CMT_Close(CMTW_CHANNEL_0);
    R_CMT_Uninit();

    R_ADC_Close();
    R_ADC_Uninit();

    R_ELC_Close();
    R_ELC_Uninit();
}
/*******************************************************************************
 End of function sample_release
*******************************************************************************/

/*******************************************************************************
* Function Name: sample_process
* Description  : led control according to A/D convert result.
* Arguments    : None.
* Return Value : none.
*******************************************************************************/
static void sample_process(void)
{
    int32_t     ret;
    int32_t     main_end_flag = false;
    uint16_t    adc_data;

    while (false == main_end_flag)
    {
        /* Wait CMTW compare match to complete */
        while (false == gb_cmtw_end_flag)
        {
            /* wait CMTW compare match end */
        }
        gb_cmtw_end_flag = false;

        /* Wait A/D conversion to complete */
        while (false == gb_adc_end_flag)
        {
            /* wait A/D conversion end */
        }
        gb_adc_end_flag = false;

        /* Read A/D conversion */
        ret = R_ADC_Read(ADC_REG_CH7, &adc_data);
        if (ESUCCESS != ret)
        {
            main_end_flag = true;
        }
        else
        {
            /* ADC level check */
            printf("adc result = %x\n", adc_data);
        }
    }
}
/*******************************************************************************
 End of function sample_process
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_ch0_inhr_callback
* Description  : Wait of CMTW compare match is finished, and LED OFF.
* Arguments    : p_args_adc
*                    not used.
* Return Value : None.
*******************************************************************************/
static void cmtw_ch0_inhr_callback(void)
{
    gb_cmtw_end_flag = true;

    return;
}
/*******************************************************************************
 End of function cmtw_ch0_inhr_callback
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_inhr_callback
* Description  : Wait of ADC convert end is finished.
* Arguments    : p_args_adc-
*                    not used.
* Return Value : None.
*******************************************************************************/
static void adc_inhr_callback(int32_t event)
{
    gb_adc_end_flag = true;

    return;
}
/*******************************************************************************
 End of function adc_inhr_callback
*******************************************************************************/

/* End of File */
