/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : MTU_ex1.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Main program for RZ/T1 MTU3a program
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Initialize and operate the MTU3a function of RZ/T1
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdio.h>
#include "cmsis_os.h"
#include "Serial.h"
#include "r_mtu3_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define MTU_COUNT_STOP        (0)
#define MTU_CMPLMT_PWM_START  (0xC0)
#define MTU_C_SET_CYCLE       (0xEA6)   // Carrier cycle = 100us (PCLKC/4)
#define MTU_C_CYCLE           (0x753)   // Carrier cycle/2 = 50us (PCLKC/4)
#define MTU_DEAD_TIME         (0x4B)    // Dead time = 2us (PCLKC/4)

#define MTU_TGRA_CYCLE        (MTU_C_CYCLE + MTU_DEAD_TIME)
#define MTU_DUTY_25           ((uint16_t) MTU_TGRA_CYCLE * 1/4)
#define MTU_DUTY_50           ((uint16_t) MTU_TGRA_CYCLE * 2/4)
#define MTU_DUTY_75           ((uint16_t) MTU_TGRA_CYCLE * 3/4)

#define MTU_ENABLE_OUTPUT     (1)
#define MTU_DISABLE_OUTPUT    (0)
#define MTU_MODE_COMPLT_PWM_1 (0xD)

#define DUTY_CNT_MAX          (3)

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
int     main(void);
int32_t mtu3_setup(void);
void port1_mpu_init(void);

uint16_t g_duty_rate[DUTY_CNT_MAX] = { MTU_DUTY_25, MTU_DUTY_50, MTU_DUTY_75 };
uint8_t  g_duty_cnt;

/*******************************************************************************
* Outline      : main processing
* Function Name: main
* Description  : Initialize the peripheral settings of RZ/T1. And Sets MTU3a to
*                Complementary PWM Mode(3 phases).
* Arguments    : none
* Return Value : none
*******************************************************************************/
int32_t main (void)
{
	int32_t		ret;

    /* Initialize Serial */
    SER_Init();

	/* Initialize PORT1 & MPC */
	port1_mpu_init();
	
	/* Initialize MTU */
    R_MTU_Init();
    ret = mtu3_setup();
    if (ESUCCESS != ret)
    {
    	goto err;
    }

    /* Start MTU3a as complementary PWM output */
    ret = R_MTU_PWM_Compliment_Control(MTU_CHANNEL_3_4, MTU_CMD_START, NULL);

    if (ESUCCESS == ret)
    {
		while (1)
		{
			/* Do nothing */
		}
    }

    R_MTU_PWM_Compliment_Close(MTU_CHANNEL_3_4);

err:

    R_MTU_Uninit();

    while (1) {                         /* endless loop                        */
        osDelay(5);                     /* wait for timeout: 5ms               */
    }

    return ret;
}
/*******************************************************************************
 End of function main
*******************************************************************************/

/*******************************************************************************
* Function Name: port1_mpu_init
* Description  : Initialize PORT1 & MPC setting as following.
*                   PORT1   : set peripheral function
*                   MPC     : set MTIOC3B, MTIOC3D, MTIOC4A, MTIOC4C, MTIOC4B, MTIOC4D function
* Arguments    : none
* Return Value : none
*******************************************************************************/
void port1_mpu_init(void)
{
    /* MTIOC3B(P16=JA2.P13), MTIOC3D(P15=JA2.P14) */
    PORT1.PDR.BIT.B6 = 0x00;
    PORT1.PDR.BIT.B5 = 0x00;
    PORT1.PMR.BIT.B6 = 0x00;
    PORT1.PMR.BIT.B5 = 0x00;
    R_MPC_WriteEnable();
    MPC.P16PFS.BIT.PSEL = 0x01;
    MPC.P15PFS.BIT.PSEL = 0x01;
    R_MPC_WriteDisable();
    PORT1.PMR.BIT.B6 = 0x01;
    PORT1.PMR.BIT.B5 = 0x01;
    
    /* MTIOC4A(P14=JA2.P15), MTIOC4C(P13=JA2.P16) */
    PORT1.PDR.BIT.B4 = 0x00;
    PORT1.PDR.BIT.B3 = 0x00;
    PORT1.PMR.BIT.B4 = 0x00;
    PORT1.PMR.BIT.B3 = 0x00;
    R_MPC_WriteEnable();
    MPC.P14PFS.BIT.PSEL = 0x01;
    MPC.P13PFS.BIT.PSEL = 0x01;
    R_MPC_WriteDisable();
    PORT1.PMR.BIT.B4 = 0x01;
    PORT1.PMR.BIT.B3 = 0x01;

    /* MTIOC4B(P12=JA2.P17), MTIOC4D(P11=JA2.P18) */
    PORT1.PDR.BIT.B2 = 0x00;
    PORT1.PDR.BIT.B1 = 0x00;
    PORT1.PMR.BIT.B2 = 0x00;
    PORT1.PMR.BIT.B1 = 0x00;
    R_MPC_WriteEnable();
    MPC.P12PFS.BIT.PSEL = 0x01;
    MPC.P11PFS.BIT.PSEL = 0x01;
    R_MPC_WriteDisable();
    PORT1.PMR.BIT.B2 = 0x01;
    PORT1.PMR.BIT.B1 = 0x01;
    
	return;
}

/*******************************************************************************
* Function Name: mtu3_setup
* Description  : Initialize MTU3a setting as following.
*                   Channel   : MTU3 and MTU4
*                   Mode      : Complementary PWM mode 1 (Transfer at crest)
*                   Clock     : PCLKC/4 (rising edge)
*                   Cycle     : 100us
*                   Dead time : 2us
*                   Duty      : 25% (Initial value)
* Arguments    : none
* Return Value : ESUCCESS
*               !ESUCCESS
*******************************************************************************/
int32_t mtu3_setup(void)
{
    int32_t     ret;

    mtu_cmpl_pwm_chnl_settings_t  mtu_ch_config;

    memset((void *)&mtu_ch_config, 0, sizeof(mtu_ch_config));

    mtu_ch_config.clock_src.source     = MTU_CLK_SRC_INTERNAL;          // Sets clock to PCLKC/4
    mtu_ch_config.clock_src.clock_edge = MTU_CLK_RISING_EDGE;           // Sets rising edge

    mtu_ch_config.clk_div.clock_div    = MTU_SRC_CLK_DIV_4;
    mtu_ch_config.clk_div.cycle_freq   = MTU_C_SET_CYCLE;               // Sets carrier cycle to 100us

    mtu_ch_config.dead_time            = MTU_DEAD_TIME;                 // Sets deadtime to 2us
    mtu_ch_config.toggle               = MTU_TOGGLE_OFF;                // Disables toggle output
    mtu_ch_config.mode                 = MTU_CMPL_PWM_MODE_1;           // Sets compliment PWM mode 1
    mtu_ch_config.p_n                  = MTU_PIN_P_N_1;                 // Sets output pin level
    mtu_ch_config.p_n_bf               = MTU_PIN_P_N_BF_OFF;            // Sets pin level to fixed setting
    mtu_ch_config.d_bf                 = MTU_CMPL_PWM_D_BF_OFF;
    mtu_ch_config.protect              = MTU_CMPL_PWM_PROTECT_OFF;      // Disables protecting register

    mtu_ch_config.pwm_output_1.olsp    = MTU_PIN_OLSP_LO_UPHI_DNLO;     // Sets output pin level
    mtu_ch_config.pwm_output_1.olsn    = MTU_PIN_OLSN_LO_UPLO_DNHI;
    mtu_ch_config.pwm_output_1.duty    = 250;                           // Sets duty rate to 25%
    mtu_ch_config.pwm_output_1.output  = MTU_CMPL_PWM_OUTPUT_ON;        // Enables PWM output pin
    mtu_ch_config.pwm_output_2.olsp    = MTU_PIN_OLSP_LO_UPHI_DNLO;     // Sets output pin level
    mtu_ch_config.pwm_output_2.olsn    = MTU_PIN_OLSN_LO_UPLO_DNHI;
    mtu_ch_config.pwm_output_2.duty    = 250;                           // Sets duty rate to 25%
    mtu_ch_config.pwm_output_2.output  = MTU_CMPL_PWM_OUTPUT_ON;        // Enables PWM output pin
    mtu_ch_config.pwm_output_3.olsp    = MTU_PIN_OLSP_LO_UPHI_DNLO;     // Sets output pin level
    mtu_ch_config.pwm_output_3.olsn    = MTU_PIN_OLSN_LO_UPLO_DNHI;
    mtu_ch_config.pwm_output_3.duty    = 250;                           // Sets duty rate to 25%
    mtu_ch_config.pwm_output_3.output  = MTU_CMPL_PWM_OUTPUT_ON;        // Enables PWM output pin

    ret = R_MTU_PWM_Compliment_Open(MTU_CHANNEL_3_4, &mtu_ch_config);   // Chanel 3 and 4 is selected

    return ret;
}
/*******************************************************************************
 End of function mtu3_setup
*******************************************************************************/


/* End of File */


