/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : adc driver functions for adc fit module.
* File Name    : r_adc.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for ADC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ADC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_adc.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static adc_err_t adc_enable_chans(adc_ch_cfg_t *p_config);
static void      adc_enable_s12adi0(void);

static void      adc_s12adi0_isr(void);
static void      adc_gbadi_isr(void);

static void      adc_set_adsstr(const adc_time_t * const adc_sst);
static adc_err_t adc_check_trigger(adc_trig_t trig);

/* In RAM */
adc_control_t adc_control =
    {
        ADC_UNINIT,             /* state            */
        ADC_MODE_MAX,           /* mode             */
        false,                  /* complete         */
        false,                  /* complete_groupb  */
        0,                      /* sem_state        */
        NULL                    /* p_callback       */
    };

/* In ROM. A/D Data Register pointers */
volatile static uint16_t * const  pdreg_ptrs[ADC_REG_MAX] =
                      { &S12ADC0.ADDR0,
                        &S12ADC0.ADDR1,
                        &S12ADC0.ADDR2,
                        &S12ADC0.ADDR3,
                        &S12ADC0.ADDR4,
                        &S12ADC0.ADDR5,
                        &S12ADC0.ADDR6,
                        &S12ADC0.ADDR7,
                        &S12ADC0.ADTSDR,      // temperature sensor
                        &S12ADC0.ADDBLDR      // double trigger register
                      };

/*******************************************************************************
* Function Name: ADC_Open
* Description  : This function applies power to the A/D peripheral, sets the
*                operational mode, trigger sources, interrupt priority, and
*                configurations common to all channels and sensors. If interrupt
*                priority is non-zero, the function takes a callback function
*                pointer for notifying the user at interrupt level whenever a
*                scan has completed.
* Arguments    : mode-
*                    Operational mode (see enumeration below)
*                p_cfg-
*                    Pointer to configuration structure (see below)
*                p_callback-
*                    Optional pointer to function called from interrupt when
*                    a scan completes
* Return Value : ADC_SUCCESS-
*                    Successful
*                ADC_ERR_AD_NOT_CLOSED-
*                    Peripheral is still running in another mode; Perform
*                    ADC_Close() first
*                ADC_ERR_INVALID_ARG-
*                    mode or element of p_cfg structure has invalid value.
*                ADC_ERR_ILLEGAL_ARG-
*                    an argument is illegal based upon mode
*                ADC_ERR_MISSING_PTR-
*                    p_cfg pointer is NULL
*******************************************************************************/
adc_err_t ADC_Open(adc_mode_t const  mode,
                   adc_cfg_t * const p_cfg,
                   void     (* const p_callback)(void *p_args))
{
    adc_err_t ret_trig;
    volatile uint32_t dummy32t;
    volatile uint16_t dummy16t;

#if ADC_CFG_PARAM_CHECKING_ENABLE == 1
    if (NULL == p_cfg)
    {
        return ADC_ERR_MISSING_PTR;
    }

    /* Check for valid argument values */
    if ((mode >= ADC_MODE_MAX)
        || (p_cfg->priority > BSP_MCU_IPL_MAX)
        || (p_cfg->priority_groupb > BSP_MCU_IPL_MAX)
        || (p_cfg->add_cnt >= ADC_ADD_MAX)
        || ((p_cfg->clearing != ADC_CLEAR_AFTER_READ_OFF) && (p_cfg->clearing != ADC_CLEAR_AFTER_READ_ON)))
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* Check for non-Group trigger and Group A trigger*/
    ret_trig = adc_check_trigger(p_cfg->trigger);
    if (ret_trig != ADC_SUCCESS)
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* Check for Group B trigger*/
    ret_trig = adc_check_trigger(p_cfg->trigger_groupb);
    if (ret_trig != ADC_SUCCESS)
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* Check alignment values only if addition is off */
    if ((p_cfg->alignment != ADC_ALIGN_LEFT) && (p_cfg->alignment != ADC_ALIGN_RIGHT))
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* In double trigger mode, SW and async triggers not allowed */
    if ((ADC_MODE_SS_ONE_CH_DBLTRIG == mode)
     && ((ADC_TRIG_SOFTWARE == p_cfg->trigger) || (ADC_TRIG_ADTRG0 == p_cfg->trigger)))
    {
        return ADC_ERR_ILLEGAL_ARG;
    }

    /* Group checking; only synchronous triggers allowed; must be unique */
    if ((ADC_MODE_SS_MULTI_CH_GROUPED == mode) || (ADC_MODE_SS_MULTI_CH_GROUPED_DBLTRIG_A == mode))
    {
        if ((ADC_TRIG_ADTRG0 == p_cfg->trigger )
            || (ADC_TRIG_ADTRG0 == p_cfg->trigger_groupb)
            || (p_cfg->trigger == p_cfg->trigger_groupb)
            || (ADC_TRIG_SOFTWARE == p_cfg->trigger)
            || (ADC_TRIG_SOFTWARE == p_cfg->trigger_groupb))
        {
            return ADC_ERR_ILLEGAL_ARG;
        }
    }

/* ADC_CFG_PARAM_CHECKING_ENABLE */
#endif

    if (adc_control.state >= ADC_OPEN)
    {
        return ADC_ERR_AD_NOT_CLOSED;
    }

    /* APPLY POWER TO PERIPHERAL */

    R_RST_WriteEnable();
    MSTP(S12ADC0) = 0;
    dummy32t = SYSTEM.MSTPCRC.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
    R_RST_WriteDisable();

    dummy16t = S12ADC0.ADCSR.WORD;  /* Workaround for lack of release procedure from the module-stop state(2/2) */

    /* ADCSR, ADEXICR, ADANSA, ADANSB, ADCER = 0 at power on */

    /* SET MODE RELATED REGISTER FIELDS */

    adc_control.mode = mode;
    if ((ADC_MODE_SS_MULTI_CH_GROUPED == mode)
        || (ADC_MODE_SS_MULTI_CH_GROUPED_DBLTRIG_A == mode))
    {
        S12ADC0.ADCSR.BIT.ADCS = ADC_ADCS_GROUP_SCAN;
    }
    else
    {
        if ((ADC_MODE_CONT_ONE_CH == mode) || (ADC_MODE_CONT_MULTI_CH == mode))
        {
            S12ADC0.ADCSR.BIT.ADCS = ADC_ADCS_CONT_SCAN;
        }

        /* other modes have ADCS=0 */
    }

    if ((ADC_MODE_SS_ONE_CH_DBLTRIG == mode)
        || (ADC_MODE_SS_MULTI_CH_GROUPED_DBLTRIG_A == mode))
    {
        S12ADC0.ADCSR.BIT.DBLE = 1;                   // enable double trigger
    }


    /* SET TRIGGER AND INTERRUPT PRIORITY REGISTER FIELDS */
    if (p_cfg->trigger != ADC_TRIG_SOFTWARE)
    {
        S12ADC0.ADSTRGR.BIT.TRSA = p_cfg->trigger;
    }
    if (ADC_TRIG_ADTRG0 == p_cfg->trigger)
    {
        S12ADC0.ADCSR.BIT.EXTRG = 1;      // set ext trigger for async trigger
    }
    if (ADC_ADCS_GROUP_SCAN == S12ADC0.ADCSR.BIT.ADCS)
    {
        S12ADC0.ADSTRGR.BIT.TRSB = p_cfg->trigger_groupb;
        InterruptHandlerRegister(S2GBADI0_IRQn, adc_gbadi_isr);
        VIC_SetPriority(S2GBADI0_IRQn, p_cfg->priority_groupb);
        VIC_SetDetectType(S2GBADI0_IRQn, VIC_TYPE_EDGE);
    }
    InterruptHandlerRegister(S12ADI0_IRQn, adc_s12adi0_isr);
    VIC_SetPriority(S12ADI0_IRQn, p_cfg->priority);
    VIC_SetDetectType(S12ADI0_IRQn, VIC_TYPE_EDGE);

    /* SET REGISTER FIELDS FOR REMAINING PARAMETERS */
    S12ADC0.ADADC.BIT.ADC = p_cfg->add_cnt;
    S12ADC0.ADCER.WORD = (uint16_t) ((uint16_t)p_cfg->alignment | (uint16_t)p_cfg->clearing);

    /* SAVE CALLBACK FUNCTION POINTER */

    adc_control.p_callback = (void(*)(int32_t))p_callback;

    /* MARK DRIVER AS OPENED */
    adc_control.state = ADC_OPEN;

    return ADC_SUCCESS;
}
/*******************************************************************************
End of function ADC_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: ADC_Control
* Description  : This function provides commands for enabling channels and
*                sensors and for runtime operations. These include enabling/
*                disabling trigger sources and interrupts, initiating a
*                software trigger, and checking for scan completion.
*
* NOTE: Enabling a channel or a sensor, or setting the sample state count reg
*       cannot be done while the ADSCR.ADST bit is set (conversion in progress).
*       Because these commands should only be called once during initialization
*       before triggers are enabled, this should not be an issue. Registers
*       with this restriction include ADANSA, ADANSB, ADADS, ADADC, ADSSTR,
*       ADEXICR, and some bits in ADSCR and TSCR.
*       No runtime operational sequence checking of any kind is performed.
*
* Arguments    : cmd-
*                    Command to run
*                p_args-
*                    Pointer to optional configuration structure
* Return Value : ADC_SUCCESS-
*                    Successful
*                ADC_ERR_MISSING_PTR-
*                    p_args pointer is NULL when required as an argument
*                ADC_ERR_INVALID_ARG-
*                    cmd or element of p_args structure has invalid value.
*                ADC_ERR_ILLEGAL_CMD-
*                    cmd is illegal based upon mode
*                ADC_ERR_SCAN_NOT_DONE-
*                    The requested scan has not completed
*******************************************************************************/
adc_err_t ADC_Control(adc_cmd_t const     cmd,
                        void * const        p_args)
{
    adc_err_t       err = ADC_SUCCESS;
    adc_time_t      *p_sample;

    switch (cmd)
    {
    case ADC_CMD_ENABLE_CHANS:
        if (NULL == p_args)
        {
            return ADC_ERR_MISSING_PTR;
        }

        err = adc_enable_chans((adc_ch_cfg_t *) p_args);
        break;

    case ADC_CMD_ENABLE_TEMP_SENSOR:
        if (adc_control.mode != ADC_MODE_SS_TEMPERATURE)
        {
            return ADC_ERR_ILLEGAL_ARG;
        }

        S12ADC0.ADEXICR.BIT.TSSA = 1;                  // select temperature sensor
        adc_enable_s12adi0();                       // setup interrupt handling
        break;

    case ADC_CMD_SET_SAMPLE_STATE_CNT:
        if (NULL == p_args)
        {
            return ADC_ERR_MISSING_PTR;
        }

        p_sample = (adc_time_t *) p_args;
#if ADC_CFG_PARAM_CHECKING_ENABLE == 1
        if ((p_sample->reg_id >= ADC_SST_NUM_REGS)
            || (p_sample->num_states < ADC_SST_CNT_MIN))
        {
            return ADC_ERR_INVALID_ARG;
        }
/* ADC_CFG_PARAM_CHECKING_ENABLE */
#endif
        adc_set_adsstr(p_sample);
        break;

    case ADC_CMD_ENABLE_TRIG:
        S12ADC0.ADCSR.BIT.TRGE = 1;           // enable sync/async triggers
        break;

    case ADC_CMD_DISABLE_TRIG:
        S12ADC0.ADCSR.BIT.TRGE = 0;           // disable sync/async triggers
        break;

    case ADC_CMD_SCAN_NOW:
        if (0 == S12ADC0.ADCSR.BIT.ADST)
        {
            S12ADC0.ADCSR.BIT.ADST = 1;
        }
        else
        {
            err = ADC_ERR_SCAN_NOT_DONE;
        }
        break;

    case ADC_CMD_ENABLE_INT:
        if (NULL == adc_control.p_callback)
        {
            return ADC_ERR_ILLEGAL_ARG;
        }
        S12ADC0.ADCSR.BIT.ADIE = 1;           // enable S12ADC0I0 interrupt
        break;

    case ADC_CMD_DISABLE_INT:
        S12ADC0.ADCSR.BIT.ADIE = 0;           // disable S12ADC0I0 interrupt
        break;

    case ADC_CMD_ENABLE_INT_GROUPB:
        if (NULL == adc_control.p_callback)
        {
            return ADC_ERR_ILLEGAL_ARG;
        }
        S12ADC0.ADCSR.BIT.GBADIE = 1;         // enable GBADI interrupt
        break;

    case ADC_CMD_DISABLE_INT_GROUPB:
        S12ADC0.ADCSR.BIT.GBADIE = 0;         // disable GBADI interrupt
        break;

    case ADC_CMD_CHECK_SCAN_DONE:           // default/Group A or Group B
        if (1 == S12ADC0.ADCSR.BIT.ADST)
        {
            err = ADC_ERR_SCAN_NOT_DONE;
        }
        break;

    case ADC_CMD_CHECK_SCAN_DONE_GROUPA:
        if (false != adc_control.complete)
        {
            adc_control.complete = false;
            VIC_EnableIRQ(S12ADI0_IRQn);
        }
        else
        {
            err = ADC_ERR_SCAN_NOT_DONE;
        }
        break;

    case ADC_CMD_CHECK_SCAN_DONE_GROUPB:
        if (false != adc_control.complete_groupb)
        {
            adc_control.complete_groupb = false;
            VIC_EnableIRQ(S2GBADI0_IRQn);
        }
        else
        {
            err = ADC_ERR_SCAN_NOT_DONE;
        }
        break;

    default:
        err = ADC_ERR_INVALID_ARG;
        break;
    }

    return err;
}
/*******************************************************************************
End of function ADC_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: ADC_Read
* Description  : This function reads conversion results from a single channel,
*                sensor, or the double trigger register.
* Arguments    : reg_id-
*                    Id for the register to read (see enum below)
*                p_data-
*                    Pointer to variable to load value into.
* Return Value : ADC_SUCCESS-
*                    Successful
*                ADC_ERR_INVALID_ARG-
*                    reg_id contains an invalid value.
*                ADC_ERR_MISSING _PTR-
*                    p_data is NULL
*******************************************************************************/
adc_err_t ADC_Read(adc_reg_t const    reg_id,
                     uint16_t * const   p_data)
{
#if ADC_CFG_PARAM_CHECKING_ENABLE == 1
    if (NULL == p_data)
    {
        return ADC_ERR_MISSING_PTR;
    }
    if ((reg_id >= ADC_REG_MAX)
        || (NULL == pdreg_ptrs[reg_id]))
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* Do not check if corresponding channel/sensor is enabled */
#endif

    *p_data = *pdreg_ptrs[reg_id];
    return ADC_SUCCESS;
}
/*******************************************************************************
End of function ADC_Read
*******************************************************************************/

/*******************************************************************************
* Function Name: ADC_ReadAll
* Description  : This function reads conversion results from all potential
*                sources, enabled or not.
* Arguments    : p_data-
*                    Pointer to structure to load register values into.
* Return Value : ADC_SUCCESS-
*                    Successful
*                ADC_ERR_MISSING _PTR-
*                    p_data is NULL
*******************************************************************************/
adc_err_t ADC_ReadAll(adc_data_t * const  p_all_data)
{
#if ADC_CFG_PARAM_CHECKING_ENABLE == 1
    if (NULL == p_all_data)
    {
        return ADC_ERR_MISSING_PTR;
    }
#endif

    p_all_data->chan[ADC_REG_CH0] = S12ADC0.ADDR0;
    p_all_data->chan[ADC_REG_CH1] = S12ADC0.ADDR1;
    p_all_data->chan[ADC_REG_CH2] = S12ADC0.ADDR2;
    p_all_data->chan[ADC_REG_CH3] = S12ADC0.ADDR3;
    p_all_data->chan[ADC_REG_CH4] = S12ADC0.ADDR4;
    p_all_data->chan[ADC_REG_CH5] = S12ADC0.ADDR5;
    p_all_data->chan[ADC_REG_CH6] = S12ADC0.ADDR6;
    p_all_data->chan[ADC_REG_CH7] = S12ADC0.ADDR7;
    p_all_data->dbltrig = S12ADC0.ADDBLDR;

    return ADC_SUCCESS;
}
/*******************************************************************************
End of function ADC_ReadAll
*******************************************************************************/

/*******************************************************************************
* Function Name: ADC_Close
* Description  : This function ends any scan in progress, disables interrupts,
*                and removes power to the A/D peripheral.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void ADC_Close(void)
{
    /* Stop triggers & conversions, and disable peripheral interrupts */
    VIC_DisableIRQ(S12ADI0_IRQn);
    VIC_DisableIRQ(S2GBADI0_IRQn);

    /* A/D conversion stop sequence process */
    if (1 == S12ADC0.ADCSR.BIT.ADCS)
    {
        S12ADC0.ADSTRGR.WORD = 0x3F3F;
        S12ADC0.ADCSR.BIT.GBADIE = 0;
        S12ADC0.ADCSR.BIT.ADIE = 0;
    }
    else
    {
        S12ADC0.ADSTRGR.BIT.TRSA = 0x3F;
        S12ADC0.ADCSR.BIT.ADIE = 0;
    }

    /* Clear register of S12ADC Unit */
    S12ADC0.ADCSR.WORD   = 0;
    S12ADC0.ADSTRGR.WORD = 0;
    S12ADC0.ADADC.BYTE   = 0;
    S12ADC0.ADCER.WORD   = 0;
    S12ADC1.ADEXICR.WORD = 0;
    S12ADC0.ADANSA.WORD  = 0;
    S12ADC0.ADANSB.WORD  = 0;

    /* Power down peripheral */
    R_RST_WriteEnable();
    MSTP(S12ADC0) = 1;
    R_RST_WriteDisable();

    /* Show driver as closed */
    adc_control.state = ADC_CLOSE;

    return;
}
/*******************************************************************************
End of function ADC_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_enable_chans
* Description  : This function does extensive checking on channel mask
*                settings based upon operational mode. Mask registers are
*                initialized and interrupts enabled in peripheral. Interrupts
*                are also enabled in ICU if corresponding priority is not 0.
*
* NOTE: A negative number is stored in two's complement form.
*       A quick way to change a binary number into two's complement is to
*       start at the right (LSB) and moving left, don't change any bits
*       until after the first "1" is reached.
*       Number          2's complement
*       0010 0110       1101 1010
*       0000 0001       1111 1111
*       Another way is to do a 1's complement on the number, then add 1 to that.
*       Number          1's complement  + 1
*       0010 0110       1101 1001       1101 1010
*       0000 0001       1111 1110       1111 1111
*
* Arguments    : p_config
*                    Pointer to channel config structure containing masks
* Return Value : ADC_SUCCESS-
*                    Successful
*                ADC_ERR_INVALID_ARG-
*                    reg_id contains an invalid value.
*                ADC_ERR_ILLEGAL_ARG-
*                    an argument is illegal based upon mode
*******************************************************************************/
static adc_err_t adc_enable_chans(adc_ch_cfg_t *p_config)
{
    uint16_t    i = 0;
    uint32_t    tmp_mask = 0;

#if ADC_CFG_PARAM_CHECKING_ENABLE == 1
    /* This command is illegal for sensor modes */
    if (ADC_MODE_SS_TEMPERATURE == adc_control.mode)
    {
        return ADC_ERR_ILLEGAL_ARG;
    }

    /* Verify at least one bonded channel is selected */
    if ((0 == p_config->chan_mask)
        || ((p_config->chan_mask & ADC_INVALID_CH_MASK) != 0))
    {
        return ADC_ERR_INVALID_ARG;
    }

    /* Verify at least one unique bonded channel is selected for Group B */
    if (ADC_ADCS_GROUP_SCAN == S12ADC0.ADCSR.BIT.ADCS)
    {
        if ((0 == p_config->chan_mask_groupb)
            || ((p_config->chan_mask_groupb & ADC_INVALID_CH_MASK) != 0))
        {
            return ADC_ERR_INVALID_ARG;
        }
        else if ((p_config->chan_mask & p_config->chan_mask_groupb) != 0)
        {
            return ADC_ERR_ILLEGAL_ARG;         // same chan in both groups
        }
        else
        {
            tmp_mask = p_config->chan_mask_groupb;
        }
    }
    else
    {
        p_config->chan_mask_groupb = 0;         // for addition mask checking
    }

    /* Addition mask should not include bits from inactive channels */
    if (S12ADC0.ADADC.BIT.ADC != ADC_ADD_OFF)
    {
        tmp_mask |= p_config->chan_mask;        // tmp_mask is Group A and B combined

        /* Bit-AND with 1s-complement */
        if ((p_config->add_mask & (~tmp_mask)) != 0)
        {
            return ADC_ERR_INVALID_ARG;
        }
    }

    /* Verify only 1 bit is set in default/Group A mask */
    if ((ADC_MODE_SS_ONE_CH == adc_control.mode)
        || (ADC_MODE_CONT_ONE_CH == adc_control.mode)
        || (1 == S12ADC0.ADCSR.BIT.DBLE ))        // double trigger mode
    {
        tmp_mask = p_config->chan_mask;     // tmp_mask is non-Group/Group A chans

        /* Bit-AND with 2s-complement (see note in function header) */
        if ((tmp_mask & ((~tmp_mask) + 1)) != tmp_mask)
        {
            return ADC_ERR_INVALID_ARG;
        }
    }

/* ADC_CFG_PARAM_CHECKING_ENABLE */
#endif

    /* SET MASKS FOR ALL CHANNELS */

    S12ADC0.ADANSA.WORD = (uint16_t) p_config->chan_mask;
    if (ADC_ADCS_GROUP_SCAN == S12ADC0.ADCSR.BIT.ADCS)
    {
        S12ADC0.ADANSB.WORD = (uint16_t) p_config->chan_mask_groupb;
    }
    if (S12ADC0.ADADC.BIT.ADC != ADC_ADD_OFF)
    {
        S12ADC0.ADADS.WORD = (uint16_t) p_config->add_mask;
    }

    /* SET DOUBLE TRIGGER CHANNEL */
    if (1 == S12ADC0.ADCSR.BIT.DBLE)
    {
        tmp_mask = p_config->chan_mask;     // tmp_mask is non-Group/Group A chans
        while (tmp_mask >>= 1)              // determine bit/ch number
        {
            i++;
        }
        S12ADC0.ADCSR.BIT.DBLANS = i;
    }

    /* ENABLE INTERRUPTS */
    adc_enable_s12adi0();
    if (ADC_ADCS_GROUP_SCAN == S12ADC0.ADCSR.BIT.ADCS)
    {
        adc_control.complete_groupb = false;
        S12ADC0.ADCSR.BIT.GBADIE = 1;         // enable in peripheral
        VIC_EnableIRQ(S2GBADI0_IRQn);
    }

    return ADC_SUCCESS;
}
/*******************************************************************************
End of function adc_enable_chans
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_enable_s12adi0
* Description  : This function clears the S12ADC0I0 interrupt flag and enables
*                interrupts in the peripheral (for IR flag usage). If priority
*                is not 0, interrupts are enabled in the ICU.

* NOTE: This has nothing to do with enabling triggers.

* Arguments    : none
* Return Value : none
*******************************************************************************/
static void adc_enable_s12adi0(void)
{
    adc_control.complete   = false;
    S12ADC0.ADCSR.BIT.ADIE = 1;               // enable in peripheral
    VIC_EnableIRQ(S12ADI0_IRQn);
}
/*******************************************************************************
End of function adc_enable_s12adi0
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_set_adsstr
* Description  : This function is setting the ADSSTR Register.
* Arguments    : adc_sst
* Return Value : none
*******************************************************************************/
static void adc_set_adsstr(const adc_time_t * const adc_sst)
{
    switch(adc_sst->reg_id)
    {
    case ADC_SST_CH0:
        S12ADC0.ADSSTR0.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH1:
        S12ADC0.ADSSTR1.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH2:
        S12ADC0.ADSSTR2.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH3:
        S12ADC0.ADSSTR3.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH4:
        S12ADC0.ADSSTR4.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH5:
        S12ADC0.ADSSTR5.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH6:
        S12ADC0.ADSSTR6.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_CH7:
        S12ADC0.ADSSTR7.BIT.SST = adc_sst->num_states;
        break;

    case ADC_SST_TEMPERATURE:
        S12ADC0.ADSSTRT.BIT.SST = adc_sst->num_states;
        break;

    default:
        /* DO NOTHING */

        break;
    }
}
/*******************************************************************************
End of function adc_set_adsstr
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_check_trigger
* Description  : This function is checking Synchronization trigger.
* Arguments    : trig
* Return Value : none
*******************************************************************************/
static adc_err_t adc_check_trigger(adc_trig_t trig)
{
    adc_err_t ret = ADC_SUCCESS;

    switch (trig)
    {
    case ADC_TRIG_ADTRG0:
    case ADC_TRIG_TRGA0N:
    case ADC_TRIG_TRGA1N:
    case ADC_TRIG_TRGA2N:
    case ADC_TRIG_TRGA3N:
    case ADC_TRIG_TRGA4N:
    case ADC_TRIG_TRGA6N:
    case ADC_TRIG_TRGA7N:
    case ADC_TRIG_TRG0N:
    case ADC_TRIG_TRG4AN:
    case ADC_TRIG_TRG4BN:
    case ADC_TRIG_TRG4AN_OR_TRG4BN:
    case ADC_TRIG_TRG4ABN:
    case ADC_TRIG_TRG7AN:
    case ADC_TRIG_TRG7BN:
    case ADC_TRIG_TRG7AN_OR_TRG7BN:
    case ADC_TRIG_TRG7ABN:
    case ADC_TRIG_GTADTRA0N:
    case ADC_TRIG_GTADTRB0N:
    case ADC_TRIG_GTADTRA1N:
    case ADC_TRIG_GTADTRB1N:
    case ADC_TRIG_GTADTRA2N:
    case ADC_TRIG_GTADTRB2N:
    case ADC_TRIG_GTADTRA3N:
    case ADC_TRIG_GTADTRB3N:
    case ADC_TRIG_GTADTRA0N_OR_GTADTRB0N:
    case ADC_TRIG_GTADTRA1N_OR_GTADTRB1N:
    case ADC_TRIG_GTADTRA2N_OR_GTADTRB2N:
    case ADC_TRIG_GTADTRA3N_OR_GTADTRB3N:
    case ADC_TRIG_TPTRGAN_0:
    case ADC_TRIG_TPTRG0AN_0:
    case ADC_TRIG_TPTRGAN_1:
    case ADC_TRIG_TPTRG6AN_1:
    case ADC_TRIG_ELCTRG0:
    case ADC_TRIG_SOFTWARE:
        break;

    default:
        ret = ADC_ERR_INVALID_ARG;
        break;
    }

    return ret;
}
/*******************************************************************************
End of function adc_check_trigger
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_s12adi0_isr
* Description  : Interrupt handler for normal/Group A/double trigger
*                scan complete.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void adc_s12adi0_isr(void)
{
    adc_control.complete = true;

    /* presence of callback function verified in Open() */
    if (adc_control.p_callback)
    {
        adc_control.p_callback(ADC_EVT_SCAN_COMPLETE);
    }
}
/*******************************************************************************
End of function adc_s12adi0_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: adc_gbadi_isr
* Description  : Interrupt handler for Group B scan complete.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void adc_gbadi_isr(void)
{
    adc_control.complete_groupb = true;

    /* presence of callback function verified in Open() */
    if (adc_control.p_callback)
    {
        adc_control.p_callback(ADC_EVT_SCAN_COMPLETE_GROUPB);
    }
}
/*******************************************************************************
End of function adc_gbadi_isr
*******************************************************************************/

/* End of File */

