/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : adc driver interface functions for adc fit module.
* File Name    : r_adc_if.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for ADC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ADC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "cmsis_os.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "r_adc.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static osSemaphoreDef(sem_adc_state);

/*******************************************************************************
* Function Name: R_ADC_Init
* Description  : This function initialize ADC driver.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_ADC_Init(void)
{
    if (adc_control.state == ADC_UNINIT)
    {
        adc_control.sem_state = osSemaphoreCreate(osSemaphore(sem_adc_state), 1);
        adc_control.state     = ADC_INIT;
    }
}
/*******************************************************************************
End of function R_ADC_Init
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_Uninit
* Description  : This function uninitialize ADC driver.
* Arguments    : None
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_Uninit(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    osStatus    sem_state_ercd;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state != ADC_UNINIT)
        {
            if (adc_control.state >= ADC_OPEN)
            {
                ADC_Close();        /* stop ADC module */
            }

            /* release semaphore */
            sem_state_ercd    = osSemaphoreDelete(adc_control.sem_state);
            if (sem_state_ercd != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Delete Error */
                ercd = EFAULT;
            }
            else
            {
                adc_control.state = ADC_UNINIT;
            }
        }
        else
        {
            ercd = EACCES;
        }

        if (ercd != ESUCCESS)
        {
            sem_ret = osSemaphoreRelease(adc_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_Uninit
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_Open
* Description  : This function applies power to the A/D peripheral, sets the
*                operational mode, trigger sources, interrupt priority, and
*                configurations common to all channels and sensors. If interrupt
*                priority is non-zero, the function takes a callback function
*                pointer for notifying the user at interrupt level whenever a
*                scan has completed.
* Arguments    : mode-
*                    Operational mode (see enumeration below)
*                p_cfg-
*                    Pointer to configuration structure (see below)
*                p_callback-
*                    Optional pointer to function called from interrupt when
*                    a scan completes
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_Open(adc_mode_t const  mode,
                   adc_cfg_t * const p_cfg,
                   void              (* const p_callback)(int32_t event))
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    adc_err_t   err;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state == ADC_CLOSE)
        {
            err = ADC_Open(mode, p_cfg, (void(*)(void *))p_callback);
            if (err != ADC_SUCCESS)
            {
                switch (err)
                {
                    case ADC_ERR_MISSING_PTR:
                    case ADC_ERR_INVALID_ARG:
                    case ADC_ERR_ILLEGAL_ARG:

                        ercd = EINVAL;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (adc_control.state == ADC_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(adc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_Control
* Description  : This function provides commands for enabling channels and
*                sensors and for runtime operations. These include enabling/
*                disabling trigger sources and interrupts, initiating a
*                software trigger, and checking for scan completion.
*
* NOTE: Enabling a channel or a sensor, or setting the sample state count reg
*       cannot be done while the ADSCR.ADST bit is set (conversion in progress).
*       Because these commands should only be called once during initialization
*       before triggers are enabled, this should not be an issue. Registers
*       with this restriction include ADANSA, ADANSB, ADADS, ADADC, ADSSTR,
*       ADEXICR, and some bits in ADSCR and TSCR.
*       No runtime operational sequence checking of any kind is performed.
*
* Arguments    : cmd-
*                    Command to run
*                p_args-
*                    Pointer to optional configuration structure
** Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_Control(adc_cmd_t const cmd, void * const p_args)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    adc_err_t   err;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state >= ADC_INIT)
        {
            err = ADC_Control(cmd, p_args);
            if (err != ADC_SUCCESS)
            {
                switch (err)
                {
                    case ADC_ERR_MISSING_PTR:
                    case ADC_ERR_INVALID_ARG:
                    case ADC_ERR_ILLEGAL_ARG:

                        ercd = EINVAL;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(adc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_Read
* Description  : This function reads conversion results from a single channel,
*                sensor, or the double trigger register.
* Arguments    : reg_id-
*                    Id for the register to read (see enum below)
*                p_data-
*                    Pointer to variable to load value into.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_Read(adc_reg_t const  reg_id,
                   uint16_t * const p_data)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    adc_err_t   err;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state >= ADC_OPEN)
        {
            err = ADC_Read(reg_id, p_data);
            if (err != ADC_SUCCESS)
            {
                switch (err)
                {
                    case ADC_ERR_MISSING_PTR:
                    case ADC_ERR_INVALID_ARG:
                    case ADC_ERR_ILLEGAL_ARG:

                        ercd = EINVAL;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(adc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_Read
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_ReadAll
* Description  : This function reads conversion results from all potential
*                sources, enabled or not.
* Arguments    : p_data-
*                    Pointer to structure to load register values into.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_ReadAll(adc_data_t * const p_all_data)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    adc_err_t   err;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state >= ADC_OPEN)
        {
            err = ADC_ReadAll(p_all_data);
            if (err != ADC_SUCCESS)
            {
                switch (err)
                {
                    case ADC_ERR_MISSING_PTR:
                    case ADC_ERR_INVALID_ARG:
                    case ADC_ERR_ILLEGAL_ARG:

                        ercd = EINVAL;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(adc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_ReadAll
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_Close
* Description  : This function ends any scan in progress, disables interrupts,
*                and removes power to the A/D peripheral.
* Arguments    : none
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ADC_Close(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;

    sem_ret = osSemaphoreWait(adc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (adc_control.state >= ADC_OPEN)
        {
            ADC_Close();
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(adc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ADC_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ADC_GetVersion
* Description  : Returns the version of this module. The version number is
*                encoded such that the top two bytes are the major version
*                number and the bottom two bytes are the minor version number.
* Arguments    : none
* Return Value : version number
*******************************************************************************/
uint32_t R_ADC_GetVersion(void)
{
    uint32_t const version = (ADC_VERSION_MAJOR << 16) | ADC_VERSION_MINOR;

    return version;
}
/*******************************************************************************
End of function R_ADC_GetVersion
*******************************************************************************/

/* End of File */

