/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : CMT(W) driver interface functions for adc fit module.
* File Name    : r_cmt_if.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMT(W) on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "r_cmt.h"

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define CMT_CHECK_CHANNEL(ch)           (((CMT_CHANNEL_0 <= (int32_t)ch) && (CMT_CHANNEL_MAX > (int32_t)ch))?true:false)

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private global variables and functions
*******************************************************************************/
static osSemaphoreDef(sem_cmt_state);

/*******************************************************************************
* Function Name: R_CMT_Init
* Description  : This function initialize CMT driver.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_CMT_Init(void)
{
    if (cmt_control.state == CMT_UNINIT)
    {
        cmt_control.sem_state = osSemaphoreCreate(osSemaphore(sem_cmt_state), 1);
        cmt_control.state     = CMT_INIT;
    }
}
/*******************************************************************************
End of function R_CMT_Init
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_Uninit
* Description  : This function uninitialize CMT driver.
* Arguments    : None
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_Uninit(void)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    osStatus    sem_state_ercd;
    uint32_t    channel;
    cmt_err_t   ret;

    sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (cmt_control.state > CMT_UNINIT)
        {
            for (channel = 0; channel < CMT_CHANNEL_MAX; channel++)
            {
                if (cmt_control.p_ch_info[channel]->state >= CMT_OPEN)
                {
                    if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                    {
                        ret = CMTW_Close(channel - CMTW_CHANNEL_0);
                    }
                    else
                    {
                        ret = CMT_Close(channel);
                    }

                    if (ret != CMT_SUCCESS)
                    {
                        /*
                            CMT_ERR_INVALID_CH
                            CMT_ERR_INVALID_ARG
                        */
                        ercd = EINVAL;
                        break;
                    }
                }
            }

            if (ercd == ESUCCESS)
            {
                /* release semaphore */
                sem_state_ercd = osSemaphoreDelete(cmt_control.sem_state);
                if (sem_state_ercd != osOK)
                {
                    /* NON NOTICE ASSERT: Semaphore Delete Error */
                    ercd = EFAULT;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }
        if (ercd != ESUCCESS)
        {
            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_CMT_Uninit
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_Open
* Description  : This function does open CMT(W) driver.
*                CMT(W) HW, driver information is initialized and module stop
*                control is set to OFF.
* Arguments    : channel-
*                    CMT(W) channel
*                p_cfg-
*                    CMT(W) configure parameter
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_Open(cmt_channel_t const channel, cmt_cfg_t * const p_cfg)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    cmt_err_t   ret;

    if (CMT_CHECK_CHANNEL((int32_t)channel) == false)
    {
        ercd = EINVAL;
    }
    else
    {
        sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

        if (sem_ret != -1)
        {
            if (cmt_control.p_ch_info[channel]->state == CMT_CLOSE)
            {
                if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                {
                    ret = CMTW_Open((channel - CMTW_CHANNEL_0), p_cfg);
                }
                else
                {
                    ret = CMT_Open(channel, p_cfg);
                }

                if (ret != CMT_SUCCESS)
                {
                    ercd = EINVAL;
                }
            }
            else if (cmt_control.p_ch_info[channel]->state == CMT_RUN)
            {
                ercd = EBUSY;
            }
            else
            {
                ercd = EACCES;
            }

            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    return ercd;
}
/*******************************************************************************
End of function R_CMT_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_Close
* Description  : This function does close CMT(W) driver.
*                Module stop control is set to ON.
* Arguments    : channel-
*                    CMT(W) channel
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_Close(cmt_channel_t const channel)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    cmt_err_t   ret;

    if (CMT_CHECK_CHANNEL((int32_t)channel) == false)
    {
        ercd = EINVAL;
    }
    else
    {
        sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

        if (sem_ret != -1)
        {
            if (cmt_control.p_ch_info[channel]->state >= CMT_OPEN)
            {
                if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                {
                    ret = CMTW_Close((channel - CMTW_CHANNEL_0));
                }
                else
                {
                    ret = CMT_Close(channel);
                }

                if (ret != CMT_SUCCESS)
                {
                    ercd = EINVAL;
                }
            }
            else
            {
                ercd = EACCES;
            }

            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    return ercd;
}
/*******************************************************************************
End of function R_CMT_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_StartPeriodic
* Description  : This function start make start timer with periodic operation.
* Arguments    : channel-
*                    CMT(W) channel
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_StartPeriodic(cmt_channel_t const channel)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    cmt_err_t   ret;

    if (CMT_CHECK_CHANNEL((int32_t)channel) == false)
    {
        ercd = EINVAL;
    }
    else
    {
        sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

        if (sem_ret != -1)
        {
            if (cmt_control.p_ch_info[channel]->state == CMT_OPEN)
            {
                if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                {
                    ret = CMTW_StartPeriodic((channel - CMTW_CHANNEL_0));
                }
                else
                {
                    ret = CMT_StartPeriodic(channel);
                }

                if (ret != CMT_SUCCESS)
                {
                    ercd = EINVAL;
                }
            }
            else if (cmt_control.p_ch_info[channel]->state == CMT_RUN)
            {
                ercd = EBUSY;
            }
            else
            {
                ercd = EACCES;
            }

            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    return ercd;
}
/*******************************************************************************
End of function R_CMT_StartPeriodic
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_StartOneShot
* Description  : This function start make start timer with one shot operation.
* Arguments    : channel-
*                    CMT(W) channel
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_StartOneShot(cmt_channel_t const channel)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    cmt_err_t   ret;

    if (CMT_CHECK_CHANNEL((int32_t)channel) == false)
    {
        ercd = EINVAL;
    }
    else
    {
        sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

        if (sem_ret != -1)
        {
            if (cmt_control.p_ch_info[channel]->state == CMT_OPEN)
            {
                if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                {
                    ret = CMTW_StartOneShot((channel - CMTW_CHANNEL_0));
                }
                else
                {
                    ret = CMT_StartOneShot(channel);
                }

                if (ret != CMT_SUCCESS)
                {
                    ercd = EINVAL;
                }
            }
            else if (cmt_control.p_ch_info[channel]->state == CMT_RUN)
            {
                ercd = EBUSY;
            }
            else
            {
                ercd = EACCES;
            }

            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    return ercd;
}
/*******************************************************************************
End of function R_CMT_StartOneShot
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_Control
* Description  : This function provides commands for changing CMT(W) setting,
*                operating counter.
*
* Arguments    : channel-
*                    CMT(W) channel
*                cmd-
*                    Command to CMTW operation
*                p_data-
*                    Pointer to optional configuration structure
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_CMT_Control(cmt_channel_t const channel, uint32_t const cmd, void * const p_data)
{
    int32_t     ercd            = ESUCCESS;
    int32_t     sem_ret;
    cmt_err_t   ret;

    if (CMT_CHECK_CHANNEL((int32_t)channel) == false)
    {
        ercd = EINVAL;
    }
    else
    {
        sem_ret = osSemaphoreWait(cmt_control.sem_state, osWaitForever);

        if (sem_ret != -1)
        {
            if (cmt_control.p_ch_info[channel]->state >= CMT_OPEN)
            {
                if ((channel == CMTW_CHANNEL_0) || (channel == CMTW_CHANNEL_1))
                {
                    ret = CMTW_Control((channel - CMTW_CHANNEL_0), cmd, p_data);
                }
                else
                {
                    ret = CMT_Control(channel, cmd, p_data);
                }

                if (ret != CMT_SUCCESS)
                {
                    ercd = EINVAL;
                }
            }
#if 0
            else if (cmt_control.p_ch_info[channel]->state == CMT_RUN)
            {
                ercd = EBUSY;
            }
#endif
            else
            {
                ercd = EACCES;
            }

            sem_ret = osSemaphoreRelease(cmt_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    return ercd;
}
/*******************************************************************************
End of function R_CMT_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMT_GetVersion
* Description  : Returns CMT(W) device driver version.
* Arguments    : none
* Return Value : version number
*******************************************************************************/
uint32_t  R_CMT_GetVersion(void)
{
    uint32_t const version = ((CMT_VERSION_MAJOR << 16) | CMT_VERSION_MINOR);
    return version;
}
/*******************************************************************************
End of function R_CMT_GetVersion
*******************************************************************************/

/* End of File */

