/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmt driver functions for cmt fit module.
* File Name    : r_cmt_userdef.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMT on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_cmt.h"
#include "r_cmt_userdef.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define CMT_FLAG_OFF     (0)
#define CMT_FLAG_ON      (1)

/*******************************************************************************
Typedef definitions
*******************************************************************************/
typedef struct st_cmt_intr
{
    IRQn_Type   vic_irq;
    uint32_t    type;
    void *      intr_handler;
} cmt_intr_t;

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private global variables and functions
*******************************************************************************/
void userdef_cmt_init(uint32_t const channel, uint16_t cks);
void userdef_cmt_isr_cmi(uint32_t channel);
void cmt0_isr(void);
void cmt1_isr(void);
void cmt2_isr(void);
void cmt3_isr(void);
void cmt4_isr(void);
void cmt5_isr(void);

static const cmt_intr_t gb_cmt_intr_table[CMT_CH_NUM] =
    {
        { CMI0_IRQn, VIC_TYPE_EDGE, (void *)cmt0_isr },
        { CMI1_IRQn, VIC_TYPE_EDGE, (void *)cmt1_isr },
        { CMI2_IRQn, VIC_TYPE_EDGE, (void *)cmt2_isr },
        { CMI3_IRQn, VIC_TYPE_EDGE, (void *)cmt3_isr },
        { CMI4_IRQn, VIC_TYPE_EDGE, (void *)cmt4_isr },
        { CMI5_IRQn, VIC_TYPE_EDGE, (void *)cmt5_isr }
    };

/*******************************************************************************
* Function Name: userdef_cmt_init
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
*              : uint16_t cks     : Select CMT clock source
*              :                  : (Setting value of CMCR CKS bit)
*              :                  :   CMT_CKS_DIVISION_8   : PCLKD / 8
*              :                  :   CMT_CKS_DIVISION_32  : PCLKD / 32
*              :                  :   CMT_CKS_DIVISION_128 : PCLKD / 128
*              :                  :   CMT_CKS_DIVISION_512 : PCLKD / 512
* Return Value : none
*******************************************************************************/
void userdef_cmt_init(uint32_t const channel, uint16_t cks)
{
    *gb_cmt_info[channel].p_cmt_ch_str              &= ~gb_cmt_info[channel].str_mask;
    gb_cmt_info[channel].p_cmt_ch_reg->CMCNT        = 0;
    gb_cmt_info[channel].p_cmt_ch_reg->CMCR.BIT.CKS = cks;

    VIC_DisableIRQ(gb_cmt_intr_table[channel].vic_irq);
    InterruptHandlerRegister(gb_cmt_intr_table[channel].vic_irq,
                             (IRQHandler)gb_cmt_intr_table[channel].intr_handler);
    VIC_SetPriority(gb_cmt_intr_table[channel].vic_irq, VIC_PRIORITY_15);
    VIC_SetDetectType(gb_cmt_intr_table[channel].vic_irq,
                      gb_cmt_intr_table[channel].type);
    VIC_EnableIRQ(gb_cmt_intr_table[channel].vic_irq);
}
/*******************************************************************************
End of function userdef_cmt_init
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt0_start
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
*              : uint16_t compare_match_cnt : compare match count
*              : void * callback            : callback function
* Return Value : none
*******************************************************************************/
void userdef_cmt_open(uint32_t const channel, cmt_cfg_t * p_cfg)
{
    /* cmt initialize */
    userdef_cmt_init(channel, (uint16_t)p_cfg->time_cnt_param.pclk_div);

    /* Initialize Mode */
    gb_cmt_info[channel].operate_mode = CMT_PERIODIC;

    /* Save clear factor */
    gb_cmt_info[channel].clear_factor = p_cfg->time_cnt_param.clear_factor;

    /* Save callback function to be used. */
    gb_cmt_info[channel].p_compare_match_callback = p_cfg->mode_param.compare_match.p_callback;

    /* constant set  */
    gb_cmt_info[channel].p_cmt_ch_reg->CMCOR = p_cfg->mode_param.compare_match.compare_match_cnt;
}
/*******************************************************************************
End of function userdef_cmt_open
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt_stop
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
* Return Value : none
*******************************************************************************/
void userdef_cmt_start(uint32_t const channel, uint32_t const operate_mode)
{
    gb_cmt_info[channel].operate_mode                = operate_mode;
    gb_cmt_info[channel].p_cmt_ch_reg->CMCR.BIT.CMIE = CMT_FLAG_ON;
    *gb_cmt_info[channel].p_cmt_ch_str               |= gb_cmt_info[channel].str_mask;
}
/*******************************************************************************
End of function userdef_cmt_start
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt_stop
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
* Return Value : none
*******************************************************************************/
void userdef_cmt_stop(uint32_t const channel)
{
    *gb_cmt_info[channel].p_cmt_ch_str &= ~gb_cmt_info[channel].str_mask;
}
/*******************************************************************************
End of function userdef_cmt_stop
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt_isr_cmi
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
* Return Value : none
*******************************************************************************/
void userdef_cmt_isr_cmi(uint32_t channel)
{
    /* If this is one-shot mode then stop timer. */
    if (CMT_ONE_SHOT == gb_cmt_info[channel].operate_mode)
    {
        gb_cmt_info[channel].state = CMT_END;
        userdef_cmt_stop(channel);
    }

    /* Check for valid callback pointer before calling it. */
    if (NULL != gb_cmt_info[channel].p_compare_match_callback)
    {
        /* Valid callback found, jump to it. */
        gb_cmt_info[channel].p_compare_match_callback();
    }
}
/*******************************************************************************
End of function userdef_cmt_isr_cmi
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt0_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt0_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_0);
}
/*******************************************************************************
End of function cmt1_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt1_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt1_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_1);
}
/*******************************************************************************
End of function cmt1_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt2_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt2_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_2);
}
/*******************************************************************************
End of function cmt2_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt3_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt3_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_3);
}
/*******************************************************************************
End of function cmt3_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt4_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt4_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_4);
}
/*******************************************************************************
End of function cmt4_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt5_isr
* Description  : Interrupt when compare match for this channel occurs.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cmt5_isr(void)
{
    userdef_cmt_isr_cmi(CMT_CH_5);
}
/*******************************************************************************
End of function cmt5_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt_set_time_cnt_param
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
*              : cmt_time_cnt_t * const p_time_cnt_param : pointer time count parameter
* Return Value : none
*******************************************************************************/
void userdef_cmt_set_time_cnt_param(uint32_t const channel,
                                    cmt_time_cnt_t * const p_time_cnt_param)
{
    /* Set CMCR.CKS */
    gb_cmt_info[channel].p_cmt_ch_reg->CMCR.BIT.CKS = (uint16_t)p_time_cnt_param->pclk_div;

    /* Save clear factor */
    gb_cmt_info[channel].clear_factor = p_time_cnt_param->clear_factor;
}
/*******************************************************************************
End of function userdef_cmt_set_time_cnt_param
*******************************************************************************/

/*******************************************************************************
* Function Name: userdef_cmt_set_compare_match_param
* Description  :
* Arguments    : uint32_t const channel : 0 ~ 5
*              : cmt_compare_match_t * const p_compare_match : pointer compare match parameter
* Return Value : none
*******************************************************************************/
void userdef_cmt_set_compare_match_param(uint32_t const channel,
                                         cmt_compare_match_t * const p_compare_match)
{
    /* Save callback function to be used. */
    gb_cmt_info[channel].p_compare_match_callback = p_compare_match->p_callback;

    /* constant set  */
    gb_cmt_info[channel].p_cmt_ch_reg->CMCOR = p_compare_match->compare_match_cnt;
}
/*******************************************************************************
End of function userdef_cmt_set_compare_match_param
*******************************************************************************/

/* End of File */

