/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmtw driver functions for cmtw fit module.
* File Name    : r_cmtw.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMTW on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_cmt.h"
#include "r_cmtw_int.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static void cmtw_init(const uint32_t channel);
static cmt_err_t cmtw_check_param(const cmtw_cfg_t * const p_cfg);
static cmt_err_t cmtw_check_count_param(const cmtw_time_cnt_t * const p_time_cnt_param);
static cmt_err_t cmtw_check_compare_match_param(const cmtw_compare_match_t * const p_compare_match_param);
static cmt_err_t cmtw_check_input_capture_param
                    (const cmtw_input_capture_t * const p_input_capture_param, const uint32_t noise_filter_clk);
static cmt_err_t cmtw_check_output_compare_param (const cmtw_output_compare_t * const p_output_compare_param);
static void cmtw_setup(const uint32_t channel, const cmtw_cfg_t * const p_cfg);
static void cmtw_set_count_param(const uint32_t channel, const cmtw_time_cnt_t * const p_time_cnt_param);
static void cmtw_set_compare_match_param(const uint32_t channel, const cmtw_compare_match_t * const p_compare_match_param);
static void cmtw_set_input_capture_param
                (const uint32_t channel, const cmtw_input_capture_t * const p_input_capture_param, const uint32_t noise_filter_clk);
static void cmtw_set_output_compare_param(const uint32_t channel, const cmtw_output_compare_t * const p_output_compare_param);
static cmt_err_t cmtw_set_ecm_param(const uint32_t channel, const cmtw_ecm_t * const p_ecm_param);
static cmt_err_t cmtw_count_start(const uint32_t channel, const uint32_t operate_mode);
static void cmtw_count_start_hw(const uint32_t channel);

/* CMTW driver information */
cmtw_info_ch_t gb_cmtw_info[CMTW_CH_NUM] =
{
    {
        CMT_CLOSE,
        CMT_PERIODIC,
        (uint16_t)CMTW_CNT_CLEAR_COMPARE_MATCH,
        false,
        NULL,
        { false, false },
        { false, false },
        { NULL, NULL },
        { NULL, NULL },
        &CMTW0
    },
    {
        CMT_CLOSE,
        CMT_PERIODIC,
        (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
        false,
        NULL,
        { false, false },
        { false, false },
        { NULL, NULL },
        { NULL, NULL },
        &CMTW1
    }
};

/*******************************************************************************
* Function Name: R_CMTW_Open
* Description  : This function does open CMTW driver.
*                CMTW HW, driver information is initialized and module stop
*                control is set to OFF.
* Arguments    : channel-
*                    CMTW channel
*                p_cfg-
*                    CMTW configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_NOT_CLOSED-
*                    Open() is called when status is not closed.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
cmt_err_t CMTW_Open(const uint32_t channel, const cmtw_cfg_t * const p_cfg)
{
    cmt_err_t retval = CMT_SUCCESS;

    volatile uint32_t dummy32t;
    volatile uint16_t dummy16t;

    /* check channel */
    if (channel >= CMTW_CH_NUM)
    {
        retval = CMT_ERR_INVALID_CH;
    }

    if (CMT_SUCCESS == retval)
    {
        /* check config parameter pointer */
        if (NULL == p_cfg)
        {
            retval = CMT_ERR_MISSING_PTR;
        }
    }

    if (CMT_SUCCESS == retval)
    {
        /* check driver status */
        if (CMT_CLOSE != gb_cmtw_info[channel].state)
        {
            retval = CMT_ERR_NOT_CLOSED;
        }
    }

    if (CMT_SUCCESS == retval)
    {
        /* check p_cfg members */
        retval = cmtw_check_param(p_cfg);
    }

    if (CMT_SUCCESS == retval)
    {
        /* init CMTW driver */
        R_RST_WriteEnable();
        if (CMTW_CH_0 == channel)
        {
            /* stop clear to CMTW0 */
            MSTP(CMTW0) = 0U;
            dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
        }
        else
        {
            /* stop clear to CMTW1 */
            MSTP(CMTW1) = 0U;
            dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
        }
        R_RST_WriteDisable();
        if (CMTW_CH_0 == channel)
        {
            dummy16t = CMTW0.CMWCR.WORD;      /* Workaround for lack of release procedure from the module-stop state(2/2) */
        }
        else
        {
            dummy16t = CMTW1.CMWCR.WORD;      /* Workaround for lack of release procedure from the module-stop state(2/2) */
        }

        /* init HW and driver information */
        cmtw_init(channel);

        cmtw_setup(channel, p_cfg);
        gb_cmtw_info[channel].state = CMT_OPEN;
    }

    return retval;
}
/*******************************************************************************
End of function R_CMTW_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMTW_Close
* Description  : This function does close CMTW driver.
*                Module stop control is set to ON.
* Arguments    : channel-
*                    CMTW channel
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*******************************************************************************/
cmt_err_t CMTW_Close(const uint32_t channel)
{
    cmt_err_t retval = CMT_SUCCESS;

    /* check channel */
    if (channel >= CMTW_CH_NUM)
    {
        retval = CMT_ERR_INVALID_CH;
    }
    else
    {
        /* uninit CMTW driver */
        cmtw_count_stop_hw(channel);

        R_RST_WriteEnable();
        if (CMTW_CH_0 == channel)
        {
            /* set stop to CMTW0 */
            MSTP(CMTW0) = 1U;
        }
        else
        {
            /* set stop to CMTW1 */
            MSTP(CMTW1) = 1U;
        }
        R_RST_WriteDisable();

        gb_cmtw_info[channel].state = CMT_CLOSE;

    }

    return retval;
}
/*******************************************************************************
End of function R_CMTW_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMTW_StartPeriodic
* Description  : This function start make start timer with periodic operation.
* Arguments    : channel-
*                    CMTW channel
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_NOT_OPENED-
*                    Function is called when status is not initialized.
*                CMT_ERR_TIMER_RUNNING-
*                    Function is called while running counter.
*******************************************************************************/
cmt_err_t CMTW_StartPeriodic(const uint32_t channel)
{
    cmt_err_t retval;

    /* call count start function with CMTW periodic operation */
    retval = cmtw_count_start(channel, CMT_PERIODIC);

    return retval;
}
/*******************************************************************************
End of function R_CMTW_StartPeriodic
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMTW_StartOneShot
* Description  : This function start make start timer with one shot operation.
* Arguments    : channel-
*                    CMTW channel
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_NOT_OPENED-
*                    Function is called when status is not initialized.
*                CMT_ERR_TIMER_RUNNING-
*                    Function is called while running counter.
*******************************************************************************/
cmt_err_t CMTW_StartOneShot(const uint32_t channel)
{
    cmt_err_t retval;

    /* call count start function with CMTW one shot operation */
    retval = cmtw_count_start(channel, CMT_ONE_SHOT);

    return retval;
}
/*******************************************************************************
End of function R_CMTW_StartOneShot
*******************************************************************************/

/*******************************************************************************
* Function Name: R_CMTW_Control
* Description  : This function provides commands for changing CMTW setting,
*                operating counter.
*
* Arguments    : channel-
*                    CMTW channel
*                cmd-
*                    Command to CMTW operation
*                p_data-
*                    Pointer to optional configuration structure
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_NOT_OPENED-
*                    Function is called when status is not closed.
*                CMT_ERR_TIMER_RUNNING-
*                    Function is called while running counter.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
cmt_err_t CMTW_Control(const uint32_t channel, const uint32_t cmd, void * const p_data)
{
    cmt_err_t       retval = CMT_SUCCESS;
    cmt_mode_t *    pmode_param;
    uint32_t *      pcmt_status;

    /* check channel */
    if (channel >= CMTW_CH_NUM)
    {
        retval = CMT_ERR_INVALID_CH;
    }

    if (CMT_SUCCESS == retval)
    {
        /* check wheter CMTW status is open or running */
        if (CMT_CLOSE == gb_cmtw_info[channel].state)
        {
            retval = CMT_ERR_NOT_OPENED;
        }
    }

    if (CMT_SUCCESS == retval)
    {
        /* operation for every CMTW command */
        switch (cmd)
        {
            /* check wheter CMTW status is open */
            case CMT_CMD_SET_TIME_CNT :
                if (CMT_RUN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_RUNNING;
                }
                else
                {
                    /* casting the void pointer to a cmtw_time_cnt_t type is valid structure,
                       because structure member has justification checked separately */
                    retval = cmtw_check_count_param((cmtw_time_cnt_t *) p_data);
                    if (CMT_SUCCESS == retval)
                    {
                        cmtw_set_count_param(channel, (cmtw_time_cnt_t *) p_data);
                    }
                }
            break;

            case CMT_CMD_SET_MODE:

                /* casting the void pointer to a cmt_mode_t type is valid structure,
                   because structure member has justification checked separately */
                pmode_param = (cmt_mode_t *)p_data;

                if (NULL == pmode_param)
                {
                    retval = CMT_ERR_MISSING_PTR;
                }
                else
                {
                    /* check wheter CMTW status is open */
                    if (CMT_RUN == gb_cmtw_info[channel].state)
                    {
                        retval = CMT_ERR_TIMER_RUNNING;
                    }
                    else
                    {
                        retval = cmtw_check_compare_match_param(&pmode_param->compare_match);
                        if (CMT_SUCCESS == retval)
                        {
                            cmtw_set_compare_match_param(channel, &pmode_param->compare_match);
                        }
                    }

                    if (CMT_SUCCESS == retval)
                    {
                        retval = cmtw_check_input_capture_param(pmode_param->input_capture, pmode_param->noise_filter_clk);
                        if (CMT_SUCCESS == retval)
                        {
                            cmtw_set_input_capture_param(channel, pmode_param->input_capture, pmode_param->noise_filter_clk);
                        }
                    }

                    if (CMT_SUCCESS == retval)
                    {
                        retval = cmtw_check_output_compare_param(pmode_param->output_compare);
                        if (CMT_SUCCESS == retval)
                        {
                            cmtw_set_output_compare_param(channel, pmode_param->output_compare);
                        }
                    }
                }
            break;

            case CMT_CMD_SET_PAUSE :
                if (CMT_OPEN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_STOP;
                }
                else
                {
                    cmtw_count_stop_hw(channel);
                    gb_cmtw_info[channel].state = CMT_PAUSE;
                }
            break;

            case CMT_CMD_SET_RESUME :
                if (CMT_OPEN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_STOP;
                }
                else if (CMT_RUN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_RUNNING;
                }
                else
                {
                    gb_cmtw_info[channel].state = CMT_RUN;
                    cmtw_count_start_hw(channel);
                }
            break;

            case CMT_CMD_SET_RESTART :
                if (CMT_OPEN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_STOP;
                }
                else if (CMT_RUN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_RUNNING;
                }
                else
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCNT = 0UL;
                    gb_cmtw_info[channel].state = CMT_RUN;
                    cmtw_count_start_hw(channel);
                }
            break;

            case CMT_CMD_SET_ECM :

                /* check wheter CMTW status is open */
                if (CMT_RUN == gb_cmtw_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_RUNNING;
                }
                else
                {
                    retval = cmtw_set_ecm_param(channel, (cmtw_ecm_t *)p_data);
                }
            break;

            case CMT_CMD_GET_STATUS :

                /* casting the void pointer to a uint32_t type is valid structure,
                   because it is used in order to return value of CMT_STATUS_RUNNING and
                   CMTW_STATUS_STOP is a member of uint32_t */
                pcmt_status = (uint32_t *)p_data;

                /* check NULL pointer for reporting status */
                if (NULL == pcmt_status)
                {
                    retval = CMT_ERR_MISSING_PTR;
                }
                else
                {
                    if (CMT_RUN == gb_cmtw_info[channel].state)
                    {
                        /* set timer running status */
                        *pcmt_status = CMT_STATUS_RUNNING;
                    }
                    else
                    {
                        /* set timer stop status */
                        *pcmt_status = CMT_STATUS_STOP;
                    }
                }
            break;

            default:
                retval = CMT_ERR_INVALID_ARG;
            break;
        }
    }

    return retval;
}
/*******************************************************************************
End of function R_CMTW_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_init
* Description  : Initialize CMTW HW and driver information.
*
* Arguments    : channel
*                    CMTW channel number.
* Return Value : None.
*******************************************************************************/
static void cmtw_init(const uint32_t channel)
{
    uint32_t input_capture_num_member;
    uint32_t output_compare_num_member;

    /* initialize CMTW regsiters every channel */
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCNT = (uint32_t)0U;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWSTR.WORD  = (uint16_t)CMTW_CMWSTR_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.WORD   = (uint16_t)CMTW_CMWCR_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.WORD  = (uint16_t)CMTW_CMWIOR_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCNT       = (uint32_t)CMTW_CMWCNT_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCOR       = (uint32_t)CMTW_CMWCOR_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR0      = (uint32_t)CMTW_CMWOCR0_INIT;
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR1      = (uint32_t)CMTW_CMWOCR1_INIT;

    /* initialize common CMTW register */
    if (CMTW_CH_0 == channel)
    {
        CMTW.NFCR0.LONG = (uint32_t)CMTW_NFCR0_INIT;

        /* init ECDMESLR register when all channel is closed */
        if (CMT_CLOSE == gb_cmtw_info[CMTW_CH_1].state)
        {
            CMTW.ECDMESLR.LONG = (uint32_t)CMTW_ECDMESLR_INIT;
        }
    }
    else
    {
        CMTW.NFCR1.LONG = (uint32_t)CMTW_NFCR1_INIT;

        /* init ECDMESLR register when all channel is closed */
        if (CMT_CLOSE == gb_cmtw_info[CMTW_CH_0].state)
        {
            CMTW.ECDMESLR.LONG = (uint32_t)CMTW_ECDMESLR_INIT;
        }
    }

    /* init CMTW infomartion structure */
    gb_cmtw_info[channel].operate_mode        = CMT_PERIODIC;
    gb_cmtw_info[channel].clear_factor             = CMTW_CNT_CLEAR_COMPARE_MATCH;
    gb_cmtw_info[channel].compare_match_enable     = false;
    gb_cmtw_info[channel].p_compare_match_callback = NULL;

    for (input_capture_num_member = CMTW_INPUT_CAPTURE_NUM_0; input_capture_num_member < CMTW_INPUT_CAPTURE_NUM; input_capture_num_member++)
    {
        gb_cmtw_info[channel].input_capture_enable[input_capture_num_member] = false;
        gb_cmtw_info[channel].p_input_capture_callback[input_capture_num_member] = NULL;
    }

    for (output_compare_num_member = CMTW_OUTPUT_COMPARE_NUM_0; output_compare_num_member < CMTW_OUTPUT_COMPARE_NUM; output_compare_num_member++)
    {
        gb_cmtw_info[channel].output_compare_enable[output_compare_num_member] = false;
        gb_cmtw_info[channel].p_output_compare_callback[output_compare_num_member] = NULL;
    }

    return ;
}
/*******************************************************************************
End of function cmtw_init
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_check_param
* Description  : Check all CMTW configure parameters.
*
* Arguments    : p_cfg-
*                    CMTW configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_check_param(const cmtw_cfg_t * const p_cfg)
{
    cmt_err_t retval;

    if (NULL == p_cfg)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        retval = cmtw_check_count_param(&p_cfg->time_cnt_param);

        if (CMT_SUCCESS == retval)
        {
            retval = cmtw_check_compare_match_param(&p_cfg->mode_param.compare_match);
        }

        if (CMT_SUCCESS == retval)
        {
            retval = cmtw_check_input_capture_param(p_cfg->mode_param.input_capture, p_cfg->mode_param.noise_filter_clk);
        }

        if (CMT_SUCCESS == retval)
        {
            retval = cmtw_check_output_compare_param(p_cfg->mode_param.output_compare);
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_check_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_check_count_param
* Description  : Check CMTW counter configure parameters.
*
* Arguments    : p_compare_match_param-
*                    CMTW counter configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_check_count_param(const cmtw_time_cnt_t * const p_time_cnt_param)
{
    cmt_err_t retval = CMT_SUCCESS;

    if (NULL == p_time_cnt_param)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        if (CMT_PCLK_DIV_MAX_OVER <= p_time_cnt_param->pclk_div)
        {
            retval = CMT_ERR_INVALID_ARG;
        }

        if (CMT_SUCCESS == retval)
        {
            if (CMTW_CNT_SIZE_MAX_OVER <= p_time_cnt_param->cnt_size)
            {
                retval = CMT_ERR_INVALID_ARG;
            }
        }

        if (CMT_SUCCESS == retval)
        {
            if (CMTW_CNT_CLEAR_MAX_OVER <= p_time_cnt_param->clear_factor)
            {
                retval = CMT_ERR_INVALID_ARG;
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_check_count_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_check_compare_match_param
* Description  : Check CMTW compare match configure parameters.
*
* Arguments    : p_compare_match_param-
*                    CMTW compare match configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_check_compare_match_param(const cmtw_compare_match_t * const p_compare_match_param)
{
    cmt_err_t retval = CMT_SUCCESS;

    if (NULL == p_compare_match_param)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        if (false != p_compare_match_param->mode_enable)
        {
            if (VIC_PRIORITY_15 < p_compare_match_param->intr_priority)
            {
                retval = CMT_ERR_INVALID_ARG;
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_check_compare_match_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_check_input_capture_param
* Description  : Check CMTW input capture configure parameters.
*
* Arguments    : p_input_capture_param-
*                    CMTW input capture configure parameter
*                noise_filter_clk-
*                    Clock divide parameter for noise filter.
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_check_input_capture_param(const cmtw_input_capture_t * const p_input_capture_param, const uint32_t noise_filter_clk)
{
    cmt_err_t retval = CMT_SUCCESS;
    uint32_t input_capture_num_member;

    if (NULL == p_input_capture_param)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        for (input_capture_num_member = CMTW_INPUT_CAPTURE_NUM_0;
             ((input_capture_num_member < CMTW_INPUT_CAPTURE_NUM) && (CMT_SUCCESS == retval)); input_capture_num_member++)
        {
            if (false != p_input_capture_param[input_capture_num_member].mode_enable)
            {
                if (CMTW_IN_CAPTURE_TRIGGER_MAX_OVER <= p_input_capture_param[input_capture_num_member].trigger)
                {
                    retval = CMT_ERR_INVALID_ARG;
                }
                else
                {
                    if (VIC_PRIORITY_15 < p_input_capture_param[input_capture_num_member].intr_priority)
                    {
                        retval = CMT_ERR_INVALID_ARG;
                    }
                }
            }

            if (false != p_input_capture_param[input_capture_num_member].filter_enable)
            {
                if (CMT_SUCCESS == retval)
                {
                    if (CMTW_NOISE_FILTER_PCLK_DIV_MAX_OVER <= noise_filter_clk)
                    {
                        retval = CMT_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_check_input_capture_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_check_output_compare_param
* Description  : Check CMTW output compare configure parameters.
*
* Arguments    : p_output_compare_param-
*                    CMTW output compare configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_check_output_compare_param(const cmtw_output_compare_t * const p_output_compare_param)
{
    cmt_err_t retval = CMT_SUCCESS;
    uint32_t output_compare_num_member;

    if (NULL == p_output_compare_param)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        for (output_compare_num_member = CMTW_OUTPUT_COMPARE_NUM_0;
             ((output_compare_num_member < CMTW_OUTPUT_COMPARE_NUM) && (CMT_SUCCESS == retval)); output_compare_num_member++)
        {
            if (false != p_output_compare_param[output_compare_num_member].mode_enable)
            {
                if (CMTW_OUT_COMPARE_VALUE_MAX_OVER <= p_output_compare_param[output_compare_num_member].output_signal)
                {
                    retval = CMT_ERR_INVALID_ARG;
                }

                if (CMT_SUCCESS == retval)
                {
                    if (VIC_PRIORITY_15 < p_output_compare_param[output_compare_num_member].intr_priority)
                    {
                        retval = CMT_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_check_output_compare_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_setup
* Description  : Setup all CMTW configure parameters.
*
* Arguments    : p_cfg-
*                    CMTW configure parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static void cmtw_setup(const uint32_t channel, const cmtw_cfg_t * const p_cfg)
{
    if (NULL == p_cfg)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        cmtw_set_count_param(channel, &p_cfg->time_cnt_param);
        cmtw_set_compare_match_param(channel, &p_cfg->mode_param.compare_match);
        cmtw_set_input_capture_param(channel, p_cfg->mode_param.input_capture, p_cfg->mode_param.noise_filter_clk);
        cmtw_set_output_compare_param(channel, p_cfg->mode_param.output_compare);
    }

    return;
}
/*******************************************************************************
End of function cmtw_setup
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_set_count_param
* Description  : Setting CMTW counter configure parameters.
*
* Arguments    : channel-
*                    CMTW channel number.
*                p_compare_match_param-
*                    CMTW counter configure parameter.
* Return Value : None.
*******************************************************************************/
static void cmtw_set_count_param(const uint32_t channel, const cmtw_time_cnt_t * const p_time_cnt_param)
{
    if (NULL == p_time_cnt_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
    /* Configure CMTW timer
       CMWCR - CMTW timer control register
       b15:b13 CCLR - Counter clear bit - set user request value
       b9      CMS - Timer counter size bit - set user request valuet
       b1:b0   CKS - Clock select bit - set user request value */
        gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CKS = p_time_cnt_param->pclk_div;
        gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CMS = p_time_cnt_param->cnt_size;

        switch (p_time_cnt_param->clear_factor)
        {
            case CMTW_CNT_CLEAR_COMPARE_MATCH :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_CMWCOR;

            break;

            case CMTW_CNT_CLEAR_NONE :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_NONE;

            break;

            case CMTW_CNT_CLEAR_INPUT_CAPTURE0 :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_CMWICR0;

            break;

            case CMTW_CNT_CLEAR_INPUT_CAPTURE1 :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_CMWICR1;

            break;

            case CMTW_CNT_CLEAR_OUTPUT_COMPARE0 :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_CMWOCR0;

            break;

            case CMTW_CNT_CLEAR_OUTPUT_COMPARE1 :

                gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCR.BIT.CCLR = (uint16_t)CMTW_CMWCR_CCLR_SET_CMWOCR1;

            break;

            default :

                /* NOTREACHED on At the time of a normal performance */

            break;
        }

        /* copy parameter to CMTW driver information */
        gb_cmtw_info[channel].clear_factor = p_time_cnt_param->clear_factor;
    }

    return;
}
/*******************************************************************************
End of function cmtw_set_count_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_set_compare_match_param
* Description  : Set CMTW compare match configure parameters.
*
* Arguments    : channel-
*                    CMTW channel number.
*                p_compare_match_param-
*                    CMTW compare match configure parameter.
* Return Value : None.
*******************************************************************************/
static void cmtw_set_compare_match_param(const uint32_t channel, const cmtw_compare_match_t * const p_compare_match_param)
{
    if (NULL == p_compare_match_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        /* Configure compare match counter
           CMWCOR - CMTW timer value of compare match register
           b31:b0   compare match count value - on enable : set user request value
                                                on disable : set HW initvalue(0xFFFFFFFF) */
        /* Configure CMTW timer I/O setting
           CMWIOR - CMTW timer I/O control register
           b15 CMWE - Compare match enable bit - on enable : Write 1 to compare match enable
                                                 on disable : Write 0 to compare match edisable */
        if (false == p_compare_match_param->mode_enable)
        {
            /* clear timer parameter */
            gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCOR = (uint32_t)CMTW_CMWCOR_INIT;
            gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.CMWE = (uint16_t)0U;
            gb_cmtw_info[channel].p_compare_match_callback = NULL;
        }
        else
        {
            /* set timer parameter */
            gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCOR = p_compare_match_param->compare_match_cnt;
            gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.CMWE = (uint16_t)1U;
            gb_cmtw_info[channel].p_compare_match_callback = p_compare_match_param->p_callback;

            cmtw_intr_register(channel, CMTW_INTR_COMPARE_MATCH, p_compare_match_param->intr_priority);
        }

        gb_cmtw_info[channel].compare_match_enable = p_compare_match_param->mode_enable;
    }

    return;
}
/*******************************************************************************
End of function cmtw_set_compare_match_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_set_input_capture_param
* Description  : Set CMTW input capture configure parameters.
*
* Arguments    : channel-
*                    CMTW channel number.
*                p_input_capture_param-
*                    CMTW input capture configure parameter.
*               noise_filter_clk-
*                    Clock divide parameter for noise filter.
* Return Value : None.
*******************************************************************************/
static void cmtw_set_input_capture_param(const uint32_t channel, const cmtw_input_capture_t * const p_input_capture_param, const uint32_t noise_filter_clk)
{
    uint32_t input_capture_num_member;

    if (NULL == p_input_capture_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        for (input_capture_num_member = CMTW_INPUT_CAPTURE_NUM_0; input_capture_num_member < CMTW_INPUT_CAPTURE_NUM; input_capture_num_member++)
        {
            if (false == p_input_capture_param[input_capture_num_member].mode_enable)
            {
                /* clear input capture parameter */
                /* Configure CMTW timer I/O setting
                   CMWIOR - CMTW timer I/O control register
                   b1:b0 IC0 - Input capture control0 bit - Write 0 to HW init value
                   b3:b2 IC0 - Input capture control1 bit - Write 0 to HW init value
                   b4 IC0E - Input capture enable0 bit - Write 0 to input capture disable
                   b5 IC1E - Input capture enable1 bit - Write 0 to input capture disable */
                if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC0 = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC0E = (uint16_t)0U;
                }
                else
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC1 = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC1E = (uint16_t)0U;
                }
                gb_cmtw_info[channel].p_input_capture_callback[input_capture_num_member] = NULL;

                if (CMTW_CH_0 == channel)
                {
                    /* Configure digital noise filter of input capture 0 setting
                       NFCR0 - digital nosie filter of input capture0 control register
                       b0 NF0EN - Digital noise filter enbale0 bit - Write 0 to noise filter disable
                       b1 NF1EN - Digital noise filter enbale1 bit - Write 0 to noise filter disable */
                    if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                    {
                        CMTW.NFCR0.BIT.NF0EN = (uint16_t)0U;
                    }
                    else
                    {
                        CMTW.NFCR0.BIT.NF1EN = (uint16_t)0U;
                    }
                }
                else
                {
                    /* Configure digital noise filter of input capture 1 setting
                       NFCR1 - digital nosie filter of input capture0 control register
                       b0 NF2EN - Digital noise filter enbale2 bit - Write 0 to noise filter disable
                       b1 NF3EN - Digital noise filter enbale3 bit - Write 0 to noise filter disable */
                    if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                    {
                        CMTW.NFCR1.BIT.NF2EN = (uint16_t)0U;
                    }
                    else
                    {
                        CMTW.NFCR1.BIT.NF3EN = (uint16_t)0U;
                    }
                }
            }
            else
            {
                /* set input capture parameter */
                /* Configure CMTW timer I/O setting
                   CMWIOR - CMTW timer I/O control register
                   b1:b0 IC0 - Input capture control0 bit - set user request value
                   b3:b2 IC0 - Input capture control1 bit - set user request value
                   b4 IC0E - Input capture enable0 bit - Write 1 to input capture enable
                   b5 IC1E - Input capture enable1 bit - Write 1 to input capture enable */
                if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC0 = p_input_capture_param[input_capture_num_member].trigger;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC0E = (uint16_t)1U;
                    cmtw_intr_register(channel, CMTW_INTR_INPUT_CAPTURE0, p_input_capture_param[input_capture_num_member].intr_priority);
                }
                else
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC1 = p_input_capture_param[input_capture_num_member].trigger;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.IC1E = (uint16_t)1U;
                    cmtw_intr_register(channel, CMTW_INTR_INPUT_CAPTURE1, p_input_capture_param[input_capture_num_member].intr_priority);
                }
                gb_cmtw_info[channel].p_input_capture_callback[input_capture_num_member] = p_input_capture_param[input_capture_num_member].p_callback;

                if (false != p_input_capture_param[input_capture_num_member].filter_enable)
                {
                    if (CMTW_CH_0 == channel)
                    {
                        /* Configure digital noise filter of input capture 0 setting
                           NFCR0 - digital nosie filter of input capture0 control register
                           b0 NF0EN - Digital noise filter enbale0 bit - Write 1 to noise filter enable
                           b1 NF1EN - Digital noise filter enbale1 bit - Write 1 to noise filter enable
                           b3:b2 NFCS0 - Digital noise filter clock select0 bit - set user request value */
                        if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                        {
                            CMTW.NFCR0.BIT.NF0EN = (uint16_t)1U;
                        }
                        else
                        {
                            CMTW.NFCR0.BIT.NF1EN = (uint16_t)1U;
                        }
                        CMTW.NFCR0.BIT.NFCS0 = (uint32_t)noise_filter_clk;
                    }
                    else
                    {
                        /* Configure digital noise filter of input capture 1 setting
                           NFCR1 - digital nosie filter of input capture1 control register
                           b0 NF2EN - Digital noise filter enbale0 bit - Write 1 to noise filter enable
                           b1 NF3EN - Digital noise filter enbale1 bit - Write 1 to noise filter enable
                           b3:b2 NFCS1 - Digital noise filter clock select1 bit - set user request value */
                        if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                        {
                            CMTW.NFCR1.BIT.NF2EN = (uint16_t)1U;
                        }
                        else
                        {
                            CMTW.NFCR1.BIT.NF3EN = (uint16_t)1U;
                        }
                        CMTW.NFCR1.BIT.NFCS1 = (uint32_t)noise_filter_clk;
                    }
                }
                else
                {
                    if (CMTW_CH_0 == channel)
                    {
                        /* Configure digital noise filter of input capture 1 setting
                           NFCR1 - digital nosie filter of input capture0 control register
                           b0 NF2EN - Digital noise filter enbale2 bit - Write 0 to noise filter disable
                           b1 NF3EN - Digital noise filter enbale3 bit - Write 0 to noise filter disable */
                        if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                        {
                            CMTW.NFCR0.BIT.NF0EN = (uint16_t)0U;
                        }
                        else
                        {
                            CMTW.NFCR0.BIT.NF1EN = (uint16_t)0U;
                        }
                    }
                    else
                    {
                        /* Configure digital noise filter of input capture 1 setting
                           NFCR1 - digital nosie filter of input capture0 control register
                           b0 NF2EN - Digital noise filter enbale2 bit - Write 0 to noise filter disable
                           b1 NF3EN - Digital noise filter enbale3 bit - Write 0 to noise filter disable */
                        if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
                        {
                            CMTW.NFCR1.BIT.NF2EN = (uint16_t)0U;
                        }
                        else
                        {
                            CMTW.NFCR1.BIT.NF3EN = (uint16_t)0U;
                        }
                    }
                }
            }

            gb_cmtw_info[channel].input_capture_enable[input_capture_num_member] = p_input_capture_param[input_capture_num_member].mode_enable;
        }
    }

    return;
}
/*******************************************************************************
End of function cmtw_set_input_capture_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_set_output_compare_param
* Description  : Set CMTW output compare configure parameters.
*
* Arguments    : channel-
*                    CMTW channel number.
*                p_output_compare_param-
*                    CMTW output compare parameter.
* Return Value : None.
*******************************************************************************/
static void cmtw_set_output_compare_param(const uint32_t channel, const cmtw_output_compare_t * const p_output_compare_param)
{
    uint32_t output_compare_num_member;

    if (NULL == p_output_compare_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        for (output_compare_num_member = CMTW_OUTPUT_COMPARE_NUM_0; output_compare_num_member < CMTW_OUTPUT_COMPARE_NUM; output_compare_num_member++)
        {
            if (false == p_output_compare_param[output_compare_num_member].mode_enable)
            {
               /* Configure CMTW timer I/O setting
                   CMWIOR - CMTW timer I/O control register
                   b9:b8   OC0 - Input capture control0 bit - Write 0 to HW init value
                   b11:b10 OC1 - Input capture control1 bit - Write 0 to HW init value
                   b12 OC0E - Input capture enable0 bit - Write 0 to output compare disable
                   b13 OC1E - Input capture enable1 bit - Write 0 to output compare disable */
                /* Configure output compar counter
                   CMWCOR - CMTW timer value of output compare register
                   b31:b0   output compare count value - set HW initvalue(0xFFFFFFFF) */
                /* clear output compare parameter */
                if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num_member)
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC0 = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC0E = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR0 = (uint32_t)CMTW_CMWOCR0_INIT;
                }
                else
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC1 = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC1E = (uint16_t)0U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR1 = (uint32_t)CMTW_CMWOCR1_INIT;
                }
                gb_cmtw_info[channel].p_output_compare_callback[output_compare_num_member] = NULL;
            }
            else
            {
                /* set output compare parameter */
                /* Configure CMTW timer I/O setting
                   CMWIOR - CMTW timer I/O control register
                   b9:b8   OC0 - Output compare control0 bit - set user request value
                   b11:b10 OC1 - Output compare control1 bit - set user request value
                   b12 OC0E - Output compare enable0 bit - Write 1 to output compare enable
                   b13 OC1E - Output compare enable1 bit - Write 1 to output compare enable */
                /* Configure output compar counter
                   CMWCOR - CMTW timer value of output compare register
                   b31:b0   output compare count value - set user request value */
                if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num_member)
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC0
                        = p_output_compare_param[output_compare_num_member].output_signal;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC0E = (uint16_t)1U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR0
                        =  p_output_compare_param[output_compare_num_member].output_compare_cnt;
                    cmtw_intr_register(channel, CMTW_INTR_OUTPUT_COMPARE0, p_output_compare_param[output_compare_num_member].intr_priority);
                }
                else
                {
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC1
                        = p_output_compare_param[output_compare_num_member].output_signal;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWIOR.BIT.OC1E = (uint16_t)1U;
                    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWOCR1
                        =  p_output_compare_param[output_compare_num_member].output_compare_cnt;
                    cmtw_intr_register(channel, CMTW_INTR_OUTPUT_COMPARE1, p_output_compare_param[output_compare_num_member].intr_priority);
                }
                gb_cmtw_info[channel].p_output_compare_callback[output_compare_num_member]
                    = p_output_compare_param[output_compare_num_member].p_callback;
            }

            gb_cmtw_info[channel].output_compare_enable[output_compare_num_member]
                = p_output_compare_param[output_compare_num_member].mode_enable;
        }
    }

    return;
}
/*******************************************************************************
End of function cmtw_set_output_compare_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_set_ecm_param
* Description  : Setting ECM parameter for output compare.
*
* Arguments    : channel-
*                    CMTW channel number.
*                p_ecm_param-
*                    ECM parameter.
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
static cmt_err_t cmtw_set_ecm_param(const uint32_t channel, const cmtw_ecm_t * const p_ecm_param)
{
    cmt_err_t retval = CMT_SUCCESS;

    if (NULL == p_ecm_param)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        if (false != p_ecm_param->ecm_enable)
        {
            if (CMTW_OUTPUT_COMPARE_NUM <= p_ecm_param->output_compare_num)
            {
                retval = CMT_ERR_INVALID_ARG;
            }
        }

        if (CMT_SUCCESS == retval)
        {
            /* Configure ECM dynamic mode error output
               ECDMESLR - ECM dynamic mode error poutput select register
               b2:b0 DMERSL - ECM dynamic mode error output select bit
                                 - on disable : write 0 to ECM dynamic mode error disable
                                   on enable : set user request value */
            if (false == p_ecm_param->ecm_enable)
            {
                /* clear ECM parameter */
                CMTW.ECDMESLR.BIT.DMERSL = (uint32_t)CMTW_ECDMESLR_DMERSL_SET_NONE;
            }
            else
            {
                /* set ECM parameter */
                if (CMTW_CH_0 == channel)
                {
                    if (CMTW_OUTPUT_COMPARE_NUM_0 == p_ecm_param->output_compare_num)
                    {
                        CMTW.ECDMESLR.BIT.DMERSL = (uint32_t)CMTW_ECDMESLR_DMERSL_SET_CMTW0_OUT0;
                    }
                    else
                    {
                        CMTW.ECDMESLR.BIT.DMERSL = (uint32_t)CMTW_ECDMESLR_DMERSL_SET_CMTW0_OUT1;
                    }
                }
                else
                {
                    if (CMTW_OUTPUT_COMPARE_NUM_0 == p_ecm_param->output_compare_num)
                    {
                        CMTW.ECDMESLR.BIT.DMERSL = (uint32_t)CMTW_ECDMESLR_DMERSL_SET_CMTW1_OUT0;
                    }
                    else
                    {
                        CMTW.ECDMESLR.BIT.DMERSL = (uint32_t)CMTW_ECDMESLR_DMERSL_SET_CMTW1_OUT1;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmtw_set_ecm_param
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_count_start
* Description  : Start timer counter.
*
* Arguments    : channel-
*                    CMTW channel number.
*                operate_mode-
*                    Timer operation mode is periodic or one shot.
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_NOT_OPENED-
*                    Function is called when status is not closed.
*                CMT_ERR_TIMER_RUNNING-
*                    Function is called while running counter.
*******************************************************************************/
static cmt_err_t cmtw_count_start(const uint32_t channel, const uint32_t operate_mode)
{
    cmt_err_t retval = CMT_SUCCESS;

    /* check channel */
    if (channel >= CMTW_CH_NUM)
    {
        retval = CMT_ERR_INVALID_CH;
    }

    if (CMT_SUCCESS == retval)
    {
        /* check wheter CMTW status is open */
        if (CMT_CLOSE == gb_cmtw_info[channel].state)
        {
            retval = CMT_ERR_NOT_OPENED;
        }

        /* check wheter CMTW status is timer stop */
        if (CMT_OPEN != gb_cmtw_info[channel].state)
        {
            retval = CMT_ERR_TIMER_RUNNING;
        }
    }

    if (CMT_SUCCESS == retval)
    {
        gb_cmtw_info[channel].p_cmtw_ch_reg->CMWCNT = 0;
        gb_cmtw_info[channel].operate_mode          = operate_mode;
        gb_cmtw_info[channel].state                 = CMT_RUN;

        cmtw_count_start_hw(channel);
    }

    return retval;
}

/*******************************************************************************
End of function cmtw_count_start
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_count_start_hw
* Description  : Setting CMTW hw to timer counter start.
*
* Arguments    : channel-
*                    CMTW channel number.
* Return Value : None.
*******************************************************************************/
static void cmtw_count_start_hw(const uint32_t channel)
{
    uint32_t input_capture_num_member;
    uint32_t output_compare_num_member;

    if (false != gb_cmtw_info[channel].compare_match_enable)
    {
        /* clear interrupt mask of compare match interrupt */
        cmtw_intr_enable(channel, CMTW_INTR_COMPARE_MATCH);
    }

    for (input_capture_num_member = CMTW_INPUT_CAPTURE_NUM_0; input_capture_num_member < CMTW_INPUT_CAPTURE_NUM; input_capture_num_member++)
    {
        if (false != gb_cmtw_info[channel].input_capture_enable[input_capture_num_member])
        {
            /* clear interrupt mask of input capture interrupt */
            if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
            {
                cmtw_intr_enable(channel, CMTW_INTR_INPUT_CAPTURE0);
            }
            else
            {
                cmtw_intr_enable(channel, CMTW_INTR_INPUT_CAPTURE1);
            }
        }
    }

    for (output_compare_num_member = CMTW_OUTPUT_COMPARE_NUM_0; output_compare_num_member < CMTW_OUTPUT_COMPARE_NUM; output_compare_num_member++)
    {
        if (false != gb_cmtw_info[channel].output_compare_enable[output_compare_num_member])
        {
            /* clear interrupt mask of output compare interrupt */
            if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num_member)
            {
                cmtw_intr_enable(channel, CMTW_INTR_OUTPUT_COMPARE0);
            }
            else
            {
                cmtw_intr_enable(channel, CMTW_INTR_OUTPUT_COMPARE1);
            }
        }
    }

    /* Start CMTW timer
       CMWSTR - Timer start register
       b0 STR - Write 1 to timer count start  */
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWSTR.BIT.STR = (uint16_t)1U;

    return;
}

/*******************************************************************************
End of function cmtw_count_start_hw
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_count_stop_hw
* Description  : Setting CMTW hw to timer counter stop.
*
* Arguments    : channel-
*                    CMTW channel number.
* Return Value : None.
*******************************************************************************/
void cmtw_count_stop_hw(const uint32_t channel)
{
    uint32_t input_capture_num_member;
    uint32_t output_compare_num_member;

    /* Start CMTW timer
       CMWSTR - Timer start register
       b0 STR - Write 0 to timer count stop  */
    gb_cmtw_info[channel].p_cmtw_ch_reg->CMWSTR.BIT.STR = (uint16_t)0U;

    cmtw_intr_disable(channel, CMTW_INTR_COMPARE_MATCH);

    for (input_capture_num_member = CMTW_INPUT_CAPTURE_NUM_0; input_capture_num_member < CMTW_INPUT_CAPTURE_NUM; input_capture_num_member++)
    {
        if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num_member)
        {
            cmtw_intr_disable(channel, CMTW_INTR_INPUT_CAPTURE0);
        }
        else
        {
            cmtw_intr_disable(channel, CMTW_INTR_INPUT_CAPTURE1);
        }
    }

    for (output_compare_num_member = CMTW_OUTPUT_COMPARE_NUM_0; output_compare_num_member < CMTW_OUTPUT_COMPARE_NUM; output_compare_num_member++)
    {
        if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num_member)
        {
            cmtw_intr_disable(channel, CMTW_INTR_OUTPUT_COMPARE0);
        }
        else
        {
            cmtw_intr_disable(channel, CMTW_INTR_OUTPUT_COMPARE1);
        }
    }

    return;
}

/*******************************************************************************
End of function cmtw_count_stop_hw
*******************************************************************************/

/* End of File */

