/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmtw driver interrupt functions for cmtw fit module.
* File Name    : r_cmtw_int.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMTW on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_cmt.h"
#include "r_cmtw_int.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/


/*******************************************************************************
Private global variables and functions
*******************************************************************************/

static void cmtw0_cmwi_isr(void);
static void cmtw1_cmwi_isr(void);
static void cmtw0_ic0i_isr ( void );
static void cmtw0_ic1i_isr ( void );
static void cmtw1_ic0i_isr ( void );
static void cmtw1_ic1i_isr ( void );
static void cmtw0_oc0i_isr ( void );
static void cmtw0_oc1i_isr ( void );
static void cmtw1_oc0i_isr ( void );
static void cmtw1_oc1i_isr ( void );

static void cmtw_cmwi_isr_common(const uint32_t channel);
static void cmtw_ic_isr_common(const uint32_t channel, const uint32_t input_capture_num);
static void cmtw_oc_isr_common(const uint32_t channel, const uint32_t output_compare_num);

static const cmtw_intr_t gb_cmtw_intr_table[CMTW_CH_NUM][CMTW_INTR_TYPE_NUM] =
{
    {
        /* casting the interrupt handler pointer to a uint32_t type is valid structure,
           because interrupr handlel address is uint32_t value.
           And, value is passed to R_ICU_Regist() which is required the value of
           uint32_t type value  */
        {CMWI0_IRQn,  VIC_TYPE_EDGE, (void *)cmtw0_cmwi_isr, CMTW_CMWCR_CMWIE_SET},
        {IC0I0_IRQn,  VIC_TYPE_EDGE, (void *)cmtw0_ic0i_isr, CMTW_CMWCR_IC0IE_SET},
        {IC1I0_IRQn,  VIC_TYPE_EDGE, (void *)cmtw0_ic1i_isr, CMTW_CMWCR_IC1IE_SET},
        {OC0I0_IRQn,  VIC_TYPE_EDGE, (void *)cmtw0_oc0i_isr, CMTW_CMWCR_OC0IE_SET},
        {OC1I0_IRQn,  VIC_TYPE_EDGE, (void *)cmtw0_oc1i_isr, CMTW_CMWCR_OC1IE_SET}
    },
    {
        /* casting the interrupt handler pointer to a uint32_t type is valid structure,
           because interrupr handlel address is uint32_t value.
           And, value is passed to R_ICU_Regist() which is required the value of
           uint32_t type value  */
        {CMWI1_IRQn,  VIC_TYPE_EDGE, (void *)cmtw1_cmwi_isr, CMTW_CMWCR_CMWIE_SET},
        {IC0I1_IRQn,  VIC_TYPE_EDGE, (void *)cmtw1_ic0i_isr, CMTW_CMWCR_IC0IE_SET},
        {IC1I1_IRQn,  VIC_TYPE_EDGE, (void *)cmtw1_ic1i_isr, CMTW_CMWCR_IC1IE_SET},
        {OC0I1_IRQn,  VIC_TYPE_EDGE, (void *)cmtw1_oc0i_isr, CMTW_CMWCR_OC0IE_SET},
        {OC1I1_IRQn,  VIC_TYPE_EDGE, (void *)cmtw1_oc1i_isr, CMTW_CMWCR_OC1IE_SET}
    }
};

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_intr_register
* Description  : Registration CMTW interrupt information.
*
* Arguments    : channel-
*                    CMTW channel number.
*                cmtw_intr_num-
*                    CMTW interrupt factor number.
*                intr_priority-
*                    CMTW interrupt priority.
* Return Value : None.
*******************************************************************************/
void cmtw_intr_register(const uint32_t channel, const uint32_t cmtw_intr_num, const int32_t intr_priority)
{
    InterruptHandlerRegister(gb_cmtw_intr_table[channel][cmtw_intr_num].vic_irq,
                             (IRQHandler)gb_cmtw_intr_table[channel][cmtw_intr_num].intr_handler);
    VIC_SetPriority(gb_cmtw_intr_table[channel][cmtw_intr_num].vic_irq, intr_priority);
    VIC_SetDetectType(gb_cmtw_intr_table[channel][cmtw_intr_num].vic_irq,
                      gb_cmtw_intr_table[channel][cmtw_intr_num].type);
    return;
}

/*******************************************************************************
End of function cmtw_intr_register
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_intr_enable
* Description  : Enable CMTW interrupt.
*
* Arguments    : channel-
*                    CMTW channel number.
*                cmtw_intr_num-
*                    CMTW interrupt factor number.
* Return Value : None.
*******************************************************************************/
void cmtw_intr_enable(const uint32_t channel, const uint32_t cmtw_intr_num)
{
    cmtw_info_ch_t * const pcmtw_info_ch = &gb_cmtw_info[channel];

    if (NULL == pcmtw_info_ch)
    {
        /* NON_NOTICE_ASSERT: pointer is certainly valid pointer, QAC mesures */
    }
    else
    {
        /* Configure CMTW timer
           CMWCR - CMTW timer control register
           Select setting configure bit accoring cmtw_intr_num value
           b3 CMWIE - Write 1 to compare match interrupt enable
           b4 IC0IE - Write 1 to input capture0 interrupt enable
           b5 IC1IE - Write 1 to input capture1 interrupt enable
           b6 OC0IE - Write 1 to output compare0 interrupt enable
           b7 OC1IE - Write 1 to output compare1 interrupt enable
           b1:b0   CKS - Clock select bit - set user request value */
        pcmtw_info_ch->p_cmtw_ch_reg->CMWCR.WORD |= gb_cmtw_intr_table[channel][cmtw_intr_num].cmwcr_mask_clear_val;

        VIC_EnableIRQ(gb_cmtw_intr_table[channel][cmtw_intr_num].vic_irq);
    }

    return;
}
/*******************************************************************************
End of function cmtw_intr_enable
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_intr_disable
* Description  : Disable CMTW interrupt.
*
* Arguments    : channel-
*                    CMTW channel number.
*                cmtw_intr_num-
*                    CMTW interrupt factor number.
* Return Value : None.
*******************************************************************************/
void cmtw_intr_disable(const uint32_t channel, const uint32_t cmtw_intr_num)
{
    cmtw_info_ch_t * const pcmtw_info_ch = &gb_cmtw_info[channel];

    if (NULL == pcmtw_info_ch)
    {
        /* NON_NOTICE_ASSERT: pointer is certainly valid pointer, QAC mesures */
    }
    else
    {
        /* Configure CMTW timer
           CMWCR - CMTW timer control register
           Select setting configure bit accoring cmtw_intr_num value
           b3 CMWIE - Write 0 to compare match interrupt disable
           b4 IC0IE - Write 0 to input capture0 interrupt disable
           b5 IC1IE - Write 0 to input capture1 interrupt disable
           b6 OC0IE - Write 0 to output compare0 interrupt disable
           b7 OC1IE - Write 0 to output compare1 interrupt disable
           b1:b0   CKS - Clock select bit - set user request value */
        pcmtw_info_ch->p_cmtw_ch_reg->CMWCR.WORD &= (uint16_t)~(gb_cmtw_intr_table[channel][cmtw_intr_num].cmwcr_mask_clear_val);

        VIC_DisableIRQ(gb_cmtw_intr_table[channel][cmtw_intr_num].vic_irq);
    }

    return;
}
/*******************************************************************************
End of function cmtw_intr_disable
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_cmwi_isr_common
* Description  : Commom operation on compare match interrupt handler.
* Arguments    : channel-
*                    CMTW channel number.
* Return Value : none
*******************************************************************************/
static void cmtw_cmwi_isr_common(const uint32_t channel)
{
    cmtw_info_ch_t * const pcmtw_info_ch = &gb_cmtw_info[channel];

    if (NULL == pcmtw_info_ch)
    {
        /* NON_NOTICE_ASSERT: pointer is certainly valid pointer, QAC mesures */
    }
    else
    {
        /* check whether clear factor are fulfilled */
        if ((CMT_ONE_SHOT == pcmtw_info_ch->operate_mode) &&
            (CMTW_CNT_CLEAR_COMPARE_MATCH == pcmtw_info_ch->clear_factor))
        {
            /* stop timer */
            cmtw_count_stop_hw(channel);

            pcmtw_info_ch->state = CMT_OPEN;
        }

        if (NULL != pcmtw_info_ch->p_compare_match_callback)
        {
            /* call function if valid function pointer */
            pcmtw_info_ch->p_compare_match_callback();
        }
    }

    return;
}
/*******************************************************************************
End of function cmtw_cmwi_isr_common
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_ic_isr_common
* Description  : Commom operation on input capture interrupt handler.
* Arguments    : channel-
*                    CMTW channel number.
*                input_capture_num
*                    Input capture number.
* Return Value : none
*******************************************************************************/
static void cmtw_ic_isr_common(const uint32_t channel, const uint32_t input_capture_num)
{
    cmtw_info_ch_t * const pcmtw_info_ch = &gb_cmtw_info[channel];
    uint32_t check_clear_factor;
    uint32_t input_capture_value;

    if (NULL == pcmtw_info_ch)
    {
        /* NON_NOTICE_ASSERT: pointer is certainly valid pointer, QAC mesures */
    }
    else
    {
        if (CMTW_CH_0 == channel)
        {
            if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num)
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_INPUT_CAPTURE0;
                input_capture_value = pcmtw_info_ch->p_cmtw_ch_reg->CMWICR0;
            }
            else
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_INPUT_CAPTURE1;
                input_capture_value = pcmtw_info_ch->p_cmtw_ch_reg->CMWICR1;
            }
        }
        else
        {
            if (CMTW_INPUT_CAPTURE_NUM_0 == input_capture_num)
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_INPUT_CAPTURE0;
                input_capture_value = pcmtw_info_ch->p_cmtw_ch_reg->CMWICR0;
            }
            else
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_INPUT_CAPTURE1;
                input_capture_value = pcmtw_info_ch->p_cmtw_ch_reg->CMWICR1;
            }
        }

        /* check whether clear factor are fulfilled */
        if ((CMT_ONE_SHOT == pcmtw_info_ch->operate_mode) &&
            (check_clear_factor == pcmtw_info_ch->clear_factor))
        {
            /* stop time */
            cmtw_count_stop_hw(channel);

            pcmtw_info_ch->state = CMT_OPEN;
        }

        if (NULL != pcmtw_info_ch->p_input_capture_callback[input_capture_num])
        {
            /* call function if valid function pointer */
            pcmtw_info_ch->p_input_capture_callback[input_capture_num](input_capture_value);
        }

        /* End interrupt sequence */
    }

    return;
}
/*******************************************************************************
End of function cmtw_ic_isr_common
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw_oc_isr_common
* Description  : Commom operation on output compare interrupt handler.
* Arguments    : channel-
*                    CMTW channel number.
*                output_compare_num
*                    Output compare number.
* Return Value : none
*******************************************************************************/
static void cmtw_oc_isr_common(const uint32_t channel, const uint32_t output_compare_num)
{
    cmtw_info_ch_t * const pcmtw_info_ch = &gb_cmtw_info[channel];
    uint32_t check_clear_factor;

    if (NULL == pcmtw_info_ch)
    {
        /* NON_NOTICE_ASSERT: pointer is certainly valid pointer, QAC mesures */
    }
    else
    {
        if (CMTW_CH_0 == channel)
        {
            if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num)
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_OUTPUT_COMPARE0;
            }
            else
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_OUTPUT_COMPARE1;
            }
        }
        else
        {
            if (CMTW_OUTPUT_COMPARE_NUM_0 == output_compare_num)
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_OUTPUT_COMPARE0;
            }
            else
            {
                check_clear_factor = (uint32_t)CMTW_CNT_CLEAR_OUTPUT_COMPARE1;
            }
        }

        /* check whether clear factor are fulfilled */
        if ((CMT_ONE_SHOT == pcmtw_info_ch->operate_mode) &&
            (check_clear_factor == pcmtw_info_ch->clear_factor))
        {
            /* stop time */
            cmtw_count_stop_hw(channel);

            pcmtw_info_ch->state = CMT_OPEN;
        }

        if (NULL != pcmtw_info_ch->p_output_compare_callback[output_compare_num])
        {
            /* call function if valid function pointer */
            pcmtw_info_ch->p_output_compare_callback[output_compare_num]();
        }

    }

    return;
}
/*******************************************************************************
End of function cmtw_oc_isr_common
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw0_cmwi_isr
* Description  : Compare match interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw0_cmwi_isr(void)
{
    cmtw_cmwi_isr_common(CMTW_CH_0);

    return;
}
/*******************************************************************************
End of function cmtw0_cmwi_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw1_cmwi_isr
* Description  : Compare match interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw1_cmwi_isr(void)
{
    cmtw_cmwi_isr_common(CMTW_CH_1);

    return;
}
/*******************************************************************************
End of function cmtw1_cmwi_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw0_ic0i_isr
* Description  : CMTW0 input capture0 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw0_ic0i_isr(void)
{
    cmtw_ic_isr_common(CMTW_CH_0, CMTW_INPUT_CAPTURE_NUM_0);

    return;
}
/*******************************************************************************
End of function cmtw0_ic0i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw0_ic1i_isr
* Description  : CMTW0 input capture1 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw0_ic1i_isr(void)
{
    cmtw_ic_isr_common(CMTW_CH_0, CMTW_INPUT_CAPTURE_NUM_1);

    return;
}
/*******************************************************************************
End of function cmtw0_ic1i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw1_ic0i_isr
* Description  : CMTW1 input capture0 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw1_ic0i_isr(void)
{
    cmtw_ic_isr_common(CMTW_CH_1, CMTW_INPUT_CAPTURE_NUM_0);

    return;
}
/*******************************************************************************
End of function cmtw1_ic0i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw1_ic1i_isr
* Description  : CMTW1 input capture1 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw1_ic1i_isr(void)
{
    cmtw_ic_isr_common(CMTW_CH_1, CMTW_INPUT_CAPTURE_NUM_1);

    return;
}
/*******************************************************************************
End of function cmtw1_ic1i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw0_oc0i_isr
* Description  : CMTW0 output compare0 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw0_oc0i_isr(void)
{
    cmtw_oc_isr_common(CMTW_CH_0, CMTW_OUTPUT_COMPARE_NUM_0);

    return;
}
/*******************************************************************************
End of function cmtw0_oc0i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw0_oc1i_isr
* Description  : CMTW0 output compare1 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw0_oc1i_isr(void)
{
    cmtw_oc_isr_common(CMTW_CH_0, CMTW_OUTPUT_COMPARE_NUM_1);

    return;
}
/*******************************************************************************
End of function cmtw0_oc1i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw1_oc0i_isr
* Description  : CMTW1 output compare0 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw1_oc0i_isr(void)
{
    cmtw_oc_isr_common(CMTW_CH_1, CMTW_OUTPUT_COMPARE_NUM_0);

    return;
}
/*******************************************************************************
End of function cmtw1_oc0i_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: cmtw1_oc1i_isr
* Description  : CMTW1 output compare1 interrupt handler.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmtw1_oc1i_isr(void)
{
    cmtw_oc_isr_common(CMTW_CH_1, CMTW_OUTPUT_COMPARE_NUM_1);

    return;
}
/*******************************************************************************
End of function cmtw1_oc1i_isr
*******************************************************************************/

/* End of File */

