/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : elc driver functions for elc fit module.
* File Name    : r_elc.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for ELC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_elc.h"
#include "r_elc_int.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static void elc_init(void);
static elc_err_t elc_check_event_resource_param(const uint32_t event_resource_param);
static elc_err_t elc_check_mtu_param(const elc_cmd_mtu_t * const p_mtu_param);
static elc_err_t elc_check_cmt_param(const elc_cmd_cmt_t * const p_cmt_param);
static elc_err_t elc_check_dsmif_param(const elc_cmd_dsmif_t * const p_dsmif_param);
static elc_err_t elc_check_s12ad_param(const elc_cmd_s12ad_t * const p_s12ad_param);
static elc_err_t elc_check_intr_param(const elc_cmd_intr_t * const p_intr_param);
static elc_err_t elc_check_out_port_group_param(const elc_cmd_out_port_group_t * const p_out_port_group_param);
static elc_err_t elc_check_in_port_group_param(const elc_cmd_in_port_group_t * const p_in_port_group_param);
static elc_err_t elc_check_single_port_param(const elc_cmd_single_port_t * const p_single_port_param);
static elc_err_t elc_check_cmtw_param(const elc_cmd_cmtw_t * const p_cmtw_param);
static elc_err_t elc_check_tpu_param(const elc_cmd_tpu_t * const p_tpu_param);
static elc_err_t elc_check_gpt_param(const elc_cmd_gpt_t * const p_gpt_param);
static elc_err_t elc_check_port_group_param(const elc_cmd_port_group_t * const p_port_group_param);
static elc_err_t elc_check_port_group_value(const elc_get_port_value_t * const p_get_port_value);

static void elc_set_mtu_param(const elc_cmd_mtu_t * const p_mtu_param);
static void elc_set_cmt_param(const elc_cmd_cmt_t * const p_cmt_param);
static void elc_set_dsmif_param(const elc_cmd_dsmif_t * const p_dsmif_param);
static void elc_set_s12ad_param(const elc_cmd_s12ad_t * const p_s12ad_param);
static void elc_set_intr_param(const elc_cmd_intr_t * const p_intr_param);
static void elc_set_out_port_group_param(const elc_cmd_out_port_group_t * const p_out_port_group_param);
static void elc_set_in_port_group_param(const elc_cmd_in_port_group_t * const p_in_port_group_param);
static void elc_set_single_port_param(const elc_cmd_single_port_t * const p_single_port_param);
static void elc_set_cmtw_param(const elc_cmd_cmtw_t * const p_cmtw_param);
static void elc_set_tpu_param(const elc_cmd_tpu_t * const p_tpu_param);
static void elc_set_gpt_param(const elc_cmd_gpt_t * const p_gpt_param);
static void elc_set_port_group_param(const elc_cmd_port_group_t * const p_port_group_param);
static void elc_set_software_event(void);
static void elc_get_port_group_value(elc_get_port_value_t * const p_get_port_value);

/* ELC driver information */
elc_control_t elc_control = { ELC_UNINIT, 0, { NULL, NULL } };

/*******************************************************************************
* Function Name: ELC_Open
* Description  : This function does open ELC driver.
*                ELC HW, driver information is initialized and module stop
*                control is set to OFF.
* Arguments    : None.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_NOT_CLOSED-
*                    Open() is called when status is not closed.
*******************************************************************************/
elc_err_t ELC_Open(void)
{
    elc_err_t retval = ELC_SUCCESS;
    
    volatile uint32_t dummy32t;
    volatile uint8_t  dummy8t;

    if (ELC_CLOSE != elc_control.state)
    {
        retval = ELC_ERR_NOT_CLOSED;
    }
    else
    {
        R_RST_WriteEnable();
        MSTP(ELC) = (uint32_t)0U;
        dummy32t = SYSTEM.MSTPCRC.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
        R_RST_WriteDisable();

        dummy8t = ELC.ELOPC.BYTE; /* Workaround for lack of release procedure from the module-stop state(2/2) */

        elc_init();

        elc_control.state = ELC_OPEN;
    }

    return retval;
}
/*******************************************************************************
End of function ELC_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: ELC_Close
* Description  : This function does close ELC driver.
*                Module stop control is set to ON.
* Arguments    : None
* Return Value : ELC_SUCCESS-
*                    Successful
*******************************************************************************/
void ELC_Close(void)
{
    elc_err_t retval;

    retval = ELC_LinkStop();

    if (ELC_SUCCESS == retval)
    {
        /* ELC_LinkStop error is not opened only */
        R_RST_WriteEnable();
        MSTP(ELC) = (uint32_t)1U;
        R_RST_WriteDisable();

        elc_control.state = ELC_CLOSE;
    }

    return;
}
/*******************************************************************************
End of function ELC_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: ELC_LinkStart
* Description  : Start ELC event link operation start.
* Arguments    : None.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_NOT_OPENED-
*                    Function is called when status is not initialized.
*******************************************************************************/
elc_err_t ELC_LinkStart(void)
{
    elc_err_t retval = ELC_SUCCESS;

    if (ELC_CLOSE == elc_control.state)
    {
        retval = ELC_ERR_NOT_OPENED;
    }
    else
    {
        /* if status is not open, event link started already */
        if (ELC_OPEN == elc_control.state)
        {
            ELC.ELCR.BIT.ELCON = (uint8_t)1U;
            elc_control.state = ELC_RUN;
        }
    }

    return retval;
}
/*******************************************************************************
End of function ELC_LinkStart
*******************************************************************************/

/*******************************************************************************
* Function Name: ELC_LinkStop
* Description  : Stop ELC event link operation start.
* Arguments    : None.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_NOT_OPENED-
*                    Function is called when status is not initialized.
*******************************************************************************/
elc_err_t ELC_LinkStop(void)
{
    elc_err_t retval = ELC_SUCCESS;

    if (ELC_CLOSE == elc_control.state)
    {
        retval = ELC_ERR_NOT_OPENED;
    }
    else
    {
        /* if status is not running, event link stoped already */
        if (ELC_RUN == elc_control.state)
        {
            ELC.ELCR.BIT.ELCON = (uint8_t)0U;
            elc_control.state = ELC_OPEN;
        }
    }

    return retval;
}
/*******************************************************************************
End of function ELC_LinkStop
*******************************************************************************/

/*******************************************************************************
* Function Name: ELC_Control
* Description  : This function provides commands for changing ELC setting,
*                operating counter.
*
* Arguments    : cmd-
*                    Command to run
*                p_data-
*                    Pointer to optional configuration structure
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    ELC setting parameters is invalid.
*                ELC_ERR_NOT_OPENED-
*                    Function is called when status is not closed.
*                ELC_ERR_MISSING_PTR-
*                    ELC setting parameter is NULL pointer.
*******************************************************************************/
elc_err_t ELC_Control(uint32_t const cmd, void * const p_data)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check ELC status */
    if (ELC_CLOSE == elc_control.state)
    {
        retval = ELC_ERR_NOT_OPENED;
    }
    else
    {
        switch (cmd)
        {
        case ELC_CMD_SET_EVENT_MTU:

            /* casting the void pointer to a elc_cmd_mtu_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_mtu_param((elc_cmd_mtu_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_mtu_param((elc_cmd_mtu_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_CMT:

            /* casting the void pointer to a elc_cmd_cmt_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_cmt_param((elc_cmd_cmt_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_cmt_param((elc_cmd_cmt_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_DSMIF:

            /* casting the void pointer to a elc_cmd_dsmif_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_dsmif_param((elc_cmd_dsmif_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_dsmif_param((elc_cmd_dsmif_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_S12AD:

            /* casting the void pointer to a elc_cmd_s12ad_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_s12ad_param((elc_cmd_s12ad_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_s12ad_param((elc_cmd_s12ad_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_INTR:

            /* casting the void pointer to a elc_cmd_intr_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_intr_param((elc_cmd_intr_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_intr_param((elc_cmd_intr_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_OUT_PORT_GROUP:

            /* casting the void pointer to a elc_cmd_out_port_group_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_out_port_group_param((elc_cmd_out_port_group_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_out_port_group_param((elc_cmd_out_port_group_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_IN_PORT_GROUP:

            /* casting the void pointer to a elc_cmd_in_port_group_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_in_port_group_param((elc_cmd_in_port_group_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_in_port_group_param((elc_cmd_in_port_group_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_SINGLE_PORT:

            /* casting the void pointer to a elc_cmd_single_port_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_single_port_param((elc_cmd_single_port_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_single_port_param((elc_cmd_single_port_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_CMTW:

            /* casting the void pointer to a elc_cmd_cmtw_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_cmtw_param((elc_cmd_cmtw_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_cmtw_param((elc_cmd_cmtw_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_TPU:

            /* casting the void pointer to a elc_cmd_tpu_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_tpu_param((elc_cmd_tpu_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_tpu_param((elc_cmd_tpu_t *) p_data);
            }

        break;

        case ELC_CMD_SET_EVENT_GPT:

            /* casting the void pointer to a elc_cmd_gpt_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_gpt_param((elc_cmd_gpt_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_gpt_param((elc_cmd_gpt_t *) p_data);
            }

        break;

        case ELC_CMD_SET_PORT_GROUP:

            /* casting the void pointer to a elc_cmd_port_group_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_port_group_param((elc_cmd_port_group_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_set_port_group_param((elc_cmd_port_group_t *) p_data);
            }

        break;

        case ELC_CMD_SET_SOFTWARE_EVENT:

            elc_set_software_event();

        break;

        case ELC_CMD_GET_PORT_GROUP_VALUE:

            /* casting the void pointer to a elc_get_port_value_t type is valid structure,
               because structure member has justification checked separately */
            retval = elc_check_port_group_value((elc_get_port_value_t *) p_data);
            if (ELC_SUCCESS == retval)
            {
                elc_get_port_group_value((elc_get_port_value_t *) p_data);
            }

        break;

        default:

            retval = ELC_ERR_INVALID_ARG;

        break;
        }
    }

    return retval;
}
/*******************************************************************************
End of function ELC_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: ELC_GetVersion
* Description  : Returns ELC device driver version.
* Arguments    : none
* Return Value : version number
*******************************************************************************/
uint32_t ELC_GetVersion(void)
{
    uint32_t const version = ((ELC_VERSION_MAJOR << 16) | ELC_VERSION_MINOR);

    return version;
}
/*******************************************************************************
End of function ELC_GetVersion
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_init
* Description  : Initialize ELC HW and driver information.
*
* Arguments    : None.
* Return Value : None.
*******************************************************************************/
static void elc_init(void)
{
    uint32_t intr_num;

    /* initialize ELC */
    ELC.ELCR.BYTE   = (uint8_t)ELC_ELCR_INIT;
    ELC.ELSR0.BYTE  = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR3.BYTE  = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR4.BYTE  = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR7.BYTE  = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR10.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR11.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR12.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR13.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR15.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR18.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR19.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR20.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR21.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR22.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR23.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR24.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR25.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR26.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR27.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR33.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR35.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR36.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR37.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR38.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR41.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR42.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR43.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR44.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELSR45.BYTE = (uint8_t)ELC_ELSR_INIT;
    ELC.ELOPA.BYTE  = (uint8_t)ELC_ELOPA_INIT;
    ELC.ELOPB.BYTE  = (uint8_t)ELC_ELOPB_INIT;
    ELC.ELOPC.BYTE  = (uint8_t)ELC_ELOPC_INIT;
    ELC.ELOPD.BYTE  = (uint8_t)ELC_ELOPD_INIT;
    ELC.PGR1.BYTE   = (uint8_t)ELC_PGR_INIT;
    ELC.PGR2.BYTE   = (uint8_t)ELC_PGR_INIT;
    ELC.PGC1.BYTE   = (uint8_t)ELC_PGC_INIT;
    ELC.PGC2.BYTE   = (uint8_t)ELC_PGC_INIT;
    ELC.PDBF1.BYTE  = (uint8_t)ELC_PDBF_INIT;
    ELC.PDBF2.BYTE  = (uint8_t)ELC_PDBF_INIT;
    ELC.PEL0.BYTE   = (uint8_t)ELC_PEL_INIT;
    ELC.PEL1.BYTE   = (uint8_t)ELC_PEL_INIT;
    ELC.PEL2.BYTE   = (uint8_t)ELC_PEL_INIT;
    ELC.PEL3.BYTE   = (uint8_t)ELC_PEL_INIT;
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_INIT;
    ELC.ELOPF.BYTE  = (uint8_t)ELC_ELOPF_INIT;
    ELC.ELOPH.BYTE  = (uint8_t)ELC_ELOPH_INIT;
    ELC.ELOPI.BYTE  = (uint8_t)ELC_ELOPI_INIT;
    ELC.ELOPJ.BYTE  = (uint8_t)ELC_ELOPJ_INIT;

    /* init ELC infomartion structure */
    for (intr_num = ELC_INTR_NUM_1; intr_num < ELC_INTR_NUM; intr_num++)
    {
        elc_control.p_callback[intr_num] = NULL;
    }
}
/*******************************************************************************
End of function elc_init
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_event_resource_param
* Description  : Check event link resource parameter.
* Arguments    : event_resource_param -
*                    Event link resource parameter.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*******************************************************************************/
static elc_err_t elc_check_event_resource_param(const uint32_t event_resource_param)
{
    elc_err_t retval = ELC_SUCCESS;

    switch (event_resource_param)
    {
        /* These 7 cases are MTU0 resource parameter check */
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0A:
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0B:
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0C:
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0D:
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0E:
        case ELC_RESOURCE_MTU0_COMPARE_MATCH_0F:
        case ELC_RESOURCE_MTU0_OVERFLOW:

        /* These 5cases are MTU3 resource parameter check */
        case ELC_RESOURCE_MTU3_COMPARE_MATCH_3A:
        case ELC_RESOURCE_MTU3_COMPARE_MATCH_3B:
        case ELC_RESOURCE_MTU3_COMPARE_MATCH_3C:
        case ELC_RESOURCE_MTU3_COMPARE_MATCH_3D:
        case ELC_RESOURCE_MTU3_OVERFLOW:

        /* These 6 cases are MTU4 resource parameter check */
        case ELC_RESOURCE_MTU4_COMPARE_MATCH_4A:
        case ELC_RESOURCE_MTU4_COMPARE_MATCH_4B:
        case ELC_RESOURCE_MTU4_COMPARE_MATCH_4C:
        case ELC_RESOURCE_MTU4_COMPARE_MATCH_4D:
        case ELC_RESOURCE_MTU4_OVERFLOW:
        case ELC_RESOURCE_MTU4_UNDERFLOW:

        /* This case is CMT1 resource parameter check */
        case ELC_RESOURCE_CMT1_COMPARE_MATCH_1:

        /* This case is ETHER resource parameter check */
        case ELC_RESOURCE_ETHER_TIMER_SYNC:

        /* These 4cases are RIIC0 resource parameter check */
        case ELC_RESOURCE_RIIC0_EVENT:
        case ELC_RESOURCE_RIIC0_RX_DATA_FULL:
        case ELC_RESOURCE_RIIC0_TX_DATA_EMPTY:
        case ELC_RESOURCE_RIIC0_TX_END:

        /* These 5cases are RSPI0 resource parameter check */
        case ELC_RESOURCE_RSPI0_ERROR:
        case ELC_RESOURCE_RSPI0_IDLE:
        case ELC_RESOURCE_RSPI0_RX_DATA_FULL:
        case ELC_RESOURCE_RSPI0_TX_DATA_EMPTY:
        case ELC_RESOURCE_RSPI0_TX_END:

        /* This case is S12AD0 resource parameter check*/
        case ELC_RESOURCE_S12AD0_CONVERT_END:

        /* These 2cases are Input port group resource parameter check */
        case ELC_RESOURCE_INPUT_PORT_GROUP1:
        case ELC_RESOURCE_INPUT_PORT_GROUP2:

        /* These 4cases are Single port resource parameter check */
        case ELC_RESOURCE_SINGLE_PORT0:
        case ELC_RESOURCE_SINGLE_PORT1:
        case ELC_RESOURCE_SINGLE_PORT2:
        case ELC_RESOURCE_SINGLE_PORT3:

        /*  This case is ELC event resource parameter check */
        case ELC_RESOURCE_ELC_SOFT_EVENT:

        /* This case is DOC resource parameter check */
        case ELC_RESOURCE_DOC_DATA_CALCULATE:

        /* This case is S12AD1 resource parameter check */
        case ELC_RESOURCE_S12AD1_CONVERT_END:

        /* This case is CMTW0 resource parameter check */
        case ELC_RESOURCE_CMTW0_COMPARE_MATCH:

        /* These 6cases are GPT0 resource parameter check */
        case ELC_RESOURCE_GPT0_COMPARE_MATCH_A:
        case ELC_RESOURCE_GPT0_COMPARE_MATCH_B:
        case ELC_RESOURCE_GPT0_COMPARE_MATCH_C:
        case ELC_RESOURCE_GPT0_COMPARE_MATCH_D:
        case ELC_RESOURCE_GPT0_OVERFLOW:
        case ELC_RESOURCE_GPT0_UNDERFLOW:

        /* These 6cases are GPT1 resource parameter check */
        case ELC_RESOURCE_GPT1_COMPARE_MATCH_A:
        case ELC_RESOURCE_GPT1_COMPARE_MATCH_B:
        case ELC_RESOURCE_GPT1_COMPARE_MATCH_C:
        case ELC_RESOURCE_GPT1_COMPARE_MATCH_D:
        case ELC_RESOURCE_GPT1_OVERFLOW:
        case ELC_RESOURCE_GPT1_UNDERFLOW:

        /* These 6cases are GPT2 resource parameter check */
        case ELC_RESOURCE_GPT2_COMPARE_MATCH_A:
        case ELC_RESOURCE_GPT2_COMPARE_MATCH_B:
        case ELC_RESOURCE_GPT2_COMPARE_MATCH_C:
        case ELC_RESOURCE_GPT2_COMPARE_MATCH_D:
        case ELC_RESOURCE_GPT2_OVERFLOW:
        case ELC_RESOURCE_GPT2_UNDERFLOW:

        /* These 6cases are GPT3 resource parameter check */
        case ELC_RESOURCE_GPT3_COMPARE_MATCH_A:
        case ELC_RESOURCE_GPT3_COMPARE_MATCH_B:
        case ELC_RESOURCE_GPT3_COMPARE_MATCH_C:
        case ELC_RESOURCE_GPT3_COMPARE_MATCH_D:
        case ELC_RESOURCE_GPT3_OVERFLOW:
        case ELC_RESOURCE_GPT3_UNDERFLOW:

        /* These 5cases are MTU6 resource parameter check */
        case ELC_RESOURCE_MTU6_COMPARE_MATCH_6A:
        case ELC_RESOURCE_MTU6_COMPARE_MATCH_6B:
        case ELC_RESOURCE_MTU6_COMPARE_MATCH_6C:
        case ELC_RESOURCE_MTU6_COMPARE_MATCH_6D:
        case ELC_RESOURCE_MTU6_OVERFLOW:

        /* These 6cases are MTU7 resource parameter check */
        case ELC_RESOURCE_MTU7_COMPARE_MATCH_7A:
        case ELC_RESOURCE_MTU7_COMPARE_MATCH_7B:
        case ELC_RESOURCE_MTU7_COMPARE_MATCH_7C:
        case ELC_RESOURCE_MTU7_COMPARE_MATCH_7D:
        case ELC_RESOURCE_MTU7_OVERFLOW:
        case ELC_RESOURCE_MTU7_UNDERFLOW:

        /* These 5cases are TPU0 resource parameter check */
        case ELC_RESOURCE_TPU0_COMPARE_MATCH_A:
        case ELC_RESOURCE_TPU0_COMPARE_MATCH_B:
        case ELC_RESOURCE_TPU0_COMPARE_MATCH_C:
        case ELC_RESOURCE_TPU0_COMPARE_MATCH_D:
        case ELC_RESOURCE_TPU0_OVERFLOW:

        /* These 4cases are TPU1 resource parameter check */
        case ELC_RESOURCE_TPU1_COMPARE_MATCH_A:
        case ELC_RESOURCE_TPU1_COMPARE_MATCH_B:
        case ELC_RESOURCE_TPU1_OVERFLOW:
        case ELC_RESOURCE_TPU1_UNDERFLOW:

        /* These 4cases are TPU2 resource parameter check */
        case ELC_RESOURCE_TPU2_COMPARE_MATCH_A:
        case ELC_RESOURCE_TPU2_COMPARE_MATCH_B:
        case ELC_RESOURCE_TPU2_OVERFLOW:
        case ELC_RESOURCE_TPU2_UNDERFLOW:

        /* These 5cases are TPU3 resource parameter check */
        case ELC_RESOURCE_TPU3_COMPARE_MATCH_A:
        case ELC_RESOURCE_TPU3_COMPARE_MATCH_B:
        case ELC_RESOURCE_TPU3_COMPARE_MATCH_C:
        case ELC_RESOURCE_TPU3_COMPARE_MATCH_D:
        case ELC_RESOURCE_TPU3_OVERFLOW:

            /* Nothing done on event link resource setting is correct */

        break;

        default :

            retval = ELC_ERR_INVALID_ARG;

        break;
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_event_resource_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_mtu_param
* Description  : Check event link parameter of MTU.
* Arguments    : p_mtu_param -
*                    Event link parameter of MTU.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_mtu_param(const elc_cmd_mtu_t * const p_mtu_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for MTU */

    if (NULL == p_mtu_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_mtu_param->elc_mtu_ch >= ELC_MTU_CH_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_mtu_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_mtu_param->resource);
                if (ELC_SUCCESS == retval)
                {
                    if (p_mtu_param->action >= ELC_MTU_MAX_OVER)
                    {
                        retval = ELC_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_mtu_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_cmt_param
* Description  : Check event link parameter of CMT.
* Arguments    : p_cmt_param -
*                    Event link parameter of CMT.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_cmt_param(const elc_cmd_cmt_t * const p_cmt_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for CMT */

    if (NULL == p_cmt_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (false != p_cmt_param->event_link_enable)
        {
            retval = elc_check_event_resource_param(p_cmt_param->resource);
            if (ELC_SUCCESS == retval)
            {
                if (p_cmt_param->action >= ELC_CMT_MAX_OVER)
                {
                    retval = ELC_ERR_INVALID_ARG;
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_cmt_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_dsmif_param
* Description  : Check event link parameter of delta sigma unit.
* Arguments    : p_dsmif_param -
*                    Event link parameter of delta sigma unit.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_dsmif_param(const elc_cmd_dsmif_t * const p_dsmif_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for delta sigma unit */

    if (NULL == p_dsmif_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_dsmif_param->elc_dsmif_ch >= ELC_DSMIF_TRIGGER_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_dsmif_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_dsmif_param->resource);
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_dsmif_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_s12ad_param
* Description  : Check event link parameter of 12bit A/D converter.
* Arguments    : p_s12ad_param -
*                    Event link parameter of 12bit A/D converter.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_s12ad_param(const elc_cmd_s12ad_t * const p_s12ad_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for 12bit A/D converter */

    if (NULL == p_s12ad_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_s12ad_param->elc_s12ad_ch >= ELC_S12AD_CH_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_s12ad_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_s12ad_param->resource);
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_s12ad_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_intr_param
* Description  : Check event link parameter of ELC interrupt.
* Arguments    : p_intr_param -
*                    Event link parameter of ELC interrupt.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_intr_param(const elc_cmd_intr_t * const p_intr_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for ELC interrupt */

    if (NULL == p_intr_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_intr_param->elc_intr_num >= ELC_INTR_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_intr_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_intr_param->resource);
                if (ELC_SUCCESS == retval)
                {
                    if (VIC_PRIORITY_15 < p_intr_param->intr_priority)
                    {
                        retval = ELC_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_intr_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_out_port_group_param
* Description  : Check event link parameter of output port group.
* Arguments    : p_out_port_group_param -
*                    Event link parameter of output port group.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_out_port_group_param(const elc_cmd_out_port_group_t * const p_out_port_group_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for output port group */

    if (NULL == p_out_port_group_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_out_port_group_param->elc_out_port_group_num >= ELC_PORT_GROUP_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_out_port_group_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_out_port_group_param->resource);
                if (ELC_SUCCESS == retval)
                {
                    if (p_out_port_group_param->action >= ELC_OUT_GROUP_MAX_OVER)
                    {
                        retval = ELC_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_out_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_in_port_group_param
* Description  : Check event link parameter of input port group.
* Arguments    : p_in_port_group_param -
*                    Event link parameter of input port group.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_in_port_group_param(const elc_cmd_in_port_group_t * const p_in_port_group_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for input port group */

    if (NULL == p_in_port_group_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_in_port_group_param->elc_in_port_group_num >= ELC_PORT_GROUP_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_in_port_group_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_in_port_group_param->resource);
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_in_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_single_port_param
* Description  : Check event link parameter of single port.
* Arguments    : p_out_port_group_param -
*                    Event link parameter of single port.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_single_port_param(const elc_cmd_single_port_t * const p_single_port_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for single port */

    if (NULL == p_single_port_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_single_port_param->elc_single_port_num >= ELC_SINGLE_PORT_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }

        if (ELC_SUCCESS == retval)
        {
            if (p_single_port_param->port_symbol >= ELC_PORT_NUM)
            {
                retval = ELC_ERR_INVALID_ARG;
            }
        }

        if (ELC_SUCCESS == retval)
        {
            if (p_single_port_param->port_num > ELC_IO_PORT_NUM)
            {
                retval = ELC_ERR_INVALID_ARG;
            }
        }

        if (ELC_SUCCESS == retval)
        {
            if (false != p_single_port_param->event_link_enable)
            {
                if ( p_single_port_param->event_direction >= ELC_SINGLE_EVENT_MAX_OVER)
                {
                    retval = ELC_ERR_INVALID_ARG;
                }
                else
                {
                    if (ELC_SINGLE_EVENT_OUTPUT == p_single_port_param->event_direction)
                    {
                        /* check output setting */
                        retval = elc_check_event_resource_param(p_single_port_param->resource);
                        if (ELC_SUCCESS == retval)
                        {
                            if (p_single_port_param->output_action >= ELC_SINGLE_PORT_ACTION_MAX_OVER)
                            {
                                retval = ELC_ERR_INVALID_ARG;
                            }
                        }
                    }
                    else
                    {
                        /* check input setting */
                        if (p_single_port_param->input_trigger >= ELC_SINGLE_PORT_TRIGGER_MAX_OVER)
                        {
                            retval = ELC_ERR_INVALID_ARG;
                        }
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_single_port_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_cmtw_param
* Description  : Check event link parameter of CMTW.
* Arguments    : p_cmtw_param -
*                    Event link parameter of CMTW.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_cmtw_param(const elc_cmd_cmtw_t * const p_cmtw_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for CMTW */

    if (NULL == p_cmtw_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (false != p_cmtw_param->event_link_enable)
        {
            retval = elc_check_event_resource_param(p_cmtw_param->resource);
            if (ELC_SUCCESS == retval)
            {
                if (p_cmtw_param->action >= ELC_CMTW_COUNT_MAX_OVER)
                {
                    retval = ELC_ERR_INVALID_ARG;
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_cmtw_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_tpu_param
* Description  : Check event link parameter of TPU.
* Arguments    : p_tpu_param -
*                    Event link parameter of TPU.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_tpu_param(const elc_cmd_tpu_t * const p_tpu_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for TPU */

    if (NULL == p_tpu_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_tpu_param->elc_tpu_ch >= ELC_TPU_CH_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_tpu_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_tpu_param->resource);
                if (ELC_SUCCESS == retval)
                {
                    if (p_tpu_param->action >= ELC_TPU_MAX_OVER)
                    {
                        retval = ELC_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_tpu_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_gpt_param
* Description  : Check event link parameter of GPT.
* Arguments    : p_gpt_param -
*                    Event link parameter of GPT.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Event link resource parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_gpt_param(const elc_cmd_gpt_t * const p_gpt_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check event link parameter for GPT */

    if (NULL == p_gpt_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_gpt_param->elc_gpt_ch >= ELC_GPT_CH_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (false != p_gpt_param->event_link_enable)
            {
                retval = elc_check_event_resource_param(p_gpt_param->resource);
                if (ELC_SUCCESS == retval)
                {
                    if (p_gpt_param->action >= ELC_GPT_MAX_OVER)
                    {
                        retval = ELC_ERR_INVALID_ARG;
                    }
                }
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_gpt_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_port_group_param
* Description  : Check setting of port group parameter.
* Arguments    : p_port_group_param -
*                    Port group parameter.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Port group parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_port_group_param(const elc_cmd_port_group_t * const p_port_group_param)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check setting of port group parameter */

    if (NULL == p_port_group_param)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_port_group_param->port_group_num >= ELC_PORT_GROUP_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
        else
        {
            if (p_port_group_param->trigger >= ELC_PORT_GROUP_MAX_OVER)
            {
                retval = ELC_ERR_INVALID_ARG;
            }
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_check_port_group_value
* Description  : Check setting of port group parameter.
* Arguments    : p_get_port_value -
*                    Getting of Port value parameter.
* Return Value : ELC_SUCCESS-
*                    Successful
*                ELC_ERR_INVALID_ARG-
*                    Setting of port group parameter is invalid.
*                ELC_ERR_MISSING_PTR-
*                    Setting parameter is NULL pointer.
*******************************************************************************/
static elc_err_t elc_check_port_group_value(const elc_get_port_value_t * const p_get_port_value)
{
    elc_err_t retval = ELC_SUCCESS;

    /* check setting of port group parameter */

    if (NULL == p_get_port_value)
    {
        retval = ELC_ERR_MISSING_PTR;
    }
    else
    {
        if (p_get_port_value->elc_port_group_num >= ELC_PORT_GROUP_NUM)
        {
            retval = ELC_ERR_INVALID_ARG;
        }
    }

    return retval;
}
/*******************************************************************************
End of function elc_check_port_group_value
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_mtu_param
* Description  : Set MTU parameter of event link.
*
* Arguments    : p_mtu_param -
*                    Event link parameter of MTU.
* Return Value : None.
*******************************************************************************/
static void elc_set_mtu_param(const elc_cmd_mtu_t * const p_mtu_param)
{
    if (NULL == p_mtu_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_mtu_param->event_link_enable)
        {
            /* set event link parameter of MTU */
            /* Configure event link resource to MTU0, MTU3, MTU4
               Correspondence relations of MTU and registers are MTU0 and ELSR0,
               MTU3 and ELSR3, MTU4 and ELSR4
               ELSR0, 3, 4 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure action of MTU0, MTU3, MTU4 when event link is run
               ELOPA - Event link option select register A
               b1:b0 MTU0MD - MTU0 action select bit - set user request value
               b7:b6 MTU3MD - MTU3 action select bit - set user request value
               ELOPB - Event link option select register B
               b1:b0 MTU4MD - MTU4 action select bit - set user request value */
            switch (p_mtu_param->elc_mtu_ch)
            {
                case ELC_MTU_CH_0:

                    ELC.ELSR0.BYTE = (uint8_t)p_mtu_param->resource;
                    ELC.ELOPA.BIT.MTU0MD = p_mtu_param->action;

                break;

                case ELC_MTU_CH_3:

                    ELC.ELSR3.BYTE = (uint8_t)p_mtu_param->resource;
                    ELC.ELOPA.BIT.MTU3MD = p_mtu_param->action;

                break;

                case ELC_MTU_CH_4:

                    ELC.ELSR4.BYTE = (uint8_t)p_mtu_param->resource;
                    ELC.ELOPB.BIT.MTU4MD = p_mtu_param->action;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of MTU */
            /* Configure event link resource to MTU0, MTU3, MTU4
               Correspondence relations of MTU and registers are MTU0 and ELSR0,
               MTU3 and ELSR3, MTU4 and ELSR4
               ELSR0, 3, 4 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of MTU0, MTU3, MTU4 when event link is run
               ELOPA - Event link option select register A
               b1:b0 MTU0MD - MTU0 action select bit - Write 0x03 to event disable
               b7:b6 MTU3MD - MTU3 action select bit - Write 0x03 to event disable
               ELOPB - Event link option select register B
               b1:b0 MTU4MD - MTU4 action select bit - Write 0x03 to event disable */
            switch (p_mtu_param->elc_mtu_ch)
            {
                case ELC_MTU_CH_0:

                    ELC.ELSR0.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPA.BIT.MTU0MD = (uint8_t)ELC_ELOPA_NONE_EVENT;

                break;

                case ELC_MTU_CH_3:

                    ELC.ELSR3.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPA.BIT.MTU3MD = (uint8_t)ELC_ELOPA_NONE_EVENT;

                break;

                case ELC_MTU_CH_4:

                    ELC.ELSR4.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPB.BIT.MTU4MD = (uint8_t)ELC_ELOPB_NONE_EVENT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_mtu_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_cmt_param
* Description  : Set CMT parameter of event link.
*
* Arguments    : p_cmt_param -
*                    Event link parameter of CMT.
* Return Value : None.
*******************************************************************************/
static void elc_set_cmt_param(const elc_cmd_cmt_t * const p_cmt_param)
{
    if (NULL == p_cmt_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_cmt_param->event_link_enable)
        {
            /* set event link parameter of CMT */
            /* Configure event link resource to CMT1
               ELSR7 - Event link coufigure register
               b7:b0 ELS - Event link select bit - Set user request value */
            /* Configure action of CMT1 when event link is run
               ELOPC - Event link option select register C
               b3:b2 CMT1MD - CMT1 action select bit - Set user request value */
            ELC.ELSR7.BYTE = (uint8_t)p_cmt_param->resource;
            ELC.ELOPC.BIT.CMT1MD = p_cmt_param->action;
        }
        else
        {
            /* clear event link parameter of CMT */
            /* Configure event link resource to CMT1
               ELSR7 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of CMT1 when event link is run
               ELOPC - Event link option select register C
               b3:b2 CMT1MD - CMT1 action select bit - Write 0x03 to event disable */
            ELC.ELSR7.BYTE = (uint8_t)ELC_ELSR_INIT;
            ELC.ELOPC.BIT.CMT1MD = (uint8_t)ELC_ELOPC_NONE_EVENT;
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_cmt_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_dsmif_param
* Description  : Set delta sigma unit parameter of event link.
*
* Arguments    : p_dsmif_param -
*                    Event link parameter of delta sigma unit.
* Return Value : None.
*******************************************************************************/
static void elc_set_dsmif_param(const elc_cmd_dsmif_t * const p_dsmif_param)
{
    if (NULL == p_dsmif_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_dsmif_param->event_link_enable)
        {
            /* set event link parameter of delta sigma unit */
            /* Configure event link resource to delta sigma unit
               Correspondence relations of Delta sigma unit channel and trigger and registers are
               channel0, trigger0 and ELSR10, channel0, trigger1 and ELSR11, channel1, trigger0 and ELSR12,
               channel1, trigger1 and ELSR13
               ELSR10, 11, 12, 13 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Set event enable/disable of dleta sigma unit when event link is run
               ELOPD - Event link option select register C
               b1:b0 DSU0MD - Delta sigma unit0 trigger0 event link select bit - Write 0x00 to event enable
               b3:b2 DSU1MD - Delta sigma unit0 trigger1 event link select bit - Write 0x00 to event enable
               b5:b4 DSX0MD - Delta sigma unit1 trigger0 event link select bit - Write 0x00 to event enable
               b7:b6 DSX1MD - Delta sigma unit1 trigger1 event link select bit - Write 0x00 to event enable */
            switch (p_dsmif_param->elc_dsmif_ch)
            {
                case ELC_DSMIF0_TRIGGER_0:

                    ELC.ELSR10.BYTE = (uint8_t)p_dsmif_param->resource;
                    ELC.ELOPD.BIT.DSU0MD = (uint8_t)ELC_ELOPD_EVENT_ENABLE;

                break;

                case ELC_DSMIF0_TRIGGER_1:

                    ELC.ELSR11.BYTE = (uint8_t)p_dsmif_param->resource;
                    ELC.ELOPD.BIT.DSU1MD = (uint8_t)ELC_ELOPD_EVENT_ENABLE;

                break;

                case ELC_DSMIF1_TRIGGER_0:

                    ELC.ELSR12.BYTE = (uint8_t)p_dsmif_param->resource;
                    ELC.ELOPD.BIT.DSX0MD = (uint8_t)ELC_ELOPD_EVENT_ENABLE;

                break;

                case ELC_DSMIF1_TRIGGER_1:

                    ELC.ELSR13.BYTE = (uint8_t)p_dsmif_param->resource;
                    ELC.ELOPD.BIT.DSX1MD = (uint8_t)ELC_ELOPD_EVENT_ENABLE;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of delta sigma unit */
            /* Configure event link resource to delta sigma unit
               Correspondence relations of Delta sigma unit channel and trigger and registers are
               channel0, trigger0 and ELSR10, channel0, trigger1 and ELSR11, channel1, trigger0 and ELSR12,
               channel1, trigger1 and ELSR13
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Set event enable/disable of dleta sigma unit when event link is run
               ELOPD - Event link option select register C
               b1:b0 DSU0MD - Delta sigma unit0 trigger0 event link select bit - Write 0x03 to event disable
               b3:b2 DSU1MD - Delta sigma unit0 trigger1 event link select bit - Write 0x03 to event disable
               b5:b4 DSX0MD - Delta sigma unit1 trigger0 event link select bit - Write 0x03 to event disable
               b7:b6 DSX1MD - Delta sigma unit1 trigger1 event link select bit - Write 0x03 to event disable */
            switch (p_dsmif_param->elc_dsmif_ch)
            {
                case ELC_DSMIF0_TRIGGER_0:

                    ELC.ELSR10.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPD.BIT.DSU0MD = (uint8_t)ELC_ELOPD_EVENT_DISABLE;

                break;

                case ELC_DSMIF0_TRIGGER_1:

                    ELC.ELSR11.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPD.BIT.DSU1MD = (uint8_t)ELC_ELOPD_EVENT_DISABLE;

                break;

                case ELC_DSMIF1_TRIGGER_0:

                    ELC.ELSR12.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPD.BIT.DSX0MD = (uint8_t)ELC_ELOPD_EVENT_DISABLE;

                break;

                case ELC_DSMIF1_TRIGGER_1:

                    ELC.ELSR13.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPD.BIT.DSX1MD = (uint8_t)ELC_ELOPD_EVENT_DISABLE;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_dsmif_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_s12ad_param
* Description  : Set 12bit A/D convert parameter of event link.
*
* Arguments    : p_s12ad_param -
*                    Event link parameter of 12bit A/D converter.
* Return Value : None.
*******************************************************************************/
static void elc_set_s12ad_param(const elc_cmd_s12ad_t * const p_s12ad_param)
{
    if (NULL == p_s12ad_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_s12ad_param->event_link_enable)
        {
            /* set event link parameter of 12bit A/D converter */
            /* Configure event link resource to 12bit A/D converter
               Correspondence relations of 12bit A/D converter channel and registers are
               channel0 and ELSR15, channel1 and ELSR45.
               ELSR15, 45 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            switch (p_s12ad_param->elc_s12ad_ch)
            {
                case ELC_S12AD_CH_0:

                    ELC.ELSR15.BYTE = (uint8_t)p_s12ad_param->resource;

                break;

                case ELC_S12AD_CH_1:

                    ELC.ELSR45.BYTE = (uint8_t)p_s12ad_param->resource;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of 12bit A/D converter */
            /* Configure event link resource to 12bit A/D converter
               Correspondence relations of 12bit A/D converter channel and registers are
               channel0 and ELSR15, channel1 and ELSR45.
               ELSR15, 45 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            switch (p_s12ad_param->elc_s12ad_ch)
            {
                case ELC_S12AD_CH_0:

                    ELC.ELSR15.BYTE = (uint8_t)ELC_ELSR_INIT;

                break;

                case ELC_S12AD_CH_1:

                    ELC.ELSR45.BYTE = (uint8_t)ELC_ELSR_INIT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_s12ad_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_intr_param
* Description  : Set event link parameter of ELC interrupt.
*
* Arguments    : p_intr_param -
*                    Event link parameter of ELC interrupt.
* Return Value : None.
*******************************************************************************/
static void elc_set_intr_param(const elc_cmd_intr_t * const p_intr_param)
{
    if (NULL == p_intr_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_intr_param->event_link_enable)
        {
            /* set event link parameter of INTR */
            /* Configure event link resource to ELC interrupt
               Correspondence relations of ELC interrupt number and registers are
               interrupt1 and ELSR18, interrupt2 and ELSR19.
               ELSR18, 19 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            switch (p_intr_param->elc_intr_num)
            {
                case ELC_INTR_NUM_1:

                    ELC.ELSR18.BYTE = (uint8_t)p_intr_param->resource;

                break;

                case ELC_INTR_NUM_2:

                    ELC.ELSR19.BYTE = (uint8_t)p_intr_param->resource;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }

            /* interrupt enable */
            elc_control.p_callback[p_intr_param->elc_intr_num] = p_intr_param->p_callback;
            elc_intr_register(p_intr_param->elc_intr_num, p_intr_param->intr_priority);
            elc_intr_enable(p_intr_param->elc_intr_num);
        }
        else
        {
            /* clear event link parameter of INT */
            /* Configure event link resource to ELC interrupt
               Correspondence relations of ELC interrupt number and registers are
               interrupt1 and ELSR18, interrupt2 and ELSR19.
               ELSR18, 19 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            switch (p_intr_param->elc_intr_num)
            {
                case ELC_INTR_NUM_1:

                    ELC.ELSR18.BYTE = (uint8_t)ELC_ELSR_INIT;

                break;

                case ELC_INTR_NUM_2:

                    ELC.ELSR19.BYTE = (uint8_t)ELC_ELSR_INIT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }

            /* disable interrupt */
            elc_intr_disable(p_intr_param->elc_intr_num);
            elc_control.p_callback[p_intr_param->elc_intr_num] = NULL;
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_intr_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_out_port_group_param
* Description  : Set event link parameter of output port group.
*
* Arguments    : p_out_port_group_param -
*                    Event link parameter of output port group.
* Return Value : None.
*******************************************************************************/
static void elc_set_out_port_group_param(const elc_cmd_out_port_group_t * const p_out_port_group_param)
{
    if (NULL == p_out_port_group_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_out_port_group_param->event_link_enable)
        {
            /* set event link parameter of output port group */
            /* Configure event link resource to output port group0, output port group1
               Correspondence relations of port number and registers are output port group0 and ELSR20,
               output port group1 and ELSR21
               ELSR20, 21 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure action of output port group0, output port group1 when event link is run
               PGC1 - Port group control regster1
               b6:b4 PGCO1 - port group action select bit - set user request value
               PGC2 - Port group control regster2
               b6:b4 PGCO2 - port group action select bit - set user request value */
            /* Configure initial port value of output port group0, output port group1
               PDBF1 - Port buffer regster1
               b7:b0 PDBF10-PDBF17 - port buffer bit - set user request value
               PDBF2 - Port buffer regster2
               b7:b0 PDBF20-PDBF27 - port buffer bit - set user request value */
            switch (p_out_port_group_param->elc_out_port_group_num)
            {
                case ELC_PORT_GROUP_NUM_0:

                    ELC.ELSR20.BYTE = (uint8_t)p_out_port_group_param->resource;
                    ELC.PGC1.BIT.PGCOn = p_out_port_group_param->action;
                    ELC.PDBF1.BYTE = p_out_port_group_param->init_value;

                break;

                case ELC_PORT_GROUP_NUM_1:

                    ELC.ELSR21.BYTE = (uint8_t)p_out_port_group_param->resource;
                    ELC.PGC2.BIT.PGCOn = p_out_port_group_param->action;
                    ELC.PDBF2.BYTE = p_out_port_group_param->init_value;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of output port group */
            /* Configure event link resource to output port group0, output port group1
               Correspondence relations of port number and registers are output port group0 and ELSR20,
               output port group1 and ELSR21
               ELSR20, 21 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of output port group0, output port group1 when event link is run
               PGC1 - Port group control regster1
               b6:b4 PGCO1 - port group action select bit - Write 0x00 to bit initial value
               PGC2 - Port group control regster2
               b6:b4 PGCO2 - port group action select bit - Write 0x00 to bit initial value */
            /* Configure initial port value of output port group0, output port group1
               PDBF1 - Port buffer regster1
               b7:b0 PDBF10-PDBF17 - port buffer bit - HW init value (0x00)
               PDBF2 - Port buffer regster2
               b7:b0 PDBF20-PDBF27 - port buffer bit - HW init value (0x00) */
            switch (p_out_port_group_param->elc_out_port_group_num)
            {
                case ELC_PORT_GROUP_NUM_0:

                    ELC.ELSR20.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PGC1.BIT.PGCOn = (uint8_t)0U;
                    ELC.PDBF1.BYTE = (uint8_t)ELC_PDBF_INIT;

                break;

                case ELC_PORT_GROUP_NUM_1:

                    ELC.ELSR21.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PGC2.BIT.PGCOn = (uint8_t)0U;
                    ELC.PDBF2.BYTE = (uint8_t)ELC_PDBF_INIT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_out_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_in_port_group_param
* Description  : Set event link parameter of input port group.
*
* Arguments    : p_in_port_group_param -
*                    Event link parameter of input port group.
* Return Value : None.
*******************************************************************************/
static void elc_set_in_port_group_param(const elc_cmd_in_port_group_t * const p_in_port_group_param)
{
    if (NULL == p_in_port_group_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_in_port_group_param->event_link_enable)
        {
            /* set event link parameter of input port group */
            /* Configure event link resource to input port group0, input port group1
               Correspondence relations of poert numbr and registers are input port group0 and ELSR22,
               input port group1 and ELSR23
               ELSR22, 23 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure over write signal value to buffer
               PGC1 - Port group control regster1
               b2 PGCOVE1 - PDBF overwrite select bit - on enable : write 1 to over write enable
                                                        on disable : write 0 to over write disable
               PGC2 - Port group control regster2
               b2 PGCOVE2 - PDBF overwrite select bit - on enable : write 1 to over write enable
                                                        on disable : write 0 to over write disable */
            switch (p_in_port_group_param->elc_in_port_group_num)
            {
                case ELC_PORT_GROUP_NUM_0:

                    ELC.ELSR22.BYTE = (uint8_t)p_in_port_group_param->resource;
                    if (false != p_in_port_group_param->overwrite_enable)
                    {
                        ELC.PGC1.BIT.PGCOVEn = (uint8_t)1U;
                    }
                    else
                    {
                        ELC.PGC1.BIT.PGCOVEn = (uint8_t)0U;
                    }

                break;

                case ELC_PORT_GROUP_NUM_1:

                    ELC.ELSR23.BYTE = (uint8_t)p_in_port_group_param->resource;
                    if (false != p_in_port_group_param->overwrite_enable)
                    {
                        ELC.PGC2.BIT.PGCOVEn = (uint8_t)1U;
                    }
                    else
                    {
                        ELC.PGC2.BIT.PGCOVEn = (uint8_t)0U;
                    }

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of input port group */
            /* Configure event link resource to input port group0, input port group1
               Correspondence relations of poert numbr and registers are input port group0 and ELSR22,
               input port group1 and ELSR23
               ELSR22, 23 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure over write signal value to buffer
               PGC1 - Port group control regster1
               b2 PGCOVE1 - PDBF overwrite select bit - write 0 to over write disable
               PGC2 - Port group control regster2
               b2 PGCOVE2 - PDBF overwrite select bit - write 0 to over write disable */
            switch (p_in_port_group_param->elc_in_port_group_num)
            {
                case ELC_PORT_GROUP_NUM_0:

                    ELC.ELSR22.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PGC1.BIT.PGCOVEn = (uint8_t)0U;

                break;

                case ELC_PORT_GROUP_NUM_1:

                    ELC.ELSR23.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PGC2.BIT.PGCOVEn = (uint8_t)0U;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_in_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_single_port_param
* Description  : Set single port parameter of event link.
*
* Arguments    : p_single_port_param -
*                    Event link parameter of single port.
* Return Value : None.
*******************************************************************************/
static void elc_set_single_port_param(const elc_cmd_single_port_t * const p_single_port_param)
{
    if (NULL == p_single_port_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_single_port_param->event_link_enable)
        {
            /* set event link parameter of single port */
            /* Configure event link resource to single port0, single port1, single port2, single port3
               Correspondence relations of MTU and registers are single port0 and ELSR24,
               single port1 and ELSR25, single port2 and ELSR26, single port3 and ELSR27,
               ELSR24, 25, 26, 27 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure connection and action of single port0
               PEL0 - Event connect port select register0
               b2:b0 PSB0 - bit number select bit - set user request value
               b4:b3 PSP0 - port number select bit - set user request value
               b6:b5 PSM0 - event link select bit - set user request value
               PEL1 - Event connect port select register1
               b2:b0 PSB1 - bit number select bit - set user request value
               b4:b3 PSP1 - port number select bit - set user request value
               b6:b5 PSM1 - event link select bit - set user request value
               PEL0 - Event connect port select register2
               b2:b0 PSB2 - bit number select bit - set user request value
               b4:b3 PSP2 - port number select bit - set user request value
               b6:b5 PSM2 - event link select bit - set user request value
               PEL0 - Event connect port select register3
               b2:b0 PSB3 - bit number select bit - set user request value
               b4:b3 PSP3 - port number select bit - set user request value
               b6:b5 PSM3 - event link select bit - set user request value */
            switch (p_single_port_param->elc_single_port_num)
            {
                case ELC_SINGLE_PORT_NUM_0:

                    if (ELC_SINGLE_EVENT_OUTPUT == p_single_port_param->event_direction)
                    {
                        ELC.ELSR24.BYTE = (uint8_t)p_single_port_param->resource;
                        ELC.PEL0.BIT.PSMn = p_single_port_param->output_action;
                    }
                    else
                    {
                        ELC.ELSR24.BYTE = (uint8_t)ELC_ELSR_INIT;
                        ELC.PEL0.BIT.PSMn = p_single_port_param->input_trigger;
                    }
                    ELC.PEL0.BIT.PSBn = p_single_port_param->port_num;
                    ELC.PEL0.BIT.PSPn = p_single_port_param->port_symbol;

                break;

                case ELC_SINGLE_PORT_NUM_1:

                    if (ELC_SINGLE_EVENT_OUTPUT == p_single_port_param->event_direction)
                    {
                        ELC.ELSR25.BYTE = (uint8_t)p_single_port_param->resource;
                        ELC.PEL1.BIT.PSMn = p_single_port_param->output_action;
                    }
                    else
                    {
                        ELC.ELSR25.BYTE = (uint8_t)ELC_ELSR_INIT;
                        ELC.PEL1.BIT.PSMn = p_single_port_param->input_trigger;
                    }
                    ELC.PEL1.BIT.PSBn = p_single_port_param->port_num;
                    ELC.PEL1.BIT.PSPn = p_single_port_param->port_symbol;

                break;

                case ELC_SINGLE_PORT_NUM_2:

                    if (ELC_SINGLE_EVENT_OUTPUT == p_single_port_param->event_direction)
                    {
                        ELC.ELSR26.BYTE = (uint8_t)p_single_port_param->resource;
                        ELC.PEL2.BIT.PSMn = p_single_port_param->output_action;
                    }
                    else
                    {
                        ELC.ELSR26.BYTE = (uint8_t)ELC_ELSR_INIT;
                        ELC.PEL2.BIT.PSMn = p_single_port_param->input_trigger;
                    }
                    ELC.PEL2.BIT.PSBn = p_single_port_param->port_num;
                    ELC.PEL2.BIT.PSPn = p_single_port_param->port_symbol;

                break;

                case ELC_SINGLE_PORT_NUM_3:

                    if (ELC_SINGLE_EVENT_OUTPUT == p_single_port_param->event_direction)
                    {
                        ELC.ELSR27.BYTE = (uint8_t)p_single_port_param->resource;
                        ELC.PEL3.BIT.PSMn = p_single_port_param->output_action;
                    }
                    else
                    {
                        ELC.ELSR27.BYTE = (uint8_t)ELC_ELSR_INIT;
                        ELC.PEL3.BIT.PSMn = p_single_port_param->input_trigger;
                    }
                    ELC.PEL3.BIT.PSBn = p_single_port_param->port_num;
                    ELC.PEL3.BIT.PSPn = p_single_port_param->port_symbol;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of single port */
            /* Configure event link resource to single port0, single port1, single port2, single port3
               Correspondence relations of MTU and registers are single port0 and ELSR24,
               single port1 and ELSR25, single port2 and ELSR26, single port3 and ELSR27,
               ELSR24, 25, 26, 27 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure connection and action of single port0
               PEL0 - Event connect port select register0
               b2:b0 PSB0 - bit number select bit - write 0 to HW initial value
               b4:b3 PSP0 - port number select bit - write 0 to HW initial value
               b6:b5 PSM0 - event link select bit - write 0 to HW initial value
               PEL1 - Event connect port select register1
               b2:b0 PSB1 - bit number select bit - write 0 to HW initial value
               b4:b3 PSP1 - port number select bit - write 0 to HW initial value
               b6:b5 PSM1 - event link select bit - write 0 to HW initial value
               PEL0 - Event connect port select register2
               b2:b0 PSB2 - bit number select bit - write 0 to HW initial value
               b4:b3 PSP2 - port number select bit - write 0 to HW initial value
               b6:b5 PSM2 - event link select bit - write 0 to HW initial value
               PEL0 - Event connect port select register3
               b2:b0 PSB3 - bit number select bit - write 0 to HW initial value
               b4:b3 PSP3 - port number select bit - write 0 to HW initial value
               b6:b5 PSM3 - event link select bit - write 0 to HW initial value */
            switch (p_single_port_param->elc_single_port_num)
            {
                case ELC_SINGLE_PORT_NUM_0:

                    ELC.ELSR24.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PEL0.BIT.PSMn = (uint8_t)0U;
                    ELC.PEL0.BIT.PSBn = (uint8_t)0U;
                    ELC.PEL0.BIT.PSPn = (uint8_t)0U;

                break;

                case ELC_SINGLE_PORT_NUM_1:

                    ELC.ELSR25.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PEL1.BIT.PSMn = (uint8_t)0U;
                    ELC.PEL1.BIT.PSBn = (uint8_t)0U;
                    ELC.PEL1.BIT.PSPn = (uint8_t)0U;

                break;

                case ELC_SINGLE_PORT_NUM_2:

                    ELC.ELSR26.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PEL2.BIT.PSMn = (uint8_t)0U;
                    ELC.PEL2.BIT.PSBn = (uint8_t)0U;
                    ELC.PEL2.BIT.PSPn = (uint8_t)0U;

                break;

                case ELC_SINGLE_PORT_NUM_3:

                    ELC.ELSR27.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.PEL3.BIT.PSMn = (uint8_t)0U;
                    ELC.PEL3.BIT.PSBn = (uint8_t)0U;
                    ELC.PEL3.BIT.PSPn = (uint8_t)0U;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_single_port_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_cmtw_param
* Description  : Set CMTW parameter of event link.
*
* Arguments    : p_cmtw_param -
*                    Event link parameter of CMTW.
* Return Value : None.
*******************************************************************************/
static void elc_set_cmtw_param(const elc_cmd_cmtw_t * const p_cmtw_param)
{
    if (NULL == p_cmtw_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_cmtw_param->event_link_enable)
        {
            /* set event link parameter of CMTW */
            /* Configure event link resource to CMTW0
               ELSR33 - Event link coufigure register
               b7:b0 ELS - Event link select bit - Set user request value */
            /* Configure action of CMTW0 when event link is run
               ELOPH - Event link option select register H
               b1:b0 CMTW0MD - CMTW0 action select bit - Set user request value */
            ELC.ELSR33.BYTE = (uint8_t)p_cmtw_param->resource;
            ELC.ELOPH.BIT.CMTW0MD = p_cmtw_param->action;
        }
        else
        {
            /* clear event link parameter of CMTW */
            /* Configure event link resource to CMTW0
               ELSR33 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of CMTW0 when event link is run
               ELOPH - Event link option select register H
               b1:b0 CMT0WMD - CMTW0 action select bit - Write 0x03 to event disable */
            ELC.ELSR33.BYTE = (uint8_t)ELC_ELSR_INIT;
            ELC.ELOPH.BIT.CMTW0MD = (uint8_t)ELC_ELOPH_NONE_EVENT;
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_cmtw_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_tpu_param
* Description  : Set TPU parameter of event link.
*
* Arguments    : p_tpu_param -
*                    Event link parameter of TPU.
* Return Value : None.
*******************************************************************************/
static void elc_set_tpu_param(const elc_cmd_tpu_t * const p_tpu_param)
{
    if (NULL == p_tpu_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_tpu_param->event_link_enable)
        {
            /* set event link parameter of TPU */
            /* Configure event link resource to TPU0, TPU1, TPU2, TPU3
               Correspondence relations of TPU and registers are TPU0 and ELSR35,
               TPU1 and ELSR36, TPU2 and ELSR37, TPU3 and ELSR38
               ELSR35, 36, 37, 38 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure action of TPU0, TPU1, TPU2, TPU3 when event link is run
               ELOPF - Event link option select register F
               b1:b0 TPU0MD - TPU0 action select bit - set user request value
               b3:b2 TPU1MD - TPU1 action select bit - set user request value
               b5:b4 TPU2MD - TPU2 action select bit - set user request value
               b7:b6 TPU3MD - TPU3 action select bit - set user request value */
            switch (p_tpu_param->elc_tpu_ch)
            {
                case ELC_TPU_CH_0:

                    ELC.ELSR35.BYTE = (uint8_t)p_tpu_param->resource;
                    ELC.ELOPF.BIT.TPU0MD = p_tpu_param->action;

                break;

                case ELC_TPU_CH_1:

                    ELC.ELSR36.BYTE = (uint8_t)p_tpu_param->resource;
                    ELC.ELOPF.BIT.TPU1MD = p_tpu_param->action;

                break;

                case ELC_TPU_CH_2:

                    ELC.ELSR37.BYTE = (uint8_t)p_tpu_param->resource;
                    ELC.ELOPF.BIT.TPU2MD = p_tpu_param->action;

                break;

                case ELC_TPU_CH_3:

                    ELC.ELSR38.BYTE = (uint8_t)p_tpu_param->resource;
                    ELC.ELOPF.BIT.TPU3MD = p_tpu_param->action;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of TPU */
            /* Configure event link resource to TPU0, TPU1, TPU2, TPU3
               Correspondence relations of TPU and registers are TPU0 and ELSR35,
               TPU1 and ELSR36, TPU2 and ELSR37, TPU3 and ELSR38
               ELSR35, 36, 37, 38 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of TPU0, TPU1, TPU2, TPU3 when event link is run
               ELOPF - Event link option select register F
               b1:b0 TPU0MD - TPU0 action select bit - Write 0x03 to event disable
               b3:b2 TPU1MD - TPU1 action select bit - Write 0x03 to event disable
               b5:b4 TPU2MD - TPU2 action select bit - Write 0x03 to event disable
               b7:b6 TPU3MD - TPU3 action select bit - Write 0x03 to event disable */
            switch (p_tpu_param->elc_tpu_ch)
            {
                case ELC_TPU_CH_0:

                    ELC.ELSR35.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPF.BIT.TPU0MD = (uint8_t)ELC_ELOPF_NONE_EVENT;

                break;

                case ELC_TPU_CH_1:

                    ELC.ELSR36.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPF.BIT.TPU1MD = (uint8_t)ELC_ELOPF_NONE_EVENT;

                break;

                case ELC_TPU_CH_2:

                    ELC.ELSR37.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPF.BIT.TPU2MD = (uint8_t)ELC_ELOPF_NONE_EVENT;

                break;

                case ELC_TPU_CH_3:

                    ELC.ELSR38.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPF.BIT.TPU3MD = (uint8_t)ELC_ELOPF_NONE_EVENT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_tpu_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_gpt_param
* Description  : Set GPT parameter of event link.
*
* Arguments    : p_gpt_param -
*                    Event link parameter of GPT.
* Return Value : None.
*******************************************************************************/
static void elc_set_gpt_param(const elc_cmd_gpt_t * const p_gpt_param)
{
    if (NULL == p_gpt_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        if (false != p_gpt_param->event_link_enable)
        {
            /* set event link parameter of GPT */
            /* Configure event link resource to GPT0, GPT1, GPT2, GPT3
               Correspondence relations of GPT and registers are GPT0 and ELSR41,
               GPT1 and ELSR42, GPT2 and ELSR43, GPT3 and ELSR44
               ELSR41, 42, 43, 44 - Event link coufigure register
               b7:b0 ELS - Event link select bit - HW init value (0x00) */
            /* Configure action of GPT0, GPT1 when event link is run
               ELOPH - Event link option select register H
               b2:b0 GPT0MD - GPT0 action select bit - set user request value
               b6:b4 GPT1MD - GPT1 action select bit - set user request value */
            /* Configure action of GPT2, GPT3 when event link is run
               ELOPJ - Event link option select register J
               b2:b0 GPT2MD - GPT2 action select bit - set user request value
               b6:b4 GPT3MD - GPT3 action select bit - set user request value */
            switch (p_gpt_param->elc_gpt_ch)
            {
                case ELC_GPT_CH_0:

                    ELC.ELSR41.BYTE = (uint8_t)p_gpt_param->resource;
                    ELC.ELOPI.BIT.GPT0MD = p_gpt_param->action;

                break;

                case ELC_GPT_CH_1:

                    ELC.ELSR42.BYTE = (uint8_t)p_gpt_param->resource;
                    ELC.ELOPI.BIT.GPT1MD = p_gpt_param->action;

                break;

                case ELC_GPT_CH_2:

                    ELC.ELSR43.BYTE = (uint8_t)p_gpt_param->resource;
                    ELC.ELOPJ.BIT.GPT2MD = p_gpt_param->action;

                break;

                case ELC_GPT_CH_3:

                    ELC.ELSR44.BYTE = (uint8_t)p_gpt_param->resource;
                    ELC.ELOPJ.BIT.GPT3MD = p_gpt_param->action;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
        else
        {
            /* clear event link parameter of GPT */
            /* Configure event link resource to GPT0, GPT1, GPT2, GPT3
               Correspondence relations of GPT and registers are GPT0 and ELSR41,
               GPT1 and ELSR42, GPT2 and ELSR43, GPT3 and ELSR44
               ELSR41, 42, 43, 44 - Event link coufigure register
               b7:b0 ELS - Event link select bit - set user request value */
            /* Configure action of GPT0, GPT1 when event link is run
               ELOPH - Event link option select register H
               b2:b0 GPT0MD - GPT0 action select bit - Write 0x03 to event disable
               b6:b4 GPT1MD - GPT1 action select bit - Write 0x03 to event disable */
            /* Configure action of GPT2, GPT3 when event link is run
               ELOPJ - Event link option select register J
               b2:b0 GPT2MD - GPT2 action select bit - Write 0x03 to event disable
               b6:b4 GPT3MD - GPT3 action select bit - Write 0x03 to event disable */
            switch (p_gpt_param->elc_gpt_ch)
            {
                case ELC_GPT_CH_0:

                    ELC.ELSR41.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPI.BIT.GPT0MD = (uint8_t)ELC_ELOPI_NONE_EVENT;

                break;

                case ELC_GPT_CH_1:

                    ELC.ELSR42.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPI.BIT.GPT1MD = (uint8_t)ELC_ELOPI_NONE_EVENT;

                break;

                case ELC_GPT_CH_2:

                    ELC.ELSR43.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPJ.BIT.GPT2MD = (uint8_t)ELC_ELOPJ_NONE_EVENT;

                break;

                case ELC_GPT_CH_3:

                    ELC.ELSR44.BYTE = (uint8_t)ELC_ELSR_INIT;
                    ELC.ELOPJ.BIT.GPT3MD = (uint8_t)ELC_ELOPJ_NONE_EVENT;

                break;

                default :

                    /* NOTREACHED on At the time of a normal performance */

                break;
            }
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_gpt_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_port_group_param
* Description  : Set setting of port group parameter.
* Arguments    : p_port_group_param -
*                    Port group parameter.
* Return Value : None.
*******************************************************************************/
static void elc_set_port_group_param(const elc_cmd_port_group_t * const p_port_group_param)
{
    if (NULL == p_port_group_param)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        /* set port group */
        switch (p_port_group_param->port_group_num)
        {
            case ELC_PORT_GROUP_NUM_0:

                ELC.PGR1.BYTE = p_port_group_param->port_group_bit;
                ELC.PGC1.BIT.PGCIn = p_port_group_param->trigger;

            break;

            case ELC_PORT_GROUP_NUM_1:

                ELC.PGR2.BYTE = p_port_group_param->port_group_bit;
                ELC.PGC2.BIT.PGCIn = p_port_group_param->trigger;

            break;

            default :

                /* NOTREACHED on At the time of a normal performance */

            break;
        }
    }

    return;
}
/*******************************************************************************
End of function elc_set_port_group_param
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_set_software_event
* Description  : Output operation of ELC software event.
* Arguments    : None.
* Return Value : None.
*******************************************************************************/
static void elc_set_software_event(void)
{
    /* Control ELC event
       ELSEGR - Event link software event occur register
       b0 SEG - software event occur bit - on output event : write 1 to output event
                                           on stop event : write 0 to stop event
       b6 WE - SEG bit write enable - on before access SEG : write 1 to SEG access enable
                                      on after acess SEG :write 0 to SEG access disable
       b7 WI - ELSEGR register write enable - on before access WE : write 1 to WE & SEG access enable
                                              on after acess WE :write 0 to WE & SEG access disable */

    /* write enable to SEG bit */
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_SET_SEG_ENABLE;

    /* outout ELC software event */
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_SET_SOFT_EVENT;

    /* reset ELC soft ware event */
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_CLR_SOFT_EVENT;

    /* write disable to SEG bit */
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_CLR_SEG_ENABLE;

    /* write disable to ELSEGR resgister */
    ELC.ELSEGR.BYTE = (uint8_t)ELC_ELSEGR_WRITE_DISABLE;

    return;
}
/*******************************************************************************
End of function elc_set_software_event
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_get_port_group_value
* Description  : Get value of input port group.
* Arguments    : p_get_port_value -
*                    Getting of Port value parameter.
* Return Value : None.
*******************************************************************************/
static void elc_get_port_group_value(elc_get_port_value_t * const p_get_port_value)
{
    if (NULL == p_get_port_value)
    {
        /* It can not happen NULL pointer before check is success, QAC measures */
    }
    else
    {
        /* get input group port value from PDBF register */
        switch (p_get_port_value->elc_port_group_num)
        {
            case ELC_PORT_GROUP_NUM_0:

                p_get_port_value->port_value = ELC.PDBF1.BYTE;

            break;

            case ELC_PORT_GROUP_NUM_1:

                p_get_port_value->port_value = ELC.PDBF2.BYTE;

            break;

            default :

                /* NOTREACHED on At the time of a normal performance */

            break;
        }
    }

    return;
}
/*******************************************************************************
End of function elc_get_port_group_value
*******************************************************************************/

/* End of File */

