/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : elc driver interface functions for elc fit module.
* File Name    : r_elc_if.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for ELC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "cmsis_os.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "r_elc.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static osSemaphoreDef(sem_elc_state);

/*******************************************************************************
* Function Name: R_ELC_Init
* Description  : This function initialize ELC driver.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_ELC_Init(void)
{
    if (elc_control.state == ELC_UNINIT)
    {
        elc_control.sem_state = osSemaphoreCreate(osSemaphore(sem_elc_state), 1);
        elc_control.state     = ELC_INIT;
    }
}
/*******************************************************************************
End of function R_ELC_Init
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_Uninit
* Description  : This function uninitialize ELC driver.
* Arguments    : None
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_Uninit(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    osStatus    sem_state_ercd;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state > ELC_UNINIT)
        {
            if (elc_control.state >= ELC_OPEN)
            {
                ELC_Close();        /* stop ELC module */
            }

            /* release semaphore */
            sem_state_ercd    = osSemaphoreDelete(elc_control.sem_state);
            if (sem_state_ercd != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Delete Error */
                ercd = EFAULT;
            }
            else
            {
            	elc_control.state = ELC_UNINIT;
            }
        }
        else
        {
            ercd = EACCES;
        }
        if(ercd != ESUCCESS)
        {
            sem_ret = osSemaphoreRelease(elc_control.sem_state);
		    if (sem_ret != osOK)
		    {
			    /* NON NOTICE ASSERT: Semaphore Release Error */
			    ercd = EFAULT;
		    }
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_Uninit
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_Open
* Description  : This function does open ELC driver.
*                ELC HW, driver information is initialized and module stop
*                control is set to OFF.
* Arguments    : None.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_Open(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    elc_err_t   err;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state == ELC_CLOSE)
        {
            err = ELC_Open();
            if (err != ELC_SUCCESS)
            {
                switch (err)
                {
                case ELC_ERR_MISSING_PTR:
                case ELC_ERR_INVALID_ARG:
                    ercd = EINVAL;
                    break;
                default:
                    ercd = EFAULT;
                    break;
                }
            }
        }
        else if (elc_control.state == ELC_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(elc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_Close
* Description  : This function does close ELC driver.
*                Module stop control is set to ON.
* Arguments    : None
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_Close(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state <= ELC_OPEN)
        {
            ELC_Close();
        }
        else if (elc_control.state == ELC_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(elc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_LinkStart
* Description  : Start ELC event link operation start.
* Arguments    : None.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_LinkStart(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    elc_err_t   err;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state == ELC_OPEN)
        {
            err = ELC_LinkStart();
            if (err != ELC_SUCCESS)
            {
                switch (err)
                {
                case ELC_ERR_MISSING_PTR:
                case ELC_ERR_INVALID_ARG:
                    ercd = EINVAL;
                    break;
                default:
                    ercd = EFAULT;
                    break;
                }
            }
        }
        else if (elc_control.state == ELC_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(elc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_LinkStart
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_LinkStop
* Description  : Stop ELC event link operation start.
* Arguments    : None.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_LinkStop(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    elc_err_t   err;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state == ELC_RUN)
        {
            err = ELC_LinkStop();
            if (err != ELC_SUCCESS)
            {
                switch (err)
                {
                case ELC_ERR_MISSING_PTR:
                case ELC_ERR_INVALID_ARG:
                    ercd = EINVAL;
                    break;
                default:
                    ercd = EFAULT;
                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(elc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_LinkStop
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_Control
* Description  : This function provides commands for changing ELC setting,
*                operating counter.
*
* Arguments    : cmd-
*                    Command to run
*                p_data-
*                    Pointer to optional configuration structure
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_ELC_Control(uint32_t const cmd, void * const p_data)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    elc_err_t   err;

    sem_ret = osSemaphoreWait(elc_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (elc_control.state >= ELC_CLOSE)
        {
            err = ELC_Control(cmd, p_data);
            if (err != ELC_SUCCESS)
            {
                switch (err)
                {
                case ELC_ERR_MISSING_PTR:
                case ELC_ERR_INVALID_ARG:
                    ercd = EINVAL;
                    break;
                case ELC_ERR_NOT_OPENED:
                case ELC_ERR_NOT_CLOSED:
                    ercd = EACCES;
                    break;
                default:
                    ercd = EFAULT;
                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(elc_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_ELC_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: R_ELC_GetVersion
* Description  : Returns ELC device driver version.
* Arguments    : none
* Return Value : version number
*******************************************************************************/
uint32_t R_ELC_GetVersion(void)
{
    uint32_t const version = ((ELC_VERSION_MAJOR << 16) | ELC_VERSION_MINOR);

    return version;
}
/*******************************************************************************
End of function R_ELC_GetVersion
*******************************************************************************/

/* End of File */

