/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : elc driver interrupt functions for elc fit module.
* File Name    : r_elc_int.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for ELC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_elc_int.h"
#include "r_elc.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*----------------------------------------------------------------------------*/
/*   Define ELC interrupt table structure.                                    */
/*----------------------------------------------------------------------------*/
typedef struct
{
    IRQn_Type   vic_irq;
    uint32_t    type;
    void *      intr_handler;
} elc_intr_t;

/*******************************************************************************
Private global variables and functions
*******************************************************************************/

static void elc_elci1_isr(void);
static void elc_elci2_isr(void);
static void elc_elci_isr_common(const uint32_t elc_intr_num);

static const elc_intr_t gb_elc_intr_table[ELC_INTR_NUM] =
{
    /* casting the interrupt handler pointer to a uint32_t type is valid structure,
       because interrupr handlel address is uint32_t value.
       And, value is passed to R_ICU_Regist() which is required the value of
       uint32_t type value */
    { ELCIRQ1_IRQn, VIC_TYPE_EDGE, (void *)elc_elci1_isr },
    { ELCIRQ2_IRQn, VIC_TYPE_EDGE, (void *)elc_elci2_isr }
};

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_intr_register
* Description  : Registration ELC interrupt information.
*
* Arguments    : elc_intr_num-
*                    ELC interrupt factor number.
*                intr_priority-
*                    ELC interrupt priority.
* Return Value : None.
*******************************************************************************/
void elc_intr_register(const uint32_t elc_intr_num, const int32_t intr_priority)
{
    InterruptHandlerRegister(gb_elc_intr_table[elc_intr_num].vic_irq,
                             (IRQHandler)gb_elc_intr_table[elc_intr_num].intr_handler);
    VIC_SetPriority(gb_elc_intr_table[elc_intr_num].vic_irq,
                    intr_priority);
    VIC_SetDetectType(gb_elc_intr_table[elc_intr_num].vic_irq,
                      gb_elc_intr_table[elc_intr_num].type);

    return;
}

/*******************************************************************************
End of function elc_intr_register
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_intr_enable
* Description  : Enable ELC interrupt.
* Arguments    : elc_intr_num-
*                    ELC interrupt factor number.
* Return Value : none
*******************************************************************************/
void elc_intr_enable(const uint32_t elc_intr_num)
{
    VIC_EnableIRQ(gb_elc_intr_table[elc_intr_num].vic_irq);

    return;
}
/*******************************************************************************
End of function elc_intr_enable
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_intr_disable
* Description  : Disable ELC interrupt.
* Arguments    : elc_intr_num-
*                    ELC interrupt factor number.
* Return Value : none
*******************************************************************************/
void elc_intr_disable(const uint32_t elc_intr_num)
{
    VIC_DisableIRQ(gb_elc_intr_table[elc_intr_num].vic_irq);

    return;
}
/*******************************************************************************
End of function elc_intr_disable
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_elci_isr_common
* Description  :
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void elc_elci_isr_common(const uint32_t elc_intr_num)
{
    if (NULL != elc_control.p_callback[elc_intr_num])
    {
        /* call function if valid function pointer */
        elc_control.p_callback[elc_intr_num]();
    }

    return;
}
/*******************************************************************************
End of function elc_elci_isr_common
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_elci1_isr
* Description  : ELC interrupt1 handler
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void elc_elci1_isr(void)
{
    elc_elci_isr_common(ELC_INTR_NUM_1);

    return;
}
/*******************************************************************************
End of function elc_elci1_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: elc_elci2_isr
* Description  : ELC interrupt2 handler
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void elc_elci2_isr(void)
{
    elc_elci_isr_common(ELC_INTR_NUM_2);

    return;
}
/*******************************************************************************
End of function elc_elci2_isr
*******************************************************************************/

/* End of File */

