/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : mtu3a driver functions for mtu3a fit module.
* File Name    : r_mtu3.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : spurce file for MTU3a driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using MTU3a on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_mtu3.h"

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Private global variables
*******************************************************************************/
/* Possible input clock divisors for internal clocking. Not all channels support all divisors. */
const uint32_t      g_mtu_clock_divisors[MTU_NUM_CLK_DIVISORS]            = { 1, 2, 4, 8, 16, 32, 64, 256, 1024 };
/* Flag to tell whether any channel of the mtu is currently in use. */
volatile uint8_t    g_num_channels_in_use = 0;
/* The selected timer clearing source. */
uint8_t             g_mtu_channel_clr_src[MTU_CHANNEL_MAX]                = { 0 };
/* Flags for do repeat or just once. */
uint8_t             g_mtu_channel_repeats[MTU_CHANNEL_MAX]                = { 0 };
/* Flags for do callback or not */
uint8_t             g_mtu_tgr_callbacks[MTU_CHANNEL_MAX][MTU_TIMERS_MAX];
uint8_t             g_mtu_tgi_icu_en_flags[MTU_CHANNEL_MAX][MTU_NUM_TGIS] = { 0 };
/* Allocate callback data storage for all channels. */
mtu_callback_data_t g_mtu_cb_data[MTU_CHANNEL_MAX];

/* Create table of available internal clock divisors for each channel, and their setting bits. */
const uint8_t g_chnl_clk_divs[MTU_CHANNEL_MAX][MTU_NUM_CLK_DIVISORS] =
    {
        MTU0_PCLK_DIVS,
        MTU1_PCLK_DIVS,
        MTU2_PCLK_DIVS,
        MTU3_PCLK_DIVS,
        MTU4_PCLK_DIVS,
        MTU5_PCLK_DIVS,
        MTU6_PCLK_DIVS,
        MTU7_PCLK_DIVS,
        MTU8_PCLK_DIVS
    };

/* Create table of available external clock source for each channel, and their setting bits. */
const uint8_t g_chnl_ext_clks[MTU_CHANNEL_MAX][MTU_NUM_EXT_CLK_SRCS] =
    {
        MTU0_EXT_CLKS,
        MTU1_EXT_CLKS,
        MTU2_EXT_CLKS,
        MTU3_EXT_CLKS,
        MTU4_EXT_CLKS,
        MTU5_EXT_CLKS,
        MTU6_EXT_CLKS,
        MTU7_EXT_CLKS,
        MTU8_EXT_CLKS
    };

/* Create table of available counter clearing sources for each channel, and their setting bits. */
const uint8_t g_chnl_clear_src[MTU_CHANNEL_MAX][MTU_NUM_CLR_SRCS] =
    {
        MTU0_CLR_SRC,
        MTU1_CLR_SRC,
        MTU2_CLR_SRC,
        MTU3_CLR_SRC,
        MTU4_CLR_SRC,
        MTU5_CLR_SRC,
        MTU6_CLR_SRC,
        MTU7_CLR_SRC,
        MTU8_CLR_SRC
    };

/* Create table of TGI ICU interrupt enable bit masks for each channel. */
const uint32_t g_mtu_tgi_vic_en_masks[MTU_CHANNEL_MAX][MTU_NUM_TGIS] =
    {
        MTU0_TGI_EN,
        MTU1_TGI_EN,
        MTU2_TGI_EN,
        MTU3_TGI_EN,
        MTU4_TGI_EN,
        MTU5_TGI_EN,
        MTU6_TGI_EN,
        MTU7_TGI_EN,
        MTU8_TGI_EN
    };

/* Create table of TGI ICU interrupt enable bit masks for each channel. */
const uint32_t g_mtu_tgi_vic_st_masks[MTU_CHANNEL_MAX][MTU_NUM_TGIS] =
    {
        MTU0_TGI_ST_MASK,
        MTU1_TGI_ST_MASK,
        MTU2_TGI_ST_MASK,
        MTU3_TGI_ST_MASK,
        MTU4_TGI_ST_MASK,
        MTU5_TGI_ST_MASK,
        MTU6_TGI_ST_MASK,
        MTU7_TGI_ST_MASK,
        MTU8_TGI_ST_MASK
    };

const void * g_mtu_intr_p[MTU_CHANNEL_MAX][MTU_NUM_TGIS] =
    {
        MTU0_INTR_P,
        MTU1_INTR_P,
        MTU2_INTR_P,
        MTU3_INTR_P,
        MTU4_INTR_P,
        MTU5_INTR_P,
        MTU6_INTR_P,
        MTU7_INTR_P,
        MTU8_INTR_P
    };

/* Create table of channel start register bits for each channel. */
const uint8_t g_mtu_tstr_bits[] =
    {
        (uint8_t)MTU_TSTR_CH0,
        (uint8_t)MTU_TSTR_CH1,
        (uint8_t)MTU_TSTR_CH2,
        (uint8_t)MTU_TSTR_CH3,
        (uint8_t)MTU_TSTR_CH4,
        (uint8_t)MTU_TSTR_CH5,
        (uint8_t)MTU_TSTR_CH6,
        (uint8_t)MTU_TSTR_CH7,
        (uint8_t)MTU_TSTR_CH8
    };

/* Create table of channel start register bits for each channel. */
const uint16_t g_mtu_group_bits[] =
    {
         (uint16_t)MTU_GRP_CH0,
         (uint16_t)MTU_GRP_CH1,
         (uint16_t)MTU_GRP_CH2,
         (uint16_t)MTU_GRP_CH3,
         (uint16_t)MTU_GRP_CH4,
         (uint16_t)PROHIBTID,
         (uint16_t)MTU_GRP_CH6,
         (uint16_t)MTU_GRP_CH7,
         (uint16_t)MTU_GRP_CH8,
    };

/* Static control structures for each channel */
#if MTU_CFG_USE_CH0 == 1
mtu_callback p_callback_mtu0;
mtu_timer_chnl_settings_t mtu0_tmr_settings;
const mtu_config_block_t g_mtu0_handle =
    {
        &p_callback_mtu0,       // pointer to callback function pointer.
        &mtu0_tmr_settings,     // location of variable timer settings struct.
        MTU0_REGS,              // Pointers to registers for this channel
        0,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN0,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN0      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH1 == 1
mtu_callback p_callback_mtu1;
mtu_timer_chnl_settings_t mtu1_tmr_settings;
const mtu_config_block_t g_mtu1_handle =
    {
        &p_callback_mtu1,       // pointer to callback function pointer.
        &mtu1_tmr_settings,     // location of variable timer settings struct.
        MTU1_REGS,              // Pointers to registers for this channel
        1,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN1,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN1      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH2 == 1
mtu_callback p_callback_mtu2;
mtu_timer_chnl_settings_t mtu2_tmr_settings;
const mtu_config_block_t g_mtu2_handle =
    {
        &p_callback_mtu2,       // pointer to callback function pointer.
        &mtu2_tmr_settings,     // location of variable timer settings struct.
        MTU2_REGS,              // Pointers to registers for this channel
        2,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN2,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN2      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH3 == 1
mtu_callback p_callback_mtu3;
mtu_timer_chnl_settings_t mtu3_tmr_settings;
const mtu_config_block_t g_mtu3_handle =
    {
        &p_callback_mtu3,       // pointer to callback function pointer.
        &mtu3_tmr_settings,     // location of variable timer settings struct.
        MTU3_REGS,              // Pointers to registers for this channel
        3,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN3,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN3      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH4 == 1
mtu_callback p_callback_mtu4;
mtu_timer_chnl_settings_t mtu4_tmr_settings;
const mtu_config_block_t g_mtu4_handle =
    {
        &p_callback_mtu4,       // pointer to callback function pointer.
        &mtu4_tmr_settings,     // location of variable timer settings struct.
        MTU4_REGS,              // Pointers to registers for this channel
        4,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN4,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN4      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH6 == 1
mtu_callback p_callback_mtu6;
mtu_timer_chnl_settings_t mtu6_tmr_settings;
const mtu_config_block_t g_mtu6_handle =
    {
        &p_callback_mtu6,       // pointer to callback function pointer.
        &mtu6_tmr_settings,     // location of variable timer settings struct.
        MTU6_REGS,              // Pointers to registers for this channel
        6,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN6,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN6      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH7 == 1
mtu_callback p_callback_mtu7;
mtu_timer_chnl_settings_t mtu7_tmr_settings;
const mtu_config_block_t g_mtu7_handle =
    {
        &p_callback_mtu7,       // pointer to callback function pointer.
        &mtu7_tmr_settings,     // location of variable timer settings struct.
        MTU7_REGS,              // Pointers to registers for this channel
        7,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN7,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN7      // Input capture filter clock setting.
    };
#endif

#if MTU_CFG_USE_CH8 == 1
mtu_callback p_callback_mtu8;
mtu_timer_chnl_settings_t mtu8_tmr_settings;
const mtu_config_block_t g_mtu8_handle =
    {
        &p_callback_mtu8,       // pointer to callback function pointer.
        &mtu8_tmr_settings,     // location of variable timer settings struct.
        MTU8_REGS,              // Pointers to registers for this channel
        8,                      // MTU channel number.
        MTU_IR_PRIORITY_CHAN8,  // Interrupt priority level.
        MTU_CFG_FILT_CHAN8      // Input capture filter clock setting.
    };
#endif

const mtu_handle_t g_mtu_handles[] =
    {
        #if MTU_CFG_USE_CH0 == 1
        (mtu_handle_t) &g_mtu0_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH1 == 1
        (mtu_handle_t) &g_mtu1_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH2 == 1
        (mtu_handle_t) &g_mtu2_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH3 == 1
        (mtu_handle_t) &g_mtu3_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH4 == 1
        (mtu_handle_t) &g_mtu4_handle,
        #else
        NULL,
        #endif
        NULL,
        #if MTU_CFG_USE_CH6 == 1
        (mtu_handle_t) &g_mtu6_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH7 == 1
        (mtu_handle_t) &g_mtu7_handle,
        #else
        NULL,
        #endif
        #if MTU_CFG_USE_CH8 == 1
        (mtu_handle_t) &g_mtu8_handle,
        #else
        NULL,
        #endif
    };

mtu_ch_info_t mtu_ch_info[MTU_CHANNEL_MAX] =
    {
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE },
        { MTU_CLOSE, MTU_MODE_NONE }
    };

mtu_control_t mtu_control =
    {
        MTU_UNINIT,
        0,
        {
            { MTU_CLOSE, { &mtu_ch_info[MTU_CHANNEL_3], &mtu_ch_info[MTU_CHANNEL_4] } },
            { MTU_CLOSE, { &mtu_ch_info[MTU_CHANNEL_6], &mtu_ch_info[MTU_CHANNEL_7] } }
        },
        {
            &mtu_ch_info[MTU_CHANNEL_0],
            &mtu_ch_info[MTU_CHANNEL_1],
            &mtu_ch_info[MTU_CHANNEL_2],
            &mtu_ch_info[MTU_CHANNEL_3],
            &mtu_ch_info[MTU_CHANNEL_4],
            &mtu_ch_info[MTU_CHANNEL_5],
            &mtu_ch_info[MTU_CHANNEL_6],
            &mtu_ch_info[MTU_CHANNEL_7],
            &mtu_ch_info[MTU_CHANNEL_8]
        }
    };

/*******************************************************************************
Private local function declarations
*******************************************************************************/
void        mtu_interrupts_enable(uint8_t channel);
void        mtu_interrupts_disable(uint8_t channel);
void        mtu_interrupts_clear(uint8_t channel);
void        mtu_interrupts_group_enable(uint16_t group);
void        mtu_interrupts_group_disable(uint16_t group);
bool_t      mtu_check_group(uint16_t group);
void        power_on_off (uint8_t on_or_off);
bool_t      mtu_calc_clock_divisor(uint8_t chan, uint8_t *div_idx, uint32_t frq_target);
uint16_t    mtu_calc_tgr_ticks(uint16_t pclk_div, uint32_t frq_target );
static void mtu_isr_common(mtu_channel_t channel, mtu_timer_num_t tgr);
void        mtu_interrupts_regster(uint8_t channel);

/*******************************************************************************
* Function Name: MTU_Control
* Description  : This function is responsible for handling special hardware or software operations for the MTU channel.
* Arguments    : channel-
*                   The channel number
*                cmd
*                   Enumerated command code
*                pcmd_data
*                   Pointer to the command-data structure parameter of type void that is used to reference the location
*                   of any data specific to the command that is needed for its completion.
* Return Value : mtu_err
*                           MTU_SUCCESS
*                           MTU_ERR_BAD_CHAN
*                           MTU_ERR_NULL_PTR
*                           MTU_ERR_UNKNOWN_CMD
*                           MTU_ERR_INVALID_ARG
*                           MTU_ERR_CH_NOT_OPENED
*******************************************************************************/
mtu_err_t MTU_Control(mtu_channel_t const channel,
                      mtu_cmd_t const     cmd,
                      void * const        pcmd_data)
{
    mtu_capture_status_t * p_capture_data;
    mtu_timer_status_t   * p_timer_data;
    mtu_group_t    * p_group_data;
    mtu_capture_set_edge_t  * p_cap_edge_data;
    uint16_t temp_byte;

    /* Command function data structure definitions. One for each command in mtu_timer_cmd_t. */
    mtu_handle_t  my_handle;
    mtu_timer_chnl_settings_t *pconfig; // Store a pointer to the user's config structure.

    #if MTU3_CFG_PARAM_CHECKING_ENABLE == 1
    if (MTU_CHANNEL_MAX <= channel)   // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }

    if (MTU_CHANNEL_5 == channel)   // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }


    switch(cmd) /* Check for valid command and data. */
    {
        case MTU_CMD_START:
        case MTU_CMD_STOP:
        case MTU_CMD_SAFE_STOP:
        case MTU_CMD_RESTART:

        break;

        case MTU_CMD_GET_STATUS:
        case MTU_CMD_SYNCHRONIZE:
        case MTU_CMD_SET_CAPT_EDGE:

            if ((NULL == pcmd_data) || (FIT_NO_PTR == pcmd_data))
            {
                return MTU_ERR_NULL_PTR;
            }

        break;

        default:

           /* Error, command not recognized. */
           return MTU_ERR_UNKNOWN_CMD;
    }

    if (mtu_control.p_ch_info[channel]->state != MTU_OPEN)
    {
        return MTU_ERR_CH_NOT_OPENED;
    }
    #endif

    my_handle = g_mtu_handles[channel];

    pconfig = my_handle->p_mtu_chnl_tmr_settings;

    switch(cmd)
    {
        case MTU_CMD_START:       // Activate clocking

            if ((NULL != pcmd_data) && (FIT_NO_PTR != pcmd_data)) // A channel group specifier was provided
            {
                p_group_data =  (mtu_group_t *)pcmd_data;

                temp_byte = (uint16_t) *p_group_data;

                if (!mtu_check_group(temp_byte) || (temp_byte & ~MTU_TSTR_MASK)) // Error in group parameter.
                {
                    return MTU_ERR_INVALID_ARG;
                }

                mtu_interrupts_group_enable(temp_byte);
                if (temp_byte & MTU_TSTRA_MASK)
                {
                    MTU.TSTRA.BYTE = (uint8_t)(temp_byte & MTU_TSTRA_MASK);  //Set the start bits for the group.
                }
                if ((temp_byte >> 8) & MTU_TSTRB_MASK)
                {
                    MTU.TSTRB.BYTE = (uint8_t)((temp_byte >> 8) & MTU_TSTRB_MASK);  //Set the start bits for the group.
                }
            }
            else    // Just this channel.
            {
                mtu_interrupts_enable(channel);
                if ((MTU_CHANNEL_6 != channel) && (MTU_CHANNEL_7 != channel))
                {
                    MTU.TSTRA.BYTE |= g_mtu_tstr_bits[channel];
                }
                else
                {
                    MTU.TSTRB.BYTE |= g_mtu_tstr_bits[channel];
                }
            }

        break;

        case MTU_CMD_STOP:       // Pause clocking

            if ((NULL != pcmd_data) && (FIT_NO_PTR != pcmd_data)) // A channel group specifier was provided
            {
                p_group_data =  (mtu_group_t *)pcmd_data;

                temp_byte = (uint16_t) *p_group_data;

                if (!mtu_check_group(temp_byte) || (temp_byte & ~MTU_TSTR_MASK)) // Error in group parameter.
                {
                    return MTU_ERR_INVALID_ARG;
                }

                mtu_interrupts_group_disable(temp_byte);
                if (temp_byte & MTU_TSTRA_MASK)
                {
                    MTU.TSTRA.BYTE = ~((uint8_t)(temp_byte & MTU_TSTRA_MASK));  //Set the start bits for the group.
                }
                if ((temp_byte >> 8) & MTU_TSTRB_MASK)
                {
                    MTU.TSTRB.BYTE = ~((uint8_t)((temp_byte >> 8) & MTU_TSTRB_MASK));  //Set the start bits for the group.
                }
            }
            else    // Just this channel.
            {
                mtu_interrupts_disable(channel);
                if ((MTU_CHANNEL_6 != channel) && (MTU_CHANNEL_7 != channel))
                {
                    MTU.TSTRA.BYTE &= (uint8_t)(~(g_mtu_tstr_bits[channel]));
                }
                else
                {
                    MTU.TSTRB.BYTE &= (uint8_t)(~(g_mtu_tstr_bits[channel]));
                }
            }

        break;

        case MTU_CMD_SAFE_STOP:   // Stop clocking and set outputs to safe state

            // First stop the clocking.
            if ((MTU_CHANNEL_6 != channel) && (MTU_CHANNEL_7 != channel))
            {
                MTU.TSTRA.BYTE &= (uint8_t)(~(g_mtu_tstr_bits[channel]));
            }
            else
            {
                MTU.TSTRB.BYTE &= (uint8_t)(~(g_mtu_tstr_bits[channel]));
            }
            // Now re-write the TIOR register to revert to 'initial' MTIOC output state.
            if ((0 != pconfig->timer_a.freq) && (MTU_ACTION_OUTPUT & pconfig->timer_a.actions.do_action))
            {
                *my_handle->regs.tiorh |= pconfig->timer_a.actions.output;  // Set bits in lower nibble
            }
            if ((0 != pconfig->timer_b.freq) && (MTU_ACTION_OUTPUT & pconfig->timer_b.actions.do_action))
            {
                *my_handle->regs.tiorh |=  (pconfig->timer_b.actions.output << 4); // Move bits to upper nibble
            }
            if ((0 != pconfig->timer_c.freq) && (MTU_ACTION_OUTPUT & pconfig->timer_c.actions.do_action))
            {
                *my_handle->regs.tiorl |= pconfig->timer_c.actions.output;  // Set bits in lower nibble
            }
            if ((0 != pconfig->timer_d.freq) && (MTU_ACTION_OUTPUT & pconfig->timer_d.actions.do_action))
            {
                *my_handle->regs.tiorl |=  (pconfig->timer_d.actions.output << 4); // Move bits to upper nibble
            }

        break;

        case MTU_CMD_RESTART:      // Zero the counter then resume clocking

            *my_handle->regs.tcnt = 0;                 // Clear the counter TCNT register.
            mtu_interrupts_enable(channel);
            if ((MTU_CHANNEL_6 != channel) && (MTU_CHANNEL_7 != channel))
            {
                MTU.TSTRA.BYTE |= g_mtu_tstr_bits[channel];
            }
            else
            {
                MTU.TSTRB.BYTE |= g_mtu_tstr_bits[channel];
            }

        break;

        case MTU_CMD_GET_STATUS:   // Retrieve the current status of the channel

            if (MTU_MODE_COMPARE_MATCH == mtu_control.p_ch_info[channel]->mode)
            {
                /* Copy void pcmd_data pointer over to a concrete type. */
                p_timer_data =  (mtu_timer_status_t *)pcmd_data;
                /* Return timer status to application */
                if ((MTU_CHANNEL_6 != channel) && (MTU_CHANNEL_7 != channel))
                {
                    p_timer_data->timer_running = (bool_t)(MTU.TSTRA.BYTE & g_mtu_tstr_bits[channel]); // Running status
                }
                else
                {
                    p_timer_data->timer_running = (bool_t)(MTU.TSTRB.BYTE & g_mtu_tstr_bits[channel]); // Running status
                }

                p_timer_data->timer_count = *my_handle->regs.tcnt;    // The current timer count value.
            }
            else if (MTU_MODE_INPUT_CAPTURE == mtu_control.p_ch_info[channel]->mode)
            {
                /* Cast void pcmd_data pointer to a concrete type. */
                p_capture_data =  (mtu_capture_status_t *)pcmd_data;

                p_capture_data->timer_count = *my_handle->regs.tcnt;    // The current timer count value.

                /* Grab the TGR register values. */
                p_capture_data->capt_a_count = *my_handle->regs.tgra;
                p_capture_data->capt_b_count = *my_handle->regs.tgrb;
                /* Not all channels have TGRC and TGRD */
                if (NULL != my_handle->regs.tgrc)
                {
                    p_capture_data->capt_c_count = *my_handle->regs.tgrc;
                }
                else
                {
                    p_capture_data->capt_c_count = 0;
                }
                if (NULL != my_handle->regs.tgrd)
                {
                    p_capture_data->capt_d_count = *my_handle->regs.tgrd;
                }
                else
                {
                    p_capture_data->capt_d_count = 0;
                }
            }
            else
            {
                return MTU_ERR_INVALID_ARG;
            }

        break;

        case MTU_CMD_SET_CAPT_EDGE:      // Set the detection edge polarity for input capture.

            if (MTU_MODE_INPUT_CAPTURE == mtu_control.p_ch_info[channel]->mode)
            {
                 /* Cast void pcmd_data pointer to a concrete type. */
                p_cap_edge_data =  (mtu_capture_set_edge_t *)pcmd_data;

                if ((MTU_CHANNEL_1 == channel) || (MTU_CHANNEL_2 == channel))
                {
                    if ((MTU_CAP_SRC_C == p_cap_edge_data->capture_src) || (MTU_CAP_SRC_D == p_cap_edge_data->capture_src))
                    {
                        return MTU_ERR_INVALID_ARG; // Resource not present on these channels.
                    }
                }
                switch (p_cap_edge_data->capture_src)
                {
                    case MTU_CAP_SRC_A:

                        *my_handle->regs.tiorl &= 0xF0;                           // First clear the lower nibble.
                        *my_handle->regs.tiorh |= p_cap_edge_data->capture_edge;  // Set bits in lower nibble

                    break;

                    case MTU_CAP_SRC_B:

                        *my_handle->regs.tiorl &= 0x0F;                                  // First clear the upper nibble.
                        *my_handle->regs.tiorh |=  (p_cap_edge_data->capture_edge << 4); // Move bits to upper nibble

                    break;

                    case MTU_CAP_SRC_C:

                        *my_handle->regs.tiorl &= 0xF0;                           // First clear the lower nibble.
                        *my_handle->regs.tiorl |= p_cap_edge_data->capture_edge;  // Set bits in lower nibble

                    break;

                    case MTU_CAP_SRC_D:

                        *my_handle->regs.tiorl &= 0x0F;                                   // First clear the upper nibble.
                        *my_handle->regs.tiorl |= (p_cap_edge_data->capture_edge << 4);   // Move bits to upper nibble

                    break;

                    default:

                        //Nothing -- unreachable.

                    break;
                }
            }
            else    // Command not valid for this mode.
            {
                return MTU_ERR_INVALID_ARG;
            }

        break;

        case MTU_CMD_SYNCHRONIZE:

            /* Copy void pcmd_data pointer over to a concrete type. */
            p_group_data =  (mtu_group_t *)pcmd_data;

            temp_byte = (uint16_t) *p_group_data;
            temp_byte &= MTU_TSYR_MASK; // Protect reserved TSYR bits.

            if (!mtu_check_group(temp_byte))
            {
                return MTU_ERR_INVALID_ARG;
            }
            if ( temp_byte & MTU_TSYRA_MASK )
            {
                MTU.TSYRA.BYTE = (uint8_t)(temp_byte & MTU_TSYRA_MASK);  //Set the SYNCn 0-4 bits.
            }
            if ((temp_byte >> 8) & MTU_TSYRB_MASK)
            {
                MTU.TSYRB.BYTE = (uint8_t)((temp_byte >> 8) & MTU_TSYRB_MASK);  //Set the SYNCn 6-7 bits.
            }

        break;

        default:

            //Nothing -- unreachable.

        break;
    }

    return MTU_SUCCESS;
}
/*******************************************************************************
 End of function MTU_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: MTU_Close
* Description  : Removes power to the MTU channel designated by the handle and disables the associated interrupts.
* Arguments    : : channel-
*                   The channel number
* Return Value : mtu_err
*                           MTU_SUCCESS
*                           MTU_ERR_BAD_CHAN
*                           MTU_ERR_CH_NOT_OPENED
*******************************************************************************/
mtu_err_t MTU_Close(mtu_channel_t const channel)
{
    bool_t      ret;

    #if MTU3_CFG_PARAM_CHECKING_ENABLE == 1
    if (MTU_CHANNEL_MAX <= channel)           // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }
    if (MTU_CHANNEL_5 == channel)
    {
        return MTU_ERR_BAD_CHAN;
    }
    #endif

    /* Check to see if the channel is currently initialized. */
    if ((MTU_UNINIT == mtu_control.state)
        || (MTU_MODE_CMPL_PWM == mtu_control.p_ch_info[channel]->mode)
        || (MTU_CLOSE >= mtu_control.p_ch_info[channel]->state))
    {
        /* This channel is not open so need not be closed. */
        return MTU_ERR_CH_NOT_OPENED;
    }

    /* Stop any current operation. */
    MTU_Control((mtu_channel_t)channel, MTU_CMD_STOP, FIT_NO_PTR);

    mtu_interrupts_disable(channel);

    mtu_control.p_ch_info[channel]->mode  = MTU_MODE_NONE;
    mtu_control.p_ch_info[channel]->state = MTU_CLOSE;

    ret = mtu_check_close();
    if (ret == true)
    {
        power_on_off(MTU_POWER_OFF);
    }

    /* Clear all control flags arrays for this channel. */
    g_mtu_channel_repeats[channel] = 0;
    g_mtu_channel_clr_src[channel] = 0;

    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_A] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_B] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_C] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_D] = 0;

    g_mtu_tgr_callbacks[channel][MTU_TIMER_A] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_B] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_C] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_D] = 0;

    return MTU_SUCCESS;
}
/*******************************************************************************
 End of function MTU_Close
*******************************************************************************/

/*******************************************************************************
Private MTU function definitions
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_calc_input_clocks
* Description  : local helper function. Figures out the register settings for clocking divisor and TGR count.
* Arguments    : frq_target -
*                   The requested frequency.
* Return Value : none
*******************************************************************************/
bool_t mtu_calc_clock_divisor(uint8_t chan, uint8_t *div_idx, uint32_t frq_target )
{
    uint32_t i;
    bool_t result = false;
    uint16_t pclk_div;
    uint16_t tgr_try;
    uint32_t tgr_try2;

    if ((uint32_t)RZT1_PCLKB_HZ >= frq_target) /* Requested frequency must not be higher than PCLK. */
    {
        /* Check the available dividers to see if we can match the frequency requested by the user. */
        for (i = 0; i < MTU_NUM_CLK_DIVISORS; i++)
        {
            /* First check to see if this channel supports this divisor. */
            if (0xFF != g_chnl_clk_divs[chan][i]) // 0xFF means not available.
            {
                pclk_div = g_mtu_clock_divisors[i];

                /* Determine minimum frequency this divider can hit. For example, if a PCLK/16 is used and PCLK is 150MHz, then
                   the minimum frequency we can support is around 143.0533HzHz. This obtained by doing the following calculation:
                   (PCLK / divider) / max_counter_value
                   Example:
                   (150,000,000 / 16) / 65,535 = 143.0533Hz */

                if (MTU_CHANNEL_8 != chan)
                {
                    tgr_try = mtu_calc_tgr_ticks(pclk_div, frq_target);

                    if (tgr_try != 0)
                    {
                        /* We can use this divisor. Return clock divisor to be used. */
                        *div_idx = i;

                        /* A valid divisor was found. */
                        result = true;

                        /* No need to check other divisors. */
                        break;
                    }
                }
                else
                {
                    tgr_try2 = mtu_calc_tgr_ticks2(pclk_div, frq_target);

                    if (tgr_try2 != 0)
                    {
                        /* We can use this divisor. Return clock divisor to be used. */
                        *div_idx = i;

                        /* A valid divisor was found. */
                        result = true;

                        /* No need to check other divisors. */
                        break;
                    }
                }
            } // 0xFF
        } // for
    }

    return result;
}
/*******************************************************************************
 End of function mtu_calc_clock_divisor
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_calc_tgr_ticks
* Description  : local helper function. Calculate the TGR tick count based on frequency target and fixed PCLK divisor.
* Arguments    : frq_target -
*                   The requested freq.
* Return Value : the TGR value on success, 0 on failure.
*******************************************************************************/
uint16_t mtu_calc_tgr_ticks(uint16_t pclk_div, uint32_t frq_target )
{
    uint16_t result = 0;
    uint32_t clock_freq;

    clock_freq = RZT1_PCLKB_HZ / pclk_div; /* Pre-scale the clock */

    if ((frq_target > (clock_freq / MTU_MAX_TIMER_TICKS)) && (frq_target < clock_freq))
    {
        /* Figure out counter ticks needed for this frequency, and return it */
        result = (uint16_t)((clock_freq/frq_target) -1 );
    }

    return result;
}
/*******************************************************************************
 End of function mtu_calc_tgr_ticks
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_calc_tgr_ticks2
* Description  : local helper function. Calculate the TGR tick count based on frequency target and fixed PCLK divisor.
* Arguments    : frq_target -
*                   The requested freq.
* Return Value : the TGR value on success, 0 on failure.
*******************************************************************************/
uint32_t mtu_calc_tgr_ticks2(uint16_t pclk_div, uint32_t frq_target )
{
    uint32_t result = 0;
    uint32_t clock_freq;

    clock_freq = RZT1_PCLKB_HZ / pclk_div; /* Pre-scale the clock */

    if ((frq_target > (clock_freq / MTU_MAX_TIMER_TICKS2)) && (frq_target < clock_freq))
    {
        /* Figure out counter ticks needed for this frequency, and return it */
        result = (uint32_t)((clock_freq/frq_target) -1 );
    }

    return result;
}
/*******************************************************************************
 End of function mtu_calc_tgr_ticks
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_channel_clear
* Description  : Clears the registers and state variables of the given channel.
* Arguments    : channel -
*                 Which channel to clear.
* Return Value : none
*******************************************************************************/
void mtu_channel_clear(uint8_t channel)
{
    mtu_handle_t  my_handle = g_mtu_handles[channel];

    /* Clear the software control flags */
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_A] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_B] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_C] = 0;
    g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_D] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_A] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_B] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_C] = 0;
    g_mtu_tgr_callbacks[channel][MTU_TIMER_D] = 0;

    /* Clear all relevant MTU registers. */
    *my_handle->regs.tmdr = 0x00;       // Clear the mode register.
    *my_handle->regs.tiorh = 0x00;      // Clear TIORs to disable outputs.

    if (NULL != my_handle->regs.tiorl)
    {
        *my_handle->regs.tiorl = 0x00;  // Some channels do not have this.
    }

    *my_handle->regs.tier = 0x00;
    *my_handle->regs.tcnt = 0x00;       // Clear the count in the TCNT register.
    *my_handle->regs.tgra = 0x00;       // Clear TGRs.
    *my_handle->regs.tgrb = 0x00;

    if (NULL != my_handle->regs.tgrc)
    {
        *my_handle->regs.tgrc = 0x00;   // Some channels do not have this.
    }

    if (NULL != my_handle->regs.tgrd)
    {
        *my_handle->regs.tgrd = 0x00;   // Some channels do not have this.
    }
}
/*******************************************************************************
 End of function mtu_channel_clear
*******************************************************************************/

/*******************************************************************************
* Function Name: power_on_off
* Description  : Switches power to an MTU channel.  Required by FIT spec.
* Arguments    : channel -
*                   Which channel to use.
*                on_or_off -
*                   What it says.
* Return Value : none
*******************************************************************************/
void power_on_off(uint8_t on_or_off)
{
    volatile uint32_t dummy32t;
    volatile uint8_t dummy8t;

    R_RST_WriteEnable();
    MSTP(MTU3) = on_or_off;
    if( on_or_off == MTU_POWER_ON )
    {
       dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
    }
    R_RST_WriteDisable();
    if( on_or_off == MTU_POWER_ON )
    {
       dummy8t = MTU0.TCR.BYTE;        /* Workaround for lack of release procedure from the module-stop state(2/2) */
    }
}
/*******************************************************************************
 End of function power_on_off
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_interrupts_enable
* Description  : Enable all MTU interrupts in ICU for the selected channel that are configured to be used.
* Arguments    : channel -
*                 Which channel to use.
* Return Value : none
*******************************************************************************/
void mtu_interrupts_enable(uint8_t channel)
{
    if ((MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A]) && (g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_A]))
    {
        VIC_EnableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A]);
    }
    if ((MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B]) && (g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_B]))
    {
        VIC_EnableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B]);
    }
    if ((MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C]) && (g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_C]))
    {
        VIC_EnableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C]);
    }
    if ((MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D]) && (g_mtu_tgi_icu_en_flags[channel][MTU_TIMER_D]))
    {
        VIC_EnableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D]);
    }
}
/*******************************************************************************
 End of function mtu_interrupts_enable
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_interrupts_disable
* Description  : Disable all MTU interrupts in ICU for the selected channel.
* Arguments    : channel -
*                 Which channel to use.
* Return Value : none
*******************************************************************************/
void mtu_interrupts_disable(uint8_t channel)
{
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A])
    {
        VIC_DisableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A]);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B])
    {
        VIC_DisableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B]);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C])
    {
        VIC_DisableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C]);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D])
    {
        VIC_DisableIRQ((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D]);
    }
}
/*******************************************************************************
 End of function mtu_interrupts_disable
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_interrupts_regster
* Description  : MTU interrupts in ICU for the selected channel.
* Arguments    : channel -
*                 Which channel to use.
* Return Value : none
*******************************************************************************/
void mtu_interrupts_regster(uint8_t channel)
{
    mtu_handle_t       my_handle = g_mtu_handles[channel];

    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A])
    {
        InterruptHandlerRegister((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A],
                                 (IRQHandler)g_mtu_intr_p[channel][MTU_TIMER_A]);
        VIC_SetPriority((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A],
                        my_handle->priority);
        VIC_SetDetectType((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_A],
                          VIC_TYPE_EDGE);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B])
    {
        InterruptHandlerRegister((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B],
                                 (IRQHandler)g_mtu_intr_p[channel][MTU_TIMER_B]);
        VIC_SetPriority((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B],
                        my_handle->priority);
        VIC_SetDetectType((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_B],
                          VIC_TYPE_EDGE);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C])
    {
        InterruptHandlerRegister((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C],
                                 (IRQHandler)g_mtu_intr_p[channel][MTU_TIMER_C]);
        VIC_SetPriority((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C],
                        my_handle->priority);
        VIC_SetDetectType((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_C],
                          VIC_TYPE_EDGE);
    }
    if (MTU_NOT_SUP2 != g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D])
    {
        InterruptHandlerRegister((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D],
                                 (IRQHandler)g_mtu_intr_p[channel][MTU_TIMER_D]);
        VIC_SetPriority((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D],
                        my_handle->priority);
        VIC_SetDetectType((IRQn_Type)g_mtu_tgi_vic_en_masks[channel][MTU_TIMER_D],
                          VIC_TYPE_EDGE);
    }
}
/*******************************************************************************
 End of function mtu_interrupts_regster
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_interrupts_group_enable
* Description  : Enable all MTU interrupts in ICU for the selected channels..
* Arguments    : channel -
*                 Which channel to use.
* Return Value : none
*******************************************************************************/
void mtu_interrupts_group_enable(uint16_t group)
{
    uint8_t i;

    for (i = 0; i < MTU_CHANNEL_MAX; i++)
    {
        if (g_mtu_group_bits[i] & group)
        {
            mtu_interrupts_enable(i);
        }
    }
}
/*******************************************************************************
 End of function mtu_interrupts_group_enable
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_interrupts_group_disable
* Description  : Disable all MTU interrupts in ICU for the selected channels.
* Arguments    : group -
*                 Which channels to disable.
* Return Value : none
*******************************************************************************/
void mtu_interrupts_group_disable(uint16_t group)
{
    uint8_t i;

    for (i = 0; i < MTU_CHANNEL_MAX; i++)
    {
        if (g_mtu_group_bits[i] & group)
        {
            mtu_interrupts_disable(i);
        }
    }
}
/*******************************************************************************
 End of function mtu_interrupts_group_disable
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_check_group
* Description  : Checks all MTU channels in the group for open status.
* Arguments    : group -
*                 Which channels to check.
* Return Value : true if all members of group are open, false if any member closed.
*******************************************************************************/
bool_t mtu_check_group(uint16_t group)
{
    uint8_t i;

    if (0 == group)
    {
        return false; // Group cannot be empty.
    }

    for (i = 0; i < MTU_CHANNEL_MAX; i++)
    {
        if (g_mtu_group_bits[i] & group)
        {
            if (MTU_CLOSE == mtu_control.p_ch_info[i]->state)
            {
                return false; // Error, channel is closed.
            }
        }
    }

    return true;
}
/*******************************************************************************
 End of function mtu_check_group
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_check_close
* Description  : This function check close all MTU chnnel.
* Arguments :    none
* Return Value : none
*******************************************************************************/
bool_t mtu_check_close(void)
{
    uint8_t     i;
    for (i = 0; i < MTU_CHANNEL_MAX; i++)
    {
        if (mtu_control.p_ch_info[i]->mode != MTU_MODE_NONE)
        {
            return false;
        }
    }
    return true;
}
/*******************************************************************************
 End of function mtu_check_close
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_isr_common
* Description  :  the common handler for all TGR interrupts.
*                 Checks whether the mtu repeats the cycle or needs to stop the timer now.
*                 Calls the user defined callback if enabled.
* Arguments : channel -
*                 Which channel to use.
*             tgr -
*                 Which TGR had the interrupt.
* Return Value : none
*******************************************************************************/
void mtu_isr_common(mtu_channel_t channel, mtu_timer_num_t tgr_num)
{
    /* See if we need to stop the timer now. Only the clear source TGR is allowed to control the oneshot mode. */
    if ((0 == g_mtu_channel_repeats[channel]) && (tgr_num == g_mtu_channel_clr_src[channel]))
    {
        MTU_Control(channel, MTU_CMD_STOP, FIT_NO_PTR);
    }

    /* Do the callback for this interrupt if it is enabled. */
    if (1 == g_mtu_tgr_callbacks[channel][tgr_num])
    {
        /* Store the event data. */
        g_mtu_cb_data[channel].channel = channel;
        g_mtu_cb_data[channel].timer_num = tgr_num;
        /* Get the TGR value.
        * Contains the counter value at time of event when doing input capture operation.
        * Contains the TGR preset (compare value) when doing compare/match operation. */
        g_mtu_cb_data[channel].count = (uint32_t)(*(g_mtu_handles[channel]->regs.tgra + tgr_num));

        /* Call the User defined callback function. */
        (*(g_mtu_handles[channel]->p_callback))((void*)&(g_mtu_cb_data[channel])); // Pass the event data.
    }
}
/*******************************************************************************
 End of function mtu_isr_common
*******************************************************************************/

/*******************************************************************************
* Description  : MTU interrupt handler routines.
*******************************************************************************/
#if MTU_CFG_USE_CH0 == 1
/*******************************************************************************
* Function Name: mtu_tgia0_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia0_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_0, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia0_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib0_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib0_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_0, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib0_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic0_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic0_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_0, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic0_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid0_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid0_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_0, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid0_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH0 */

#if MTU_CFG_USE_CH1 == 1
/*******************************************************************************
* Function Name: mtu_tgia1_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia1_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_1, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia1_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib1_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib1_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_1, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib1_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH1 */

#if MTU_CFG_USE_CH2 == 1
/*******************************************************************************
* Function Name: mtu_tgia2_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia2_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_2, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia2_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib2_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib2_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_2, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib2_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH2 */

#if MTU_CFG_USE_CH3 == 1
/*******************************************************************************
* Function Name: mtu_tgia3_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia3_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_3, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia3_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib3_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib3_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_3, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib3_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic3_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic3_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_3, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic3_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid3_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid3_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_3, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid3_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH3 */

#if MTU_CFG_USE_CH4 == 1
/*******************************************************************************
* Function Name: mtu_tgia4_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia4_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_4, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia4_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib4_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib4_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_4, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib4_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic4_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic4_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_4, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic4_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid4_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid4_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_4, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid4_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH4 */

#if MTU_CFG_USE_CH6 == 1
/*******************************************************************************
* Function Name: mtu_tgia6_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia6_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_6, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia6_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib6_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib6_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_6, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib6_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic6_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic6_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_6, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic6_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid6_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid6_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_6, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid6_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH6 */

#if MTU_CFG_USE_CH7 == 1
/*******************************************************************************
* Function Name: mtu_tgia7_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia7_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_7, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia7_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib7_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib7_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_7, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib7_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic7_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic7_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_7, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic7_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid7_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid7_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_7, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid7_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH7 */

#if MTU_CFG_USE_CH8 == 1
/*******************************************************************************
* Function Name: mtu_tgia8_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgia8_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_8, MTU_TIMER_A);
}
/*******************************************************************************
End of function mtu_tgia8_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgib8_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgib8_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_8, MTU_TIMER_B);
}
/*******************************************************************************
End of function mtu_tgib8_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgic8_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgic8_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_8, MTU_TIMER_C);
}
/*******************************************************************************
End of function mtu_tgic8_isr
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_tgid8_isr
* Description  : Interrupt input capture / compare match.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void mtu_tgid8_isr(void)
{
    mtu_isr_common(MTU_CHANNEL_8, MTU_TIMER_D);
}
/*******************************************************************************
End of function mtu_tgid8_isr
*******************************************************************************/
#endif /* MTU_CFG_USE_CH8 */

/* End of File */

