/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : mtu3a driver compliment pwm mode functions for mtu3a fit module.
* File Name    : r_mtu3_cmpl_pwm.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : spurce file for MTU3a driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using MTU3a on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_mtu3.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define MTU_CLK_DIV_1         (0x00)   // PCLK/1
#define MTU_CLK_DIV_4         (0x01)   // PCLK/4
#define MTU_CLK_DIV_16        (0x02)   // PCLK/16
#define MTU_CLK_DIV_64        (0x03)   // PCLK/64
#define MTU_CLK_DIV_256       (0x04)   // PCLK/256
#define MTU_CLK_DIV_1024      (0x05)   // PCLK/1024
#define MTU_CLK_DIV_2         (0x01)   // PCLK/2
#define MTU_CLK_DIV_8         (0x02)   // PCLK/8
#define MTU_CLK_DIV_32        (0x03)   // PCLK/32
#define MTU_CLK_TCR_0         (0x00)   // 0

#define MTU_CLK_MTCLKA        (0x06)   // MTCLKA
#define MTU_CLK_MTCLKB        (0x07)   // MTCLKB

#define MTU_TCNT_0            (0x0000) // TCNT 0

#define MTU_TCNT_1            (0)      // Output PWM Number1
#define MTU_TCNT_2            (1)      // Output PWM Number2
#define MTU_TCNT_3            (2)      // Output PWM Number3

#define MTU_DEADTIME_OFF      (0)      // Dead time disable
#define MTU_DEADTIME_ON       (1)      // Dead time enable

#define MTU_DEADTIME_DEF      (0x0001) // Dead time default

#define MTU_TMDR_BFA          (0x10)   // TGRA and TGRC used together for buffer operation
#define MTU_TMDR_BFB          (0x20)   // TGRB and TGRD used together for buffer operation

#define MTU_ENABLE_OUTPUT     (1)
#define MTU_DISABLE_OUTPUT    (0)

#define MTU_COUNT_STOP        (0)
#define MTU_CMPLMT_PWM_START  (0xC0)
#define MTU_CNT_UPDOWN        (0x80)
#define MTU_TMDR2_DRS         (0x01)

typedef enum {
    MTU_PROTECT_ENABLE = 0,
    MTU_PROTECT_DISABLE
} mtu_cmpl_pwm_protect_t;

typedef enum {
    MTU_MODULE_STOP_DIS = 0,
    MTU_MODULE_STOP_ENA = 1
} mtu_cmpl_pwm_module_t;

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/
// Pointers to timer configuration data

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static void mtu_cmpl_pwm_count_stop(mtu_cmpl_pwm_channel_t channel);
static void mtu_cmpl_pwm_clk_edge(mtu_cmpl_pwm_channel_t   channel,
                                  mtu_clk_src_t            clock_src,
                                  mtu_cmpl_pwm_clk_div_t   clk_div);
static void mtu_cmpl_pwm_count_set(mtu_cmpl_pwm_channel_t channel);
static void mtu_cmpl_pwm_duty_buff(mtu_cmpl_pwm_channel_t  channel,
                                   uint8_t                 output_no,
                                   uint16_t                cycle_freq,
                                   uint16_t                duty,
                                   uint16_t                dead_time,
                                   mtu_cmpl_pwm_d_bf_t     d_bf);
static void mtu_cmpl_pwm_deadtime(mtu_cmpl_pwm_channel_t   channel,
                                  uint16_t                 dead_time);
static void mtu_cmpl_pwm_carrier_cycle(mtu_cmpl_pwm_channel_t  channel,
                                       uint16_t                cycle_freq,
                                       uint16_t                dead_time);
static void mtu_cmpl_pwm_output_toggle(mtu_cmpl_pwm_channel_t  channel,
                                       mtu_cmpl_pwm_toggle_t   toggle);
static void mtu_cmpl_pwm_output_level_kind(mtu_cmpl_pwm_channel_t  channel,
                                           mtu_cmpl_pwm_p_n_t      p_n);
static void mtu_cmpl_pwm_output_level_set_1(mtu_cmpl_pwm_channel_t  channel,
                                            mtu_cmpl_pwm_settings_t pwm_output);
static void mtu_cmpl_pwm_output_level_set_2(mtu_cmpl_pwm_channel_t  channel,
                                            uint8_t                 output_no,
                                            mtu_cmpl_pwm_settings_t pwm_output);
static void mtu_cmpl_pwm_output_level_buff(mtu_cmpl_pwm_channel_t  channel,
                                           mtu_cmpl_pwm_p_n_bf_t   p_n_bf);
static void mtu_cmpl_pwm_mode(mtu_cmpl_pwm_channel_t  channel,
                              mtu_cmpl_pwm_mode_t     mode,
                              mtu_cmpl_pwm_d_bf_t     d_bf);
static void mtu_cmpl_pwm_output_enable(mtu_cmpl_pwm_channel_t   channel,
                                       mtu_cmpl_pwm_output_st_t output,
                                       uint8_t                  output_no);
static void mtu_cmpl_pwm_protect(mtu_cmpl_pwm_channel_t channel,
                                 mtu_cmpl_pwm_protect_t protect);

/*******************************************************************************
* Function Name: MTU_PWM_Compliment_Open
* Description  : This function is to supply power to the channel
*                of the MTU for complementary PWM, to initialize the
*                registers associated with preparing for comparison /
*                match timer operation, to apply the user-configurable options.
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
* Return Value : MTU_SUCCESS -
*                   Successful; channel initialized
*                MTU_ERR_BAD_CHAN -
*                   Channel number is invalid for part
*                MTU_ERR_CH_NOT_CLOSED -
*                   Channel currently in operation; Perform MTU_Close() first
*                MTU_ERR_NULL_PTR -
*                   pconfig pointer is NULL
*                MTU_ERR_INVALID_ARG -
*                   An element of the pconfig structure contains an invalid value.
*******************************************************************************/
mtu_err_t MTU_PWM_Compliment_Open(mtu_cmpl_pwm_channel_t const         channel,
                                  mtu_cmpl_pwm_chnl_settings_t * const pconfig)
{
#if MTU3_CFG_PARAM_CHECKING_ENABLE == 1
    /* channel check */
    if (MTU_CMPL_PWM_CHANNEL_MAX <= channel)           // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }

    /* pconfig pointer check */
    if (NULL == pconfig)
    {
        return MTU_ERR_NULL_PTR;
    }

    /* source check */
    if (MTU_CHANNEL_3_4 == channel)
    {
        if ((pconfig->clock_src.source == MTU_CLK_SRC_EXT_MTCLKC)
         || (pconfig->clock_src.source == MTU_CLK_SRC_EXT_MTCLKD)
         || (pconfig->clock_src.source == MTU_CLK_SRC_CASCADE))
        {
            return MTU_ERR_INVALID_ARG;
        }
    }
    else if (MTU_CHANNEL_6_7 == channel)
    {
        if (pconfig->clock_src.source != MTU_CLK_SRC_INTERNAL)
        {
            return MTU_ERR_INVALID_ARG;
        }
    }
    else
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* clock_edge check */
    if ((pconfig->clock_src.clock_edge != MTU_CLK_RISING_EDGE)
     && (pconfig->clock_src.clock_edge != MTU_CLK_FALLING_EDGE)
     && (pconfig->clock_src.clock_edge != MTU_CLK_ANY_EDGE))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* clock_div check */
    switch (pconfig->clk_div.clock_div)
    {
        case MTU_SRC_CLK_DIV_1:
        case MTU_SRC_CLK_DIV_4:
        case MTU_SRC_CLK_DIV_16:
        case MTU_SRC_CLK_DIV_64:
        case MTU_SRC_CLK_DIV_256:
        case MTU_SRC_CLK_DIV_1024:
        case MTU_SRC_CLK_DIV_2:
        case MTU_SRC_CLK_DIV_8:
        case MTU_SRC_CLK_DIV_32:

        break;

        default:

            return MTU_ERR_INVALID_ARG;

    }

    /* toggle check */
    if ((pconfig->toggle != MTU_TOGGLE_OFF)
     && (pconfig->toggle != MTU_TOGGLE_ON))
    {
         return MTU_ERR_INVALID_ARG;
    }

    /* PWM mode cheak */
    if ((pconfig->mode != MTU_CMPL_PWM_MODE_1)
     && (pconfig->mode != MTU_CMPL_PWM_MODE_2)
     && (pconfig->mode != MTU_CMPL_PWM_MODE_3))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* p_n cheak */
    if ((pconfig->p_n != MTU_PIN_P_N_1)
     && (pconfig->p_n != MTU_PIN_P_N_2))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* p_n_bf check */
    if ((pconfig->p_n_bf != MTU_PIN_P_N_BF_OFF)
     && (pconfig->p_n_bf != MTU_PIN_P_N_BF_CREST)
     && (pconfig->p_n_bf != MTU_PIN_P_N_BF_TROUGH)
     && (pconfig->p_n_bf != MTU_PIN_P_N_BF_CREST_TROUGH))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* d_bf check */
    if ((pconfig->d_bf != MTU_CMPL_PWM_D_BF_OFF)
     && (pconfig->d_bf != MTU_CMPL_PWM_D_BF_ON))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* pwm_output_1 check */
    if ((pconfig->pwm_output_1.olsp != MTU_PIN_OLSP_HI_UPLO_DNHI)
     && (pconfig->pwm_output_1.olsp != MTU_PIN_OLSP_LO_UPHI_DNLO))
    {
        return MTU_ERR_INVALID_ARG;
    }

    if ((pconfig->pwm_output_1.olsn != MTU_PIN_OLSN_HI_UPHI_DNLO)
     && (pconfig->pwm_output_1.olsn != MTU_PIN_OLSN_LO_UPLO_DNHI))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* pwm_output_2 check */
    if ((pconfig->pwm_output_2.olsp != MTU_PIN_OLSP_HI_UPLO_DNHI)
     && (pconfig->pwm_output_2.olsp != MTU_PIN_OLSP_LO_UPHI_DNLO))
    {
        return MTU_ERR_INVALID_ARG;
    }

    if ((pconfig->pwm_output_2.olsn != MTU_PIN_OLSN_HI_UPHI_DNLO)
     && (pconfig->pwm_output_2.olsn != MTU_PIN_OLSN_LO_UPLO_DNHI))
    {
        return MTU_ERR_INVALID_ARG;
    }


    /* pwm_output_3 check */
    if ((pconfig->pwm_output_3.olsp != MTU_PIN_OLSP_HI_UPLO_DNHI)
     && (pconfig->pwm_output_3.olsp != MTU_PIN_OLSP_LO_UPHI_DNLO))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* Output Level check */
    if ((pconfig->pwm_output_3.olsn != MTU_PIN_OLSN_HI_UPHI_DNLO)
     && (pconfig->pwm_output_3.olsn != MTU_PIN_OLSN_LO_UPLO_DNHI))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* Complementary PWM Close check */
    if ((MTU_CLOSE != mtu_control.cmpl_pwm_ch_info[channel].state)
        || (MTU_MODE_NONE != mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->mode)
        || (MTU_MODE_NONE != mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->mode))
    {
        return MTU_ERR_CH_NOT_CLOSED;
    }


#endif // MTU3_CFG_PARAM_CHECKING_ENABLE

    /* Release of register protect */
    mtu_cmpl_pwm_protect(channel, MTU_PROTECT_DISABLE);

    /* Release of the module stop */
    power_on_off(MTU_POWER_ON);

    /* Stop count operation */
    mtu_cmpl_pwm_count_stop(channel);

    /* Counter clock, counter clear source selection */
    mtu_cmpl_pwm_clk_edge(channel, pconfig->clock_src, pconfig->clk_div);

    /* TCNT setting */
    mtu_cmpl_pwm_count_set(channel);

    /* TGR setting output 1-3 */
    mtu_cmpl_pwm_duty_buff(channel, MTU_TCNT_1,
                           pconfig->clk_div.cycle_freq,
                           pconfig->pwm_output_1.duty,
                           pconfig->dead_time,
                           pconfig->d_bf);

    mtu_cmpl_pwm_duty_buff(channel, MTU_TCNT_2,
                           pconfig->clk_div.cycle_freq,
                           pconfig->pwm_output_2.duty,
                           pconfig->dead_time,
                           pconfig->d_bf);

    mtu_cmpl_pwm_duty_buff(channel, MTU_TCNT_3,
                           pconfig->clk_div.cycle_freq,
                           pconfig->pwm_output_3.duty,
                           pconfig->dead_time,
                           pconfig->d_bf);

    /* Enable/disable dead time generation */
    mtu_cmpl_pwm_deadtime(channel, pconfig->dead_time);

    /* carrier cycle setting */
    mtu_cmpl_pwm_carrier_cycle(channel, pconfig->clk_div.cycle_freq, pconfig->dead_time);

    /* Enable PWM cyclic output, set PWM output level */
    mtu_cmpl_pwm_output_toggle(channel, pconfig->toggle);

    mtu_cmpl_pwm_output_level_kind(channel, pconfig->p_n);
    if (MTU_PIN_P_N_1 ==  pconfig->p_n)
    {
        mtu_cmpl_pwm_output_level_set_1(channel, pconfig->pwm_output_1);
    }
    else
    {
        mtu_cmpl_pwm_output_level_set_2(channel, MTU_TCNT_1,  pconfig->pwm_output_1);
        mtu_cmpl_pwm_output_level_set_2(channel, MTU_TCNT_2,  pconfig->pwm_output_2);
        mtu_cmpl_pwm_output_level_set_2(channel, MTU_TCNT_3,  pconfig->pwm_output_3);
        mtu_cmpl_pwm_output_level_buff(channel, pconfig->p_n_bf);
    }

    /* Complementary PWM mode setting */
    mtu_cmpl_pwm_mode(channel, pconfig->mode, pconfig->d_bf);

    /* Enable waveform output */
    mtu_cmpl_pwm_output_enable(channel, pconfig->pwm_output_1.output, MTU_TCNT_1);
    mtu_cmpl_pwm_output_enable(channel, pconfig->pwm_output_2.output, MTU_TCNT_2);
    mtu_cmpl_pwm_output_enable(channel, pconfig->pwm_output_3.output, MTU_TCNT_3);

    if (MTU_CMPL_PWM_PROTECT_ON == pconfig->protect)
    {
        /* Prohibition of register protect */
        mtu_cmpl_pwm_protect(channel, MTU_PROTECT_ENABLE);
    }

    /* Status CLOSE -> OPEN */
    mtu_control.cmpl_pwm_ch_info[channel].state = MTU_OPEN;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->state = MTU_OPEN;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->state = MTU_OPEN;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->mode  = MTU_MODE_CMPL_PWM;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->mode  = MTU_MODE_CMPL_PWM;

    return MTU_SUCCESS;
}
/*******************************************************************************
 End of function MTU_PWM_Compliment_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: MTU_PWM_Compliment_Close
* Description  : Removes power to the MTU channel designated by the handle and
*                disables the associated interrupts.
* Arguments    : channel-
*                   The channel number
* Return Value : MTU_SUCCESS -
*                    Successful; channel closed
*                MTU_ERR_CH_NOT_OPEN -
*                    The channel has not been opened so closing has no effect.
*                MTU_ERR_BAD_CHAN -
*                    Channel number is invalid for part
*******************************************************************************/
mtu_err_t MTU_PWM_Compliment_Close(mtu_cmpl_pwm_channel_t const channel)
{
    bool_t      ret;

#if MTU3_CFG_PARAM_CHECKING_ENABLE == 1
    /* channel check */
    if (MTU_CMPL_PWM_CHANNEL_MAX <= channel)  // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }

    /* source check */
    if ((MTU_CHANNEL_3_4 != channel)
     && (MTU_CHANNEL_6_7 != channel))
    {
        return MTU_ERR_BAD_CHAN;
    }

    /* Open check */
    if (MTU_OPEN < mtu_control.cmpl_pwm_ch_info[channel].state)
    {
        return MTU_ERR_CH_NOT_OPENED;
    }

#endif /* MTU3_CFG_PARAM_CHECKING_ENABLE */

    /* Count Stop */
    MTU_PWM_Compliment_Control(channel, MTU_CMD_STOP, NULL);

    /* Status OPEN -> CLOSE */
    mtu_control.cmpl_pwm_ch_info[channel].state = MTU_CLOSE;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->state = MTU_CLOSE;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->state = MTU_CLOSE;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->mode  = MTU_MODE_NONE;
    mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->mode  = MTU_MODE_NONE;

    /* Module Stop */
    ret = mtu_check_close();
    if (true == ret)
    {
        power_on_off(MTU_POWER_OFF);
    }

    return MTU_SUCCESS;
}
/*******************************************************************************
 End of function MTU_PWM_Compliment_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: MTU_Conmpliment_PWM_Control
* Description  : This function, the start of the channel of the MTU for
*                complementary PWM, stops, and information acquisition.
* Arguments    : channel -
*                   Number of the MTU channel to be initialized
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
* Return Value : MTU_SUCCESS -
*                   Successful; channel initialized
*                MTU_ERR_BAD_CHAN -
*                   Channel number is invalid for part
*                MTU_ERR_CH_NOT_OPEN -
*                    The channel has not been opened so closing has no effect.
*                MTU_ERR_UNKNOWN_CMD -
*                    Unknown command
*                MTU_ERR_NULL_PTR -
*                   pconfig pointer is NULL
*******************************************************************************/
mtu_err_t MTU_PWM_Compliment_Control(mtu_cmpl_pwm_channel_t const channel,
                                     mtu_cmd_t const              cmd,
                                     void * const                 pcmd_data)
{
    mtu_cmpl_pwm_chnl_status_t *status = (mtu_cmpl_pwm_chnl_status_t *)pcmd_data;
    uint8_t count_st = 0;
    uint8_t count_updown = 0;

#if MTU3_CFG_PARAM_CHECKING_ENABLE == 1
    /* channel check */
    if (MTU_CMPL_PWM_CHANNEL_MAX <= channel)  // First check for channel number out of range
    {
        return MTU_ERR_BAD_CHAN;
    }

    /* source check */
    if ((MTU_CHANNEL_3_4 != channel)
     && (MTU_CHANNEL_6_7 != channel))
    {
        return MTU_ERR_INVALID_ARG;
    }

    /* Command check */
    switch (cmd)
    {
        case MTU_CMD_START:
        case MTU_CMD_STOP:
        case MTU_CMD_GET_STATUS:

        break;

        default:

            return MTU_ERR_UNKNOWN_CMD;
    }

    /* Get status NULL pointer check */
    if (MTU_CMD_GET_STATUS == cmd)
    {
        if (NULL == pcmd_data)
        {
            return MTU_ERR_NULL_PTR;
        }
    }

    /* Complementary PWM Open check */
    if (MTU_OPEN < mtu_control.cmpl_pwm_ch_info[channel].state)
    {
        return MTU_ERR_CH_NOT_OPENED;
    }
#endif // MTU3_CFG_PARAM_CHECKING_ENABLE

    switch (cmd)
    {
        case MTU_CMD_START:   // Start PWM

            if (MTU_CHANNEL_3_4 == channel)
            {
                MTU.TSTRA.BYTE |= MTU_CMPLMT_PWM_START;
            }
            else
            {
                MTU.TSTRB.BYTE |= MTU_CMPLMT_PWM_START;
            }

        break;

        case MTU_CMD_STOP:   // STOP PWM

            if (MTU_CHANNEL_3_4 == channel)
            {
                MTU.TSTRA.BIT.CST3 = MTU_COUNT_STOP;
                MTU.TSTRA.BIT.CST4 = MTU_COUNT_STOP;
            }
            else
            {
                MTU.TSTRB.BIT.CST6 = MTU_COUNT_STOP;
                MTU.TSTRB.BIT.CST7 = MTU_COUNT_STOP;
            }

        break;

        case MTU_CMD_GET_STATUS: // Get status

            if (MTU_CHANNEL_3_4 == channel)
            {
                count_st = MTU.TSTRA.BYTE;
                count_st &= MTU_CMPLMT_PWM_START;
                if (MTU_CMPLMT_PWM_START == count_st)
                {
                    status->c_st = MTU_CMPL_PWM_ST_COUNT_ON;
                }
                else
                {
                    status->c_st = MTU_CMPL_PWM_ST_COUNT_OFF;
                }
                count_updown = MTU3.TSR.BYTE;
                count_updown &= MTU_CNT_UPDOWN;
                if (MTU_CNT_UPDOWN == count_updown)
                {
                    status->d_st = MTU_CMPL_PWM_DIRECT_UP;
                }
                else
                {
                    status->d_st = MTU_CMPL_PWM_DIRECT_DOWN;
                }
            }
            else
            {
                count_st = MTU.TSTRB.BYTE;
                count_st &= MTU_CMPLMT_PWM_START;
                if (MTU_CMPLMT_PWM_START == count_st)
                {
                    status->c_st = MTU_CMPL_PWM_ST_COUNT_ON;
                }
                else
                {
                    status->c_st = MTU_CMPL_PWM_ST_COUNT_OFF;
                }
                count_updown =  MTU6.TSR.BYTE;
                count_updown &= MTU_CNT_UPDOWN;
                if (MTU_CNT_UPDOWN == count_updown)
                {
                    status->d_st = MTU_CMPL_PWM_DIRECT_UP;
                }
                else
                {
                    status->d_st = MTU_CMPL_PWM_DIRECT_DOWN;
                }
            }

        break;

        default:

            /* none */

        break;
    }

    return MTU_SUCCESS;
}
/*******************************************************************************
 End of function MTU_PWM_Compliment_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_protect
* Description  : Of register protect Enable / Disable
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_protect(mtu_cmpl_pwm_channel_t channel,
                                 mtu_cmpl_pwm_protect_t protect)
{
    volatile uint8_t dummy;

    if (MTU_CHANNEL_3_4 == channel)
    {
        if (MTU_PROTECT_ENABLE == protect)
        {
            /* ON the register protect */
            dummy = MTU.TRWERA.BYTE;
            MTU.TRWERA.BIT.RWE = 0;
        }
        else
        {
            /* OFF the register protect */
            MTU.TRWERA.BIT.RWE = 1;
        }
    }
    else
    {
        if (MTU_PROTECT_ENABLE == protect)
        {
            /* ON the register protect */
            dummy = MTU.TRWERA.BYTE;
            MTU.TRWERB.BIT.RWE = 0;
        }
        else
        {
            /* OFF the register protect */
            MTU.TRWERB.BIT.RWE = 1;
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_protect
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_count_stop
* Description  : TCNT count stop
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_count_stop(mtu_cmpl_pwm_channel_t channel)
{
    /* Stop MTU3 and MTU4 count operation */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TSTRA.BIT.CST3 = MTU_COUNT_STOP;
        MTU.TSTRA.BIT.CST4 = MTU_COUNT_STOP;
    }
    else  /* Stop MTU6 and MTU7 count operation */
    {
        MTU.TSTRB.BIT.CST6 = MTU_COUNT_STOP;
        MTU.TSTRB.BIT.CST7 = MTU_COUNT_STOP;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_count_stop
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_clk_edge
* Description  : Set the PWM clock and edge
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_clk_edge(mtu_cmpl_pwm_channel_t     channel,
                                  mtu_clk_src_t              clock_src,
                                  mtu_cmpl_pwm_clk_div_t     clk_div)
{
    uint8_t tcr = 0;
    uint8_t tcr2 = 0;

    /* Sets count clock and edge settings */
    if (MTU_CLK_SRC_INTERNAL == clock_src.source)
    {
        switch (clk_div.clock_div)
        {
            case MTU_SRC_CLK_DIV_1:

                tcr = MTU_CLK_DIV_1;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_4:

                tcr = MTU_CLK_DIV_4;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_16:

                tcr = MTU_CLK_DIV_16;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_64:

                tcr = MTU_CLK_DIV_64;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_256:

                tcr = MTU_CLK_DIV_256;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_1024:

                tcr = MTU_CLK_DIV_1024;
                tcr2 = MTU_CLK_TCR_0;

            break;

            case MTU_SRC_CLK_DIV_2:

                tcr = MTU_CLK_TCR_0;
                tcr2 = MTU_CLK_DIV_2;

            break;

            case MTU_SRC_CLK_DIV_8:

                tcr = MTU_CLK_TCR_0;
                tcr2 = MTU_CLK_DIV_8;

            break;

            case MTU_SRC_CLK_DIV_32:

                tcr = MTU_CLK_TCR_0;
                tcr2 = MTU_CLK_DIV_32;

            break;

            default:

                return ;
        }
        tcr |= clock_src.clock_edge;
        if (MTU_CHANNEL_3_4 == channel)
        {
            MTU3.TCR.BYTE  = tcr;
            MTU3.TCR2.BYTE = tcr2;
            MTU4.TCR.BYTE  = tcr;
            MTU4.TCR2.BYTE = tcr2;
        }
        else
        {
            MTU6.TCR.BYTE  = tcr;
            MTU6.TCR2.BYTE = tcr2;
            MTU7.TCR.BYTE  = tcr;
            MTU7.TCR2.BYTE = tcr2;
        }
    }
    else if (MTU_CLK_SRC_EXT_MTCLKA == clock_src.source)
    {
        /* Sets External count clock and edge settings */
        if (MTU_CHANNEL_3_4 == channel)
        {
            tcr = MTU_CLK_MTCLKA;
            tcr2 = MTU_CLK_TCR_0;
            tcr |= clock_src.clock_edge;
            MTU3.TCR.BYTE = tcr;
            MTU3.TCR2.BYTE = tcr2;
            MTU4.TCR.BYTE = tcr;
            MTU4.TCR2.BYTE = tcr2;
        }
    }
    else if (MTU_CLK_SRC_EXT_MTCLKB == clock_src.source)
    {
        /* Sets External count clock and edge settings */
        if (MTU_CHANNEL_3_4 == channel)
        {
             tcr = MTU_CLK_MTCLKB;
             tcr2 = MTU_CLK_TCR_0;
             tcr |= clock_src.clock_edge;
             MTU3.TCR.BYTE = tcr;
             MTU3.TCR2.BYTE = tcr2;
             MTU4.TCR.BYTE = tcr;
             MTU4.TCR2.BYTE = tcr2;
        }
    }
    else
    {
        /* Do Nothing */
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_clk_edge
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_count_set
* Description  : TCNT settings
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_count_set(mtu_cmpl_pwm_channel_t channel)
{
    /* TCNT set to 0 */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU4.TCNT = MTU_TCNT_0;
    }
    else
    {
        MTU7.TCNT = MTU_TCNT_0;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_count_set
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_duty_buff
* Description  : From Duty ratio, and set the width of the PWM
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                output_no -
*                   Output Channel Number
*                cycle_freq -
*                   cycle freq of the MTU Complimentary PWM
*                duty -
*                   Duty ratio
*                dead_time -
*                   dead time of the MTU Complimentary PWM
*                d_bf -
*                   Use / unused double buffer
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_duty_buff(mtu_cmpl_pwm_channel_t  channel,
                                   uint8_t                 output_no,
                                   uint16_t                cycle_freq,
                                   uint16_t                duty,
                                   uint16_t                dead_time,
                                   mtu_cmpl_pwm_d_bf_t     d_bf)
{
    uint16_t cycle = 0;
    uint16_t buff = 0;

    cycle = (cycle_freq * 1/2);    /* cycle / 2 */
    /* Calculated from the Duty ratio */
    if (0 == dead_time)
    {
        buff = (uint16_t)(((cycle + 1) * duty)/1000);
    }
    else
    {
        buff = (uint16_t)(((cycle + dead_time) * duty)/1000);
    }

    if (MTU_CHANNEL_3_4 == channel)
    {
        if (MTU_TCNT_1 == output_no)
        {
            MTU3.TGRB = buff;
            MTU3.TGRD = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU3.TGRE = (uint16_t)(buff - 1);
            }
        }
        else if (MTU_TCNT_2 == output_no)
        {
            MTU4.TGRA = buff;
            MTU4.TGRC = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU4.TGRE = (uint16_t)(buff - 1);
            }
        }
        else
        {
            MTU4.TGRB = buff;
            MTU4.TGRD = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU4.TGRF = (uint16_t)(buff - 1);
            }
            else
            {
                /* Do Nothing */
            }
        }
    }
    else
    {
        if (MTU_TCNT_1 == output_no)
        {
            MTU6.TGRB = buff;
            MTU6.TGRD = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU6.TGRE = (uint16_t)(buff - 1);
            }
        }
        else if (MTU_TCNT_2 == output_no)
        {
            MTU7.TGRA = buff;
            MTU7.TGRC = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU7.TGRE = (uint16_t)(buff - 1);
            }
        }
        else
        {
            MTU7.TGRB = buff;
            MTU7.TGRD = buff;
            if (MTU_CMPL_PWM_D_BF_ON == d_bf)
            {
                MTU7.TGRF = (uint16_t)(buff - 1);
            }
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_duty_buff
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_deadtime
* Description  : Of dead time Enable / Disable setting
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                dead_time -
*                   dead time of the MTU Complimentary PWM
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_deadtime(mtu_cmpl_pwm_channel_t  channel,
                                  uint16_t                dead_time)
{
    if (MTU_CHANNEL_3_4 == channel)
    {
        if (0 != dead_time)
        {
            MTU.TDERA.BIT.TDER = MTU_DEADTIME_ON;
            MTU.TDDRA = dead_time;
        }
        else
        {
            MTU.TDERA.BIT.TDER = MTU_DEADTIME_OFF;
            MTU.TDDRA = MTU_DEADTIME_DEF;
        }
    }
    else
    {
        if (0 != dead_time)
        {
            MTU.TDERB.BIT.TDER = MTU_DEADTIME_ON;
            MTU.TDDRB = dead_time;
        }
        else
        {
            MTU.TDERB.BIT.TDER = MTU_DEADTIME_OFF;
            MTU.TDDRB = MTU_DEADTIME_DEF;
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_deadtime
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_carrier_cycle
* Description  : Set compare / match
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                cycle_freq -
*                   cycle freq of the MTU Complimentary PWM
*                dead_time -
*                   dead time of the MTU Complimentary PWM
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_carrier_cycle(mtu_cmpl_pwm_channel_t  channel,
                                       uint16_t                cycle_freq,
                                       uint16_t                dead_time)
{
    uint16_t cycle = 0;
    uint16_t buff = 0;

    cycle = (cycle_freq * 1/2);
    buff = (uint16_t)(cycle + dead_time);

    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TCDRA = cycle;
        MTU.TCBRA = cycle;
        if (0 != dead_time)
        {
            MTU3.TGRA = buff;
            MTU3.TGRC = buff;
        }
        else
        {
            MTU3.TGRA = (uint16_t)(cycle + MTU_DEADTIME_DEF);
            MTU3.TGRC = (uint16_t)(cycle + MTU_DEADTIME_DEF);
        }
    }
    else
    {
        MTU.TCDRB = cycle;
        MTU.TCBRB = cycle;
        if (0 != dead_time)
        {
            MTU6.TGRA = buff;
            MTU6.TGRC = buff;
        }
        else
        {
            MTU6.TGRA = (uint16_t)(cycle + MTU_DEADTIME_DEF);
            MTU6.TGRC = (uint16_t)(cycle + MTU_DEADTIME_DEF);
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_carrier_cycle
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_output_toggle
* Description  : Toggle output enable / disable from MTIOC3A / MTIOC6A
*                synchronized with the PWM period
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                p_n -
*                   output level of the MTU Complimentary PWM
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_toggle(mtu_cmpl_pwm_channel_t  channel,
                                       mtu_cmpl_pwm_toggle_t   toggle)
{
    /*  MTIOC3A / MTIOC6A toggle output enable / disable */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TOCR1A.BIT.PSYE = toggle;
    }
    else
    {
        MTU.TOCR1B.BIT.PSYE = toggle;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_output_toggle
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_output_level_kind
* Description  : Select a setting type of PWM output level
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                p_n -
*                   output level of the MTU Complimentary PWM
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_level_kind(mtu_cmpl_pwm_channel_t  channel,
                                           mtu_cmpl_pwm_p_n_t      p_n)
{
    /* Common / individual output level type */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TOCR1A.BIT.TOCS = p_n;
    }
    else
    {
        MTU.TOCR1B.BIT.TOCS = p_n;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_output_level_kind
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_output_level_set_1
* Description  : Setting the PWM output level Common
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                pwm_output -
*                   output setting
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_level_set_1(mtu_cmpl_pwm_channel_t  channel,
                                            mtu_cmpl_pwm_settings_t pwm_output)
{
    /* Output level setting of the P / N */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TOCR1A.BIT.OLSP = pwm_output.olsp;
        MTU.TOCR1A.BIT.OLSN = pwm_output.olsn;
    }
    else
    {
        MTU.TOCR1B.BIT.OLSP = pwm_output.olsp;
        MTU.TOCR1B.BIT.OLSN = pwm_output.olsn;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_output_level_set_1
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_output_level_set_2
* Description  : Setting the PWM output level individual
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                output_no -
*                   Output Channel Number
*                pwm_output -
*                   output setting
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_level_set_2(mtu_cmpl_pwm_channel_t  channel,
                                            uint8_t                 output_no,
                                            mtu_cmpl_pwm_settings_t pwm_output)
{
    /* Output level setting of the P / N */
    if (MTU_CHANNEL_3_4 == channel)
    {
        if (MTU_TCNT_1 == output_no)
        {
            MTU.TOCR2A.BIT.OLS1P = pwm_output.olsp;
            MTU.TOCR2A.BIT.OLS1N = pwm_output.olsn;
            MTU.TOLBRA.BIT.OLS1P = pwm_output.olsp;
            MTU.TOLBRA.BIT.OLS1N = pwm_output.olsn;
        }
        else if (MTU_TCNT_2 == output_no)
        {
            MTU.TOCR2A.BIT.OLS2P = pwm_output.olsp;
            MTU.TOCR2A.BIT.OLS2N = pwm_output.olsn;
            MTU.TOLBRA.BIT.OLS2P = pwm_output.olsp;
            MTU.TOLBRA.BIT.OLS2N = pwm_output.olsn;
        }
        else
        {
            MTU.TOCR2A.BIT.OLS3P = pwm_output.olsp;
            MTU.TOCR2A.BIT.OLS3N = pwm_output.olsn;
            MTU.TOLBRA.BIT.OLS3P = pwm_output.olsp;
            MTU.TOLBRA.BIT.OLS3N = pwm_output.olsn;
        }
    }
    else
    {
        if (MTU_TCNT_1 == output_no)
        {
            MTU.TOCR2B.BIT.OLS1P = pwm_output.olsp;
            MTU.TOCR2B.BIT.OLS1N = pwm_output.olsn;
            MTU.TOLBRB.BIT.OLS1P = pwm_output.olsp;
            MTU.TOLBRB.BIT.OLS1N = pwm_output.olsn;
        }
        else if (MTU_TCNT_2 == output_no)
        {
            MTU.TOCR2B.BIT.OLS2P = pwm_output.olsp;
            MTU.TOCR2B.BIT.OLS2N = pwm_output.olsn;
            MTU.TOLBRB.BIT.OLS2P = pwm_output.olsp;
            MTU.TOLBRB.BIT.OLS2N = pwm_output.olsn;
        }
        else
        {
            MTU.TOCR2B.BIT.OLS3P = pwm_output.olsp;
            MTU.TOCR2B.BIT.OLS3N = pwm_output.olsn;
            MTU.TOLBRB.BIT.OLS3P = pwm_output.olsp;
            MTU.TOLBRB.BIT.OLS3N = pwm_output.olsn;
        }
    }


}
/*******************************************************************************
 End of function mtu_cmpl_pwm_output_level_set_2
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_output_level_buff
* Description  : Transfer settings from the PWM output level buffer
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                p_n_bf -
*                   timing for transferring data
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_level_buff(mtu_cmpl_pwm_channel_t  channel,
                                           mtu_cmpl_pwm_p_n_bf_t   p_n_bf)
{
    /* P / N the setting of either buffer transfer */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU.TOCR2A.BIT.BF = p_n_bf;
    }
    else
    {
        MTU.TOCR2B.BIT.BF = p_n_bf;
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_output_level_buff
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_mode
* Description  : Mode setting of the complementary PWM
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                p_n_bf -
*                   timing for transferring data
*                d_bf -
*                   Use / unused double buffer
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_mode(mtu_cmpl_pwm_channel_t  channel,
                              mtu_cmpl_pwm_mode_t     mode,
                              mtu_cmpl_pwm_d_bf_t     d_bf)
{
    uint8_t setmode = 0;

    /* Complementary PWM 1-3 setting */
    setmode = (uint8_t)(mode | MTU_TMDR_BFA | MTU_TMDR_BFB);
    /* Set in the case of the double buffer is enabled */
    if (MTU_CHANNEL_3_4 == channel)
    {
        MTU3.TMDR1.BYTE = setmode;
    }
    else
    {
        MTU6.TMDR1.BYTE = setmode;
    }
    if (MTU_CMPL_PWM_D_BF_OFF != d_bf)
    {
        if (MTU_CHANNEL_3_4 == channel)
        {
            MTU.TMDR2A.BYTE = MTU_TMDR2_DRS;
        }
        else
        {
            MTU.TMDR2B.BYTE = MTU_TMDR2_DRS;
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_mode
*******************************************************************************/

/*******************************************************************************
* Function Name: mtu_cmpl_pwm_enable_output
* Description  : Enable waveform output
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
* Return Value : none
*******************************************************************************/
static void mtu_cmpl_pwm_output_enable(mtu_cmpl_pwm_channel_t   channel,
                                       mtu_cmpl_pwm_output_st_t output,
                                       uint8_t                  output_no)
{
    /* Enable waveform output */
    if (MTU_CHANNEL_3_4 == channel)
    {
        if (MTU_TCNT_1 == output_no)
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERA.BIT.OE3B = MTU_ENABLE_OUTPUT;
                MTU.TOERA.BIT.OE3D = MTU_ENABLE_OUTPUT;
            }
        }
        else if (MTU_TCNT_2 == output_no)
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERA.BIT.OE4A = MTU_ENABLE_OUTPUT;
                MTU.TOERA.BIT.OE4C = MTU_ENABLE_OUTPUT;
            }
        }
        else
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERA.BIT.OE4B = MTU_ENABLE_OUTPUT;
                MTU.TOERA.BIT.OE4D = MTU_ENABLE_OUTPUT;
            }
        }
    }
    else
    {
        if (MTU_TCNT_1 == output_no)
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERB.BIT.OE6B = MTU_ENABLE_OUTPUT;
                MTU.TOERB.BIT.OE6D = MTU_ENABLE_OUTPUT;
            }
        }
        else if (MTU_TCNT_2 == output_no)
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERB.BIT.OE7A = MTU_ENABLE_OUTPUT;
                MTU.TOERB.BIT.OE7C = MTU_ENABLE_OUTPUT;
            }
        }
        else
        {
            if (MTU_CMPL_PWM_OUTPUT_ON == output)
            {
                MTU.TOERB.BIT.OE7B = MTU_ENABLE_OUTPUT;
                MTU.TOERB.BIT.OE7D = MTU_ENABLE_OUTPUT;
            }
        }
    }
}
/*******************************************************************************
 End of function mtu_cmpl_pwm_enable_output
*******************************************************************************/

/* End of File */

