/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : mtu3a driver interface functions for mtu3a fit module.
* File Name    : r_mtu3_if.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for MTU3a driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using MTU3a on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_system.h"
#include "Renesas_RZ_T1.h"

#include "r_mtu3.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define MTU_CHECK_CMPL_PWM_CHANNEL(ch)      (((MTU_CHANNEL_3_4 <= (int32_t)ch) && (MTU_CMPL_PWM_CHANNEL_MAX > (int32_t)ch))?true:false)
#define MTU_CHECK_CHANNEL(ch)               (((MTU_CHANNEL_0 <= (int32_t)ch) && (MTU_CHANNEL_MAX > (int32_t)ch))?true:false)

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static osSemaphoreDef(sem_mtu_state);

/*******************************************************************************
* Function Name: R_MTU_Init
* Description  : This function initialize MTU driver.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_MTU_Init(void)
{
    if (mtu_control.state == MTU_UNINIT)
    {
        mtu_control.sem_state = osSemaphoreCreate(osSemaphore(sem_mtu_state), 1);
        mtu_control.state     = MTU_INIT;
    }
}
/*******************************************************************************
End of function R_MTU_Init
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Uninit
* Description  : This function uninitialize MTU driver.
* Arguments    : None
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_Uninit(void)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    osStatus    sem_state_ercd;

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (mtu_control.state != MTU_UNINIT)
        {
            /* close MTU channel */

            /* release semaphore */
            sem_state_ercd    = osSemaphoreDelete(mtu_control.sem_state);
            if (sem_state_ercd != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Delete Error */
                ercd = EFAULT;
            }
            else
            {
                mtu_control.state = MTU_UNINIT;
            }
        }
        else
        {
            ercd = EACCES;
        }
        
        if(ercd != ESUCCESS)
        {
            sem_ret = osSemaphoreRelease(mtu_control.sem_state);
            if (sem_ret != osOK)
            {
                /* NON NOTICE ASSERT: Semaphore Release Error */
                ercd = EFAULT;
            }
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_Uninit
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_PWM_Compliment_Open
* Description  : This function is to supply power to the channel
*                of the MTU for complementary PWM, to initialize the
*                registers associated with preparing for comparison /
*                match timer operation, to apply the user-configurable options.
* Arguments    : channel -
*                   Number of the MTU Complimentary PWM channel
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_PWM_Compliment_Open(mtu_cmpl_pwm_channel_t const         channel,
                                  mtu_cmpl_pwm_chnl_settings_t * const pconfig)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if ((MTU_CHECK_CMPL_PWM_CHANNEL((uint32_t)channel) == false)
        || (pconfig == NULL))
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[0]->mode == MTU_MODE_NONE)
            && (mtu_control.cmpl_pwm_ch_info[channel].p_ch_info[1]->mode == MTU_MODE_NONE))
        {
            err = MTU_PWM_Compliment_Open(channel, pconfig);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.cmpl_pwm_ch_info[channel].state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_PWM_Compliment_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_PWM_Compliment_Close
* Description  : Removes power to the MTU channel designated by the handle and
*                disables the associated interrupts.
* Arguments    : channel-
*                   The channel number
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t  R_MTU_PWM_Compliment_Close(mtu_cmpl_pwm_channel_t const channel)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if (MTU_CHECK_CMPL_PWM_CHANNEL((uint32_t)channel) == false)
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (mtu_control.cmpl_pwm_ch_info[channel].state >= MTU_OPEN)
        {
            err = MTU_PWM_Compliment_Close(channel);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_PWM_Compliment_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Conmpliment_PWM_Control
* Description  : This function, the start of the channel of the MTU for
*                complementary PWM, stops, and information acquisition.
* Arguments    : channel -
*                   Number of the MTU channel to be initialized
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_PWM_Compliment_Control(mtu_cmpl_pwm_channel_t const channel,
                                     mtu_cmd_t const              cmd,
                                     void * const                 pcmd_data)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if (MTU_CHECK_CMPL_PWM_CHANNEL((uint32_t)channel) == false)
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if (mtu_control.cmpl_pwm_ch_info[channel].state >= MTU_OPEN)
        {
            err = MTU_PWM_Compliment_Control(channel, cmd, pcmd_data);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_PWM_Compliment_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Timer_Open
* Description  : This function applies power to the MTU channel,
*                initializes the associated registers to prepare for compare/match timer operations,
*                and applies user-configurable options.
* Arguments    : channel -
*                   Number of the MTU channel to be initialized
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
*                pcallback -
*                   Pointer to function called from interrupt
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_Timer_Open(mtu_channel_t const               channel,
                         mtu_timer_chnl_settings_t * const pconfig,
                         void                     (* const pcallback)(void * pdata))
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if ((MTU_CHECK_CHANNEL((uint32_t)channel) == false)
        || (pconfig == NULL))
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.p_ch_info[channel]->mode == MTU_MODE_NONE)
            && (mtu_control.p_ch_info[channel]->state == MTU_CLOSE))
        {
            err = MTU_Timer_Open(channel, pconfig, pcallback);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.p_ch_info[channel]->state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_Timer_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Capture_Open
* Description  : This function applies power to the MTU channel,
*                initializes the associated registers,
*                applies user-configurable options,
*                and provides the channel handle for use with other API functions.
* Arguments    : channel -
*                   Number of the MTU channel to be initialized
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
*                pcallback -
*                   Pointer to function called from interrupt
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_Capture_Open(mtu_channel_t const                 channel,
                           mtu_capture_chnl_settings_t * const pconfig,
                           void                       (* const pcallback)(void * pdata))
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if ((MTU_CHECK_CHANNEL((uint32_t)channel) == false)
        || (pconfig == NULL))
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.p_ch_info[channel]->mode == MTU_MODE_NONE)
            && (mtu_control.p_ch_info[channel]->state == MTU_CLOSE))
        {
            err = MTU_Capture_Open(channel, pconfig, pcallback);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.p_ch_info[channel]->state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_Capture_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_PWM_Open
* Description  : This function applies power to the MTU channel,
*                initializes the associated registers to prepare for compare/match timer operations,
*                and applies user-configurable options.
* Arguments    : channel -
*                   Number of the MTU channel to be initialized
*                pconfig -
*                   Pointer to MTU channel configuration data structure.
*                pcallback -
*                   Pointer to function called from interrupt
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_PWM_Open(mtu_channel_t const             channel,
                       mtu_pwm_chnl_settings_t * const pconfig,
                       void                   (* const pcallback)(void * pdata))
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if ((MTU_CHECK_CHANNEL((uint32_t)channel) == false)
        || (pconfig == NULL))
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.p_ch_info[channel]->mode == MTU_MODE_NONE)
            && (mtu_control.p_ch_info[channel]->state == MTU_CLOSE))
        {
            err = MTU_PWM_Open(channel, pconfig, pcallback);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.p_ch_info[channel]->state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_PWM_Open
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Control
* Description  : This function is responsible for handling special hardware or software operations for the MTU channel.
* Arguments    : channel-
*                   The channel number
*                cmd
*                   Enumerated command code
*                pcmd_data
*                   Pointer to the command-data structure parameter of type void that is used to reference the location
*                   of any data specific to the command that is needed for its completion.
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_Control(mtu_channel_t const channel,
                      mtu_cmd_t const     cmd,
                      void * const        pcmd_data)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if (MTU_CHECK_CHANNEL((uint32_t)channel) == false)
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.p_ch_info[channel]->mode != MTU_MODE_CMPL_PWM)
            && (mtu_control.p_ch_info[channel]->state >= MTU_OPEN))
        {
            err = MTU_Control(channel, cmd, pcmd_data);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.p_ch_info[channel]->state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_Close
* Description  : Removes power to the MTU channel designated by the handle and disables the associated interrupts.
* Arguments    : : channel-
*                   The channel number
* Return Value : int32_t ercd   : error number
*                               : ESUCCESS
*                               : EINVAL
*                               : EACCES
*                               : EBUSY
*******************************************************************************/
int32_t R_MTU_Close(mtu_channel_t const channel)
{
    int32_t     ercd = ESUCCESS;
    int32_t     sem_ret;
    mtu_err_t   err;

    if (MTU_CHECK_CHANNEL((uint32_t)channel) == false)
    {
        return EINVAL;
    }

    sem_ret = osSemaphoreWait(mtu_control.sem_state, osWaitForever);

    if (sem_ret != -1)
    {
        if ((mtu_control.p_ch_info[channel]->mode != MTU_MODE_CMPL_PWM)
            && (mtu_control.p_ch_info[channel]->state >= MTU_OPEN))
        {
            err = MTU_Close(channel);
            if (err != MTU_SUCCESS)
            {
                switch (err)
                {
                    case MTU_ERR_BAD_CHAN:
                    case MTU_ERR_UNKNOWN_CMD:
                    case MTU_ERR_INVALID_ARG:
                    case MTU_ERR_ARG_RANGE:
                    case MTU_ERR_NULL_PTR:
                    case MTU_ERR_UNDEF:

                        ercd = EINVAL;

                    break;

                    case MTU_ERR_CH_NOT_OPENED:
                    case MTU_ERR_CH_NOT_CLOSED:

                        ercd = EACCES;

                    break;

                    case MTU_ERR_LOCK:

                        ercd = EBUSY;

                    break;

                    default:

                        ercd = EFAULT;

                    break;
                }
            }
        }
        else if (mtu_control.p_ch_info[channel]->state == MTU_RUN)
        {
            ercd = EBUSY;
        }
        else
        {
            ercd = EACCES;
        }

        sem_ret = osSemaphoreRelease(mtu_control.sem_state);
        if (sem_ret != osOK)
        {
            /* NON NOTICE ASSERT: Semaphore Release Error */
            ercd = EFAULT;
        }
    }
    else
    {
        ercd = EFAULT;
    }

    return ercd;
}
/*******************************************************************************
End of function R_MTU_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: R_MTU_GetVersion
* Description  : Returns the version of this module. The version number is
*                encoded such that the top two bytes are the major version
*                number and the bottom two bytes are the minor version number.
* Arguments    : none
* Return Value : version number
*******************************************************************************/
uint32_t R_MTU_GetVersion(void)
{
    uint32_t const version = (MTU_VERSION_MAJOR << 16) | MTU_VERSION_MINOR;

    return version;
}
/*******************************************************************************
End of function R_MTU_GetVersion
*******************************************************************************/

/* End of File */

