/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : vic.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : VIC functions
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Implementation of VIC functions declared in CMSIS Cortex-R4F Core Peripheral Access Layer
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#include "Renesas_RZ_T1.h"
#include "r_system.h"
#include <iodefine.h>

#define ICU_PRIORITY_15         (15)

/* Globals for use of post-scatterloading code that must access GIC */
const uint32_t      VIC_BASE = (uint32_t)Renesas_RZ_T1_VIC_BASE;
extern uint32_t     VICVectorAddr[];
uint32_t            IRQNumber2;

/*******************************************************************************
* Function Name: VIC_EnableIRQ
* Description  : Enable IRQ interrupt
* Arguments    : IRQn
*                    Vector interrupt number (1 to 300).
* Return Value : none
*******************************************************************************/
void VIC_EnableIRQ(IRQn_Type IRQn)
{
    /* Define IENn register address pointer */
    volatile uint32_t *p_ien_base;

    /* Variable to specify register suffix */
    uint32_t reg_num;   // IENn (n = reg_num)
    uint32_t bit_num;   // IENn.IENm (m = bit_num)
    uint32_t vec_num;   // Vector Number

    vec_num = IRQn;

    /* Check IRQn */
    if ((vec_num <= 0) || (vec_num > 300)) {
        return;
    }

    /* Calcurate register address and register suffix number */
    if ( 255 >= vec_num )  // Vector number : 1 to 255
    {
        /* Set each pointer base address as IEN0 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_ien_base = (uint32_t*)&(VIC.IEN0.LONG);

        /* Calcurate register suffix number */
        reg_num = vec_num / 32;  // IENn (n = reg_num)
        bit_num = vec_num % 32;  // IENn.IENm (m = bit_num)
    }
    else  /* Vector number : 256 to 300 */
    {
        /* Set each pointer address as IEN8 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_ien_base = (uint32_t*)&(VIC.IEN8.LONG);

        /* Calcurate register suffix number. And subtract 8 from reg_num
        because IEN8 is base address in this case */
        reg_num = (vec_num / 32) - 8;  // IENn (n = 8 + reg_num)
        bit_num = (vec_num % 32);      // IENn.IENm (m = bit_num)
    }

    /* Set interrupt enable register (enable interrupt) */
    p_ien_base += reg_num;              // Specify IENn register address
    *p_ien_base |= ( 1 << bit_num );    // Set IENn.IENm bit
}

/*******************************************************************************
* Function Name: VIC_DisableIRQ
* Description  : Disable IRQ interrupt
* Arguments    : IRQn
*                    Vector interrupt number (1 to 300).
* Return Value : none
*******************************************************************************/
void VIC_DisableIRQ(IRQn_Type IRQn)
{
    /* Define IECn register address pointer */
    volatile uint32_t *p_iec_base;

    /* Variable to specify register suffix */
    uint32_t reg_num;   // IECn (n = reg_num)
    uint32_t bit_num;   // IECn.IECm (m = bit_num)
    uint32_t vec_num;   // Vector Number

    vec_num = IRQn;

    /* Check IRQn */
    if ((vec_num <= 0) || (vec_num > 300)) {
        return;
    }

    /* Calcurate register address and register suffix number */
    if ( 255 >= vec_num )  // Vector number : 1 to 255
    {
        /* Set each pointer base address as IEC0 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_iec_base = (uint32_t*)&(VIC.IEC0.LONG);

        /* Calcurate register suffix number */
        reg_num = vec_num / 32;    // IECn (n = reg_num)
        bit_num = vec_num % 32;    // IECn.IECm (m = bit_num)
    }
    else  // Vector number : 256 to 300
    {
        /* Set each pointer address as IEC8 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_iec_base = (uint32_t*)&(VIC.IEC8.LONG);

        /* Calcurate register suffix number. And subtract 8 from reg_num
        because IEC8 is base address in this case */
        reg_num = (vec_num / 32) - 8;   // IECn (n = 8 + reg_num)
        bit_num = (vec_num % 32);       // IECn.IECm (m = bit_num)
    }

    /* Set interrupt enable clear register (disable interrupt) */
    p_iec_base += reg_num;              // Specify IECn register address
    *p_iec_base |= ( 1 << bit_num );    // Set IECn.IECm bit
}

/*******************************************************************************
* Function Name: VIC_EnableIRQ
* Description  : Set Interrrupt Detect Type
* Arguments    : Vector interrupt number (1 to 300).
*              : Type 0:Level,1:Edge
* Return Value : none
*******************************************************************************/
void VIC_SetDetectType(IRQn_Type IRQn, uint32_t detecttype)
{
    /* Define PLSn register address pointer */
    volatile uint32_t *p_pls_base;

    /* Variable to specify register suffix */
    uint32_t reg_num;   // IENn (n = reg_num)
    uint32_t bit_num;   // IENn.IENm (m = bit_num)
    uint32_t vec_num;   // Vector Number
    uint32_t set_data;  //

    vec_num = IRQn;

    /* Check IRQn */
    if ((vec_num <= 0) || (vec_num > 300)) {
        return;
    }

    /* Set each pointer base address as PLS0 or PLS8 */
    /* Calcurate register address and register suffix number */
    /* Calcurate register suffix number */
    reg_num = vec_num / 32;  // PLSn (n = reg_num)
    if (reg_num >= 8)
    {
        p_pls_base = (uint32_t*)&(VIC.PLS8.LONG);
        p_pls_base += (reg_num - 8);
    }
    else
    {
        p_pls_base = (uint32_t*)&(VIC.PLS0.LONG);
        p_pls_base += reg_num;
    }
    bit_num = vec_num % 32;  // PLSn.PLSm (m = bit_num)

    /* Set interrupt enable register (enable interrupt) */
    set_data = *p_pls_base;             // Get PLSn Regeister
    if (detecttype)
        set_data |= ( 1 << bit_num );   // Set PLSn.PLSm bit
    else
        set_data &= (~( 1 << bit_num ));
    
    *p_pls_base = set_data;             // Set PLSn.PLSm bit
}

/*******************************************************************************
* Function Name: VIC_SetPriority
* Description  : Setting interrupt priority level.
* Arguments    : IRQn
*                    Vector interrupt number (1 to 300).
*                priority
*                    Interrupt priority level (0 to 31
*                                               Vector number 1   to 255:  0 to 15
*                                               Vector number 256 to 300: 16 to 31)
* Return Value : none
*******************************************************************************/
void VIC_SetPriority(IRQn_Type IRQn, uint32_t priority)
{
    /* Define PRLn registers address pointer */
    volatile uint32_t *p_prl_base;

    /* Variable to specify register suffix */
    uint32_t vec_num;   // Vector Number

    vec_num = IRQn;

    /* Check IRQn */
    if ((vec_num <= 0) || (vec_num > 300)) {
        return;
    }

    /* Calcurate register address and register suffix number */
    if ( 255 >= vec_num )  // Vector number : 1 to 255
    {
        if (priority > 15) {
            return;
        }

        p_prl_base = (uint32_t*)&(VIC.PRL1.LONG);
    }
    else  // Vector number : 256 to 294
    {
        if ((priority < 16) || (priority > 31)) {
            return;
        }
        priority -= 16;
        p_prl_base = (uint32_t*)&(VIC.PRL256.LONG);
        vec_num -= 255;         // Offset (PRLn and VADn base is changed (eg. VAD1 to VAD256)
    }

    /* Set interrupt priority level (0 to 15) or (16 to 31) */
    p_prl_base += ( vec_num - 1 );  // Specify PRLn register address
    *p_prl_base = priority;         // Set PRLn to priority value
}

/*******************************************************************************
* Function Name: VIC_SetPriority
* Description  : Setting interrupt priority level.
* Arguments    : IRQn
*                    Vector interrupt number (1 to 300).
* Return Value : uint32_t
*                   Interrupt priority level (0 to 31
*                       Vector number 1   to 255:  0 to 15
*                       Vector number 256 to 300: 16 to 31)
*******************************************************************************/
uint32_t VIC_GetPriority(IRQn_Type IRQn)
{
    /* Define PRLn registers address pointer */
    volatile uint32_t *p_prl_base;

    /* Variable to specify register suffix */
    uint32_t vec_num;       // Vector Number
    uint32_t priority = 0;  // Priority Level

    vec_num = IRQn;

    /* Check IRQn */
    if ((vec_num <= 0) || (vec_num > 300)) {
        return 0xFFFFFFFF;
    }

    /* Calcurate register address and register suffix number */
    if ( 255 >= vec_num )  // Vector number : 1 to 255
    {
        /* Set each pointer base address as PLS0, PRL1, VAD1 and PIC0 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_prl_base = (uint32_t*)&(VIC.PRL1.LONG);

    }
    else  // Vector number : 256 to 294
    {
        /* Set each pointer address as PLS8, PRL256, VAD256 and PIC8 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_prl_base = (uint32_t*)&(VIC.PRL256.LONG);

        vec_num -= 255;                 // Offset (PRLn and VADn base is changed (eg. VAD1 to VAD256)
    }

    /* Set interrupt priority level (0 to 15) or (16 to 31) */
    p_prl_base += ( vec_num - 1 );  // Specify PRLn register address
    priority = *p_prl_base;         // Set PRLn to priority value

    if (vec_num > 255) {
        priority += 16;
    }

    return priority;
}

/*******************************************************************************
* Function Name: VIC_Init
* Description  : Initialize VIC.
* Arguments    : none
* Return Value : none
********************************************************************************/
void VIC_Init(void)
{
    volatile uint32_t *     p_prl;
    volatile uint32_t *     p_vad;
    volatile uint32_t *     p_pic;

    uint32_t                i;

    /* Disable Interrupt */
    DISABLE_IRQ();

    /* Initialize interrupt priority level.(PRL1 - PRL255) */
    p_prl       = (uint32_t *)&(VIC.PRL1.LONG);
    for (i = 1; i <= 255; i++) {
        *(p_prl++) = ICU_PRIORITY_15;
    }
    /* Initialize interrupt priority level.(PRL256 - PRL300) */
    p_prl       = (uint32_t *)&(VIC.PRL256.LONG);
    for (; i <= 300 ; i++) {
        *(p_prl++) = ICU_PRIORITY_15;
    }

    /* Initialize interrupt vector address.(VAD1 - VAD255) */
    p_vad = (uint32_t *)&(VIC.VAD1.LONG);
    for (i = 0; i < 255; i++) {
        *(p_vad++) = VICVectorAddr[i];
    }
    /* Initialize interrupt vector address.(VAD256 - VAD300) */
    p_vad = (uint32_t *)&(VIC.VAD256.LONG);
    for ( ; i < 300; i++) {
        *(p_vad++) = VICVectorAddr[i];
    }

    /* Clear interrupt edig.(PIC1-PIC255) */
    p_pic       = (uint32_t *)&(VIC.PIC0.LONG);
    *(p_pic++) = 0xFFFFFFFE;        /* PLS0 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS1 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS2 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS3 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS4 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS5 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS6 */
    *(p_pic++) = 0xFFFFFFFF;        /* PLS7 */
    /* Clear interrupt edig.(PIC256-PIC300) */
    p_pic      = (uint32_t *)&(VIC.PIC8.LONG);
    *(p_pic++) = 0xFFFFFFFF;        /* PLS8 */
    *(p_pic++) = 0x00001FFF;        /* PLS9 */

    /* Dummy writing to HVA0 register */
    HVA0_DUMMY_WRITE();

    /* Enable Interrupt */
    __enable_irq();
}

/*******************************************************************************
* Function Name: VIC_Init
* Description  : Initialize and Enabled VIC.
* Arguments    : none
* Return Value : none
********************************************************************************/
void VIC_Enable(void)
{
    VIC_Init();
    __enable_vic_port();

}

/*******************************************************************************
* Function Name: VIC_ClearPIC
* Description  : Clear PIC
* Arguments    : IRQn
*                    Vector interrupt number (1 to 300).
* Return Value : none
********************************************************************************/
void VIC_ClearPIC(IRQn_Type IRQn)
{
    /* Define PLSn and PICn registers address pointer */
    volatile uint32_t *p_pls_base;
    volatile uint32_t *p_pic_base;

    /* Variable to specify register suffix */
    uint32_t reg_num;  // PLSn, PICn (n = reg_num)
    uint32_t bit_num;  // PLSn.PLSm, PICn.PICm (m = bit_num)
    uint32_t vec_num;

    vec_num = IRQn;

    if (vec_num < 256) {
        /* Set each pointer base address as PLS0 and PIC0 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_pls_base = (uint32_t*)&(VIC.PLS0.LONG);
        p_pic_base = (uint32_t*)&(VIC.PIC0.LONG);

        /* Calcurate register suffix number */
        reg_num = vec_num / 32;  // PLSn, PICn (n = reg_num)
        bit_num = vec_num % 32;  // PLSn.PLSm, PICn.PICm (m = bit_num)

    }
    else {
        /* Set each pointer address as PLS8 and PIC8 */
        /* Casting the pointer to a (uint32_t *) is valid because this pointer
           will reference 32 bit I/O register address */
        p_pls_base = (uint32_t*)&(VIC.PLS8.LONG);
        p_pic_base = (uint32_t*)&(VIC.PIC8.LONG);

        /* Calcurate register suffix number. And subtract 8 from reg_num
        because PLS8 and PIC8 are base address in this case */
        reg_num = (vec_num / 32) - 8;   // PLSn, PICn (n = 8 + reg_num)
        bit_num = vec_num % 32;         // PLSn.PLSm, PICn.PICm (m = bit_num)
    }

    p_pls_base += reg_num;  // Specify PLSn register address
    p_pic_base += reg_num;  // Specify PICn register address

    if (*p_pls_base & (1 << bit_num)) {
        *p_pic_base |= (1 << bit_num);  // Set PICn.PICm bit to 1
    }
}

/*******************************************************************************
* Function Name: VIC_EndInterrupt
* Description  : Set HAV0
* Arguments    : value
* Return Value : none
********************************************************************************/
void VIC_EndInterrupt(void)
{
    /* Dummy writing to HVA0 register */
    HVA0_DUMMY_WRITE();
}
